import { HTTP_INTERCEPTORS, HttpEvent, HttpErrorResponse } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { HttpInterceptor, HttpHandler, HttpRequest } from '@angular/common/http';

import { TokenStorageService } from '../_services/token-storage.service';
import { AuthService } from '../_services/auth.service';
import { BehaviorSubject, Observable, throwError } from 'rxjs';
import { catchError, filter, switchMap, take } from 'rxjs/operators';
import {CustomHttpResponse} from "../_models";
import {Router} from "@angular/router";
import {DataSharingService} from "../_services/data-sharing.service";

const TOKEN_HEADER_KEY = 'Authorization';

@Injectable()
export class AuthInterceptor implements HttpInterceptor {
  private isRefreshing = false;
  private refreshTokenSubject: BehaviorSubject<any> = new BehaviorSubject<any>(null);
  constructor(private token: TokenStorageService,
              private authService: AuthService,
              private dataSharingService: DataSharingService,
              private router: Router) { }

  intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    let authReq = req;
    const token = this.token.getToken();
    if (token != null) {
      //authReq = req.clone({ headers: req.headers.set(TOKEN_HEADER_KEY, 'Bearer ' + token) });
      authReq = this.addTokenHeader(req,token);
    }
    return next.handle(authReq).pipe(catchError(error => {
      if (error instanceof HttpErrorResponse && authReq.url.includes('api/refresh_token') && error.status === 401) {
        // When Refresh token is revoked or expired
        this.isRefreshing = false;

        this.token.signOut();
        this.dataSharingService.isUserLoggedIn.next(false);
        this.router.navigate(['/login']);
        return throwError(error);
      }

      if (error instanceof HttpErrorResponse && !authReq.url.includes('auth/login') && error.status === 401) {
        // When Access Token is expired
        return this.handle401Error(authReq, next);
      }

      return throwError(error);
    }));
  }
  private handle401Error(request: HttpRequest<any>, next: HttpHandler) {
    if (!this.isRefreshing) {
      this.isRefreshing = true;
      this.refreshTokenSubject.next(null);
      const token = this.token.getRefreshToken();
      if (token)
        return this.authService.refreshToken(token).pipe(
          switchMap((resp: CustomHttpResponse<any>) => {
            this.isRefreshing = false;
            let token = "";
            if(resp.token) {
              token = resp.token;
            }

            this.token.saveToken(token);
            this.refreshTokenSubject.next(token);

            return next.handle(this.addTokenHeader(request, token));
          }),
          catchError((err) => {
            this.isRefreshing = false;

            this.token.signOut();
            return throwError(err);
          })
        );
    }
    return this.refreshTokenSubject.pipe(
      filter(token => token !== null),
      take(1),
      switchMap((token) => next.handle(this.addTokenHeader(request, token)))
    );
  }
  private addTokenHeader(request: HttpRequest<any>, token: string) {
    /* for Spring Boot back-end */
    // return request.clone({ headers: request.headers.set(TOKEN_HEADER_KEY, 'Bearer ' + token) });
    /* for Node.js Express back-end */
    //console.log(request.url);
    if(request.url.includes('translation/get_list_translation')){
      return request;
    }
    return request.clone({ headers: request.headers.set(TOKEN_HEADER_KEY, 'Bearer ' + token) });
  }
}

export const authInterceptorProviders = [
  { provide: HTTP_INTERCEPTORS, useClass: AuthInterceptor, multi: true }
];
