import { Injectable } from '@angular/core';
import {environment} from "../../environments/environment";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Observable} from "rxjs";
import {
  ArticlesList, BlockSerialList,
  BrandInfoList,
  BrandPrixVente,
  CustomHttpResponse,
  Historique,
  LigneInfoList, LinkedDocuments, ListDepotTNT, ListServiceExpedition,
  Manufacturers, NumeroSerialList,
  StockLocationList, StoreClientCategories, StoreInfoList, StoreSupplierCategories
} from "../_models";
import {Article, ArticleVat, SearchArticle} from "../_models/article";
import {APP_CONSTANTS} from "../_helpers/constants";

@Injectable({
  providedIn: 'root'
})
export class CommonService {

  constructor(private httpClient: HttpClient) { }

  formatBlockedSerialNumbers(data: string): BlockSerialList[] {
    if(data === undefined || data === null || data.length === 0) {
      return [];
    }

    let res: BlockSerialList[] = [];
    const splitSerials: string[] = data.split(',');

    res = splitSerials.map((r: string) => {
      const [ serialNo, documentNo, documentType ] = r.split(APP_CONSTANTS.BLOCK_SERIALS_SEPARATOR);

      return {
        serial_no: serialNo,
        document_no: documentNo,
        document_type: documentType
      };
    });

    return res;
  }

  formatSerialNumbers(data: string[], verified = false): NumeroSerialList[] {
    if(data.length === 0) {
      return [];
    }

    return data.map((row: string) => {
      if(row.includes(APP_CONSTANTS.OCCASION_SEPARATOR)) {
        const [ serial_no, serial_tag, brand_lbl ] = row.split(APP_CONSTANTS.OCCASION_SEPARATOR); // Ex: TEST1~#~OC
        return { serial_no, serial_tag, brand_lbl, verified, disabled: false };
      }

      return { serial_no: row, serial_tag: null, brand_lbl:null, verified, disabled: false };
    });
  }

  searchArticlesListByRef(data: {}): Observable<CustomHttpResponse<SearchArticle>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    let params = data || {};

    params = {
      ...params,
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.get<CustomHttpResponse<SearchArticle>>(environment.apiUrl + '/api/common/search_articles', options);
  }

  checkSerialsDuplication(data: {}) {
    return this.httpClient.post(environment.apiUrl + "/api/common/check_serialnumber_exists", data);
  }

  getStockLocationList(): Observable<CustomHttpResponse<StockLocationList[]>> {
    return this.httpClient.get<CustomHttpResponse<StockLocationList[]>>(environment.apiUrl + "/api/common/getstocklocationlist");
  }

  getAllLigneList(): Observable<CustomHttpResponse<LigneInfoList[]>> {
    return this.httpClient.get<CustomHttpResponse<LigneInfoList[]>>(environment.apiUrl + "/api/common/getLigneList");
  }

  getAllStockList(): Observable<CustomHttpResponse<StockLocationList[]>> {
    return this.httpClient.get<CustomHttpResponse<StockLocationList[]>>(environment.apiUrl + "/api/common/getstocklocationlist");
  }

  getLinkedDocuments(data: { document_type: string, document_no: string }): Observable<CustomHttpResponse<LinkedDocuments>> {
    return this.httpClient.post<CustomHttpResponse<LinkedDocuments>>(environment.apiUrl + "/api/common/getLinkedDocuments", data);
  }

  getArticleDetail(article_id: number, location_id: number): Observable<CustomHttpResponse<Article>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const options = {
      headers: headers,
      params: {
        location_id
      }
    };

    return this.httpClient.get<CustomHttpResponse<Article>>(environment.apiUrl + `/api/common/article_detail/${article_id}`, options);
  }

  getListManufacturers(): Observable<CustomHttpResponse<Manufacturers[]>> {
    return this.httpClient.get<CustomHttpResponse<Manufacturers[]>>(environment.apiUrl + "/api/common/list_manufacturers");
  }

  getListArticles(data?: { brand_id: number, store_id: number, store_type: string }): Observable<CustomHttpResponse<ArticlesList[]>> {
    const payload = {
      brand_id: (data && data.brand_id) || null,
      store_id: (data && data.store_id) || null,
      store_type: (data && data.store_type) || null,
    };

    return this.httpClient.post<CustomHttpResponse<ArticlesList[]>>(environment.apiUrl + "/api/common/getArticleList", payload);
  }

  getListBrands(): Observable<CustomHttpResponse<BrandInfoList[]>> {
    return this.httpClient.get<CustomHttpResponse<BrandInfoList[]>>(environment.apiUrl + "/api/common/getBrandList");
  }

  getListStores(data: { type: string }): Observable<CustomHttpResponse<StoreInfoList[]>> {
    return this.httpClient.post<CustomHttpResponse<StoreInfoList[]>>(environment.apiUrl + "/api/common/getStoresList", data);
  }

  getListDefriement(): Observable<CustomHttpResponse<any[]>> {
    return this.httpClient.get<CustomHttpResponse<any[]>>(environment.apiUrl + "/api/common/getDefriementList");
  }

  getListPays(): Observable<CustomHttpResponse<any[]>> {
    return this.httpClient.get<CustomHttpResponse<any[]>>(environment.apiUrl + "/api/common/getPaysList");
  }

  getListStoreCategories(type: 'client' | 'supplier'): Observable<CustomHttpResponse<StoreSupplierCategories[] | StoreClientCategories[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      cattype: type
    };

    const options = {
      headers: headers,
      params
    };

    return this.httpClient.get<CustomHttpResponse<StoreSupplierCategories[] | StoreClientCategories[]>>(environment.apiUrl + "/api/common/getCategoryList", options);
  }

  getListCodPosVille(data: {}): Observable<CustomHttpResponse<any[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const options = {
      headers: headers,
      params: data
    };

    return this.httpClient.get<CustomHttpResponse<any[]>>(environment.apiUrl + "/api/common/getPOSVille", options);
  }

  getDepotTNTList(): Observable<CustomHttpResponse<ListDepotTNT[]>> {
    return this.httpClient.get<CustomHttpResponse<ListDepotTNT[]>>(environment.apiUrl + "/api/common/getDepotTNTList");
  }

  getServiceExpeditionList(): Observable<CustomHttpResponse<ListServiceExpedition[]>> {
    return this.httpClient.get<CustomHttpResponse<ListServiceExpedition[]>>(environment.apiUrl + "/api/common/getServiceExpeditionList");
  }

  getPrixDeVenteBasedOnBrand(data: any): Observable<CustomHttpResponse<BrandPrixVente>> {
    return this.httpClient.post<CustomHttpResponse<BrandPrixVente>>(environment.apiUrl + "/api/common/getBrandPrixVente", data);
  }

  searchHistorique(data:Historique, page: number, pageSize: number): Observable<CustomHttpResponse<Historique[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };
    return this.httpClient.post<CustomHttpResponse<Historique[]>>(environment.apiUrl + '/api/common/getDocumentActivityList', data, options);
  }

}
