import { Component, OnInit } from '@angular/core';
import {FormBuilder, FormGroup} from "@angular/forms";
import {CustomHttpResponse, AchatCommandeList} from "../../_models";
import {Router} from "@angular/router";
import {MatDialog} from "@angular/material/dialog";
import {DialogService} from "../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {TranslateService} from "@ngx-translate/core";
import {PageEvent} from "@angular/material/paginator";
import {AchatCommandeService} from "../../_services/achat-commande.service";
import * as moment from "moment";
import {CUSTOM_DATE_API_FORMAT} from "../../_helpers/custom-date-formats";
import {APP_CONSTANTS} from "../../_helpers/constants";
import {lastValueFrom} from "rxjs";
import { DashboardService } from 'src/app/_services/dashboard.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-achat-commande',
  templateUrl: './achat-commande.component.html',
  styleUrls: ['./achat-commande.component.scss']
})
export class AchatCommandeComponent implements OnInit {

  searchForm: FormGroup = this.formBuilder.group({
    document_no: [''],
    start_date: [{ value: '', disabled: true }],
    end_date: [{ value: '', disabled: true }],
    status: [''],
  });

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  listAchatCommande: AchatCommandeList[] | undefined;
  displayedColumnsReception: string[] = ['DOCUMENT_DATE', 'DOCUMENT_NO', 'SUPPLIER', 'CLIENT', 'ITEM_REFERENCE', 'STATUS', 'TOTAL_PRICE', 'ACTIONS'];
  rolePermission:any;

  statusList = APP_CONSTANTS.ACHAT_DOCUMENT_STATUS;

  constructor(private formBuilder: FormBuilder,
              public dialog: MatDialog,
              private achatCommandeService: AchatCommandeService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private translateService: TranslateService,
              private dashboardService: DashboardService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    const statusValue = this.dashboardService.dashboardStatusValue;
    this.dashboardService.dashboardStatusValue = '';
    if(statusValue){
      this.searchForm.get('status')?.setValue(parseInt(statusValue));
      this.searchFormSubmitter();
      return;
    }
    this.checkForListCache();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_ACHAT_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        document_no: formCache.document_no,
        start_date: formCache.start_date,
        end_date: formCache.end_date,
        status: formCache.status
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ACHAT_LIST);
      if(dataCache !== undefined) {

        this.listAchatCommande = dataCache.data as AchatCommandeList[];
        this.listAchatCommande = this.getAchatFormatedList(this.listAchatCommande);
        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  getAchatFormatedList(listAchatCommande: AchatCommandeList[]){
    listAchatCommande = listAchatCommande.map((list) =>{
      list.supplier_brands = list.supplier_brands.toString().split(",");
      list.client_brands = list.client_brands.toString().split(",");
      list.item_references = list.item_references.toString().split(",");
      return list;
    });
    return listAchatCommande;
  }

  get startDate() {
    return this.searchForm.get('start_date');
  }

  get endDate() {
    return this.searchForm.get('end_date');
  }

  isDocumentValidated(item: AchatCommandeList): boolean {
    return item.status === APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_VALIDATED;
  }

  isDocumentCanceled(item: AchatCommandeList): boolean {
    return item.status === APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_CANCELLED;
  }

  isAttendueReceptionStatusInvoiced(item: AchatCommandeList): boolean {
    return item.status  === APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_INVOICED;
  }


  documentStatus(item: AchatCommandeList) {
    const statusList = APP_CONSTANTS.ACHAT_DOCUMENT_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_PENDING:
        result = `<span class='badge bg-secondary'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_INPROGRESS:
        result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_VALIDATED:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_CANCELLED:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_INVOICED:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST_FORM, JSON.stringify(formData));
    const start_date = formData.start_date ? moment(formData.start_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";
    const end_date = formData.end_date ? moment(formData.end_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";

    const payload = {
      document_no: formData.document_no,
      start_date: start_date,
      end_date: end_date,
      status: formData.status
    };

    this.currentPage = 1;

    this._getAchatCommandeList(payload, this.currentPage, this.pageSize);
  }

  _getAchatCommandeList(payload: { document_no: string, start_date: string, end_date: string }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<AchatCommandeList[]>) => {
        if(res.status === "success") {
          this.listAchatCommande = res.data;
          this.listAchatCommande = this.getAchatFormatedList(this.listAchatCommande);
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.achatCommandeService.getAchatCommandeList(payload, page, pageSize).subscribe(observer);
  }

  __updateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ACHAT_LIST);
    if(listCache !== undefined) {
      let listAchatCommande: any[] = listCache.data;
      listAchatCommande = (listAchatCommande as AchatCommandeList[]).filter(r => r.document_no != id);

      listCache.data = listAchatCommande;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listAchatCommande.length === 0) {
        this.searchFormSubmitter();
      }
    }
}


  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();

    const start_date = formData.start_date ? moment(formData.start_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";
    const end_date = formData.end_date ? moment(formData.end_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";

    const payload = {
      document_no: formData.document_no,
      start_date: start_date,
      end_date: end_date,
      status: formData.status
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._getAchatCommandeList(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveAchatCommande(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeAchatCommande(document_no: string, item:AchatCommandeList) {
    if((this.isDocumentValidated(item) && !this.rolePermission['1_7']) || (!this.isDocumentValidated(item) && !this.rolePermission['1_4'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    
    const removeConfirmation = await this.confirmRemoveAchatCommande(document_no);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteAchatCommande(document_no);
    }
  }

  async _deleteAchatCommande(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          const split_doc_no = res.data.splitted_document_no;
          if(split_doc_no){
            await this.dialogService.displayAlert({
              title: 'KSUCCESS',
              message: this.translateService.instant('KACHAT_CANCELED_SEPARATED_SUCCESS', { DOCUMENT_NO: split_doc_no })
            })
          }else { 
            await this.dialogService.displayAlert({
              title: 'KSUCCESS',
              message: this.translateService.instant('KACHAT_CANCELED_SUCCESS')
            })
          }
          this.searchFormSubmitter();
          this.__updateListCache(document_no);
          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.achatCommandeService.splittingAchatCommande(document_no).subscribe(observer);
  }

}
