import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { PageEvent } from '@angular/material/paginator';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { ArticleCategoryInfo, CustomHttpResponse } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { lastValueFrom } from 'rxjs';
import { ArticleCategoryService } from 'src/app/_services/article-category.service';

@Component({
  selector: 'app-article-category-list',
  templateUrl: './article-category-list.component.html',
  styleUrls: ['./article-category-list.component.scss']
})
export class ArticleCategoryListComponent implements OnInit {


  categoryList:ArticleCategoryInfo[]=[];
  Submitted: boolean = false;
  loading:boolean = false;
  searchForm:FormGroup = this.formBuilder.group({
    keyword: new FormControl(''),
  });
  isLoading = false;
  displayedColumnsCategory: string[] = ['ACLBL' ,'ACSTAT' ,'ACTIONS'];
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  rolePermission:any;

  constructor(private formBuilder: FormBuilder,
              public dialog: MatDialog,
              private loadingService: NgxSpinnerService,
              private translateService:TranslateService,
              private dialogService: DialogService,
              private articleCategoryService:ArticleCategoryService,
              private dataSharingService:DataSharingService) {}

  ngOnInit(): void {
    this.checkForListCache();
    this.loadDependencies();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST_FORM);
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        keyword: formCache.keyword
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST);
      if(dataCache !== undefined) {

        this.categoryList = dataCache.data as ArticleCategoryInfo[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
    this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    await this.loadingService.hide();
  }

  get fr() {
    return this.searchForm.controls;
  }

  documentStatus(item: ArticleCategoryInfo) {
    const statusList = APP_CONSTANTS.ARTICLE_CATEGORY_STATUS;

    const statusData = statusList.find((r) => r.value == item.category_status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.category_status) {
      case APP_CONSTANTS.ARTICLE_CATEGORY_ACTIVE:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ARTICLE_CATEGORY_IN_ACTIVE:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }
  
  searchFormSubmitter(){
    this.Submitted = true;
    if(this.searchForm.invalid){
      return;
    }
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST_FORM, JSON.stringify(formData));
    const payload = {
      keyword: formData.keyword
    };
    this.currentPage = 1;
    this._searchArticleCategoryInfo(payload, this.currentPage, this.pageSize);
  }

  _searchArticleCategoryInfo(data: { keyword: string }, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<ArticleCategoryInfo[]>) => {
        if(res.status === "success") {
          this.categoryList = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST);
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.articleCategoryService.searchCategoryInfo(data, page, pageSize).subscribe( observer);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST);
    if(listCache !== undefined) {
      let listContacts: any[] = listCache.data;
      listContacts = (listContacts as ArticleCategoryInfo[]).filter(r => r.category_id != id);

      listCache.data = listContacts;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listContacts.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      keyword: formData.keyword
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._searchArticleCategoryInfo(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveArticleCategory(cat_label: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: cat_label })}</h3>`;
    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeCategory(cat_id: number, cat_label:string) {
    if(!this.rolePermission['9_104']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveArticleCategory(cat_label);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteCategoryInfo(cat_id);
    }

  }

  async _deleteCategoryInfo(cat_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(cat_id);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.articleCategoryService.deleteCategoryById(cat_id).subscribe(observer);
  }
}
