import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import {firstValueFrom} from "rxjs";
import { ArticleCategoryInfo, CustomHttpResponse } from 'src/app/_models';
import { ArticleCategoryService } from 'src/app/_services/article-category.service';

@Component({
  selector: 'app-edit-article-category',
  templateUrl: './edit-article-category.component.html',
  styleUrls: ['./edit-article-category.component.scss']
})
export class EditArticleCategoryComponent implements OnInit {


  submitted: boolean = false;
  modifyCategoryForm: FormGroup = this.formBuilder.group({
    category_status: new FormControl('', [Validators.required]),
    category_label: new FormControl('', [Validators.required, Validators.maxLength(APP_CONSTANTS.ARTICLE_CATEGORY_LABEL_MAX_LENGTH), Validators.minLength(APP_CONSTANTS.ARTICLE_CATEGORY_LABEL_MIN_LENGTH)]),
  });

  category_id: number | undefined;
  article_Category_Detail: ArticleCategoryInfo | undefined;

  statusList = APP_CONSTANTS.ARTICLE_CATEGORY_STATUS;

  readOnly: boolean = false;
  pageTitle: string = 'KARTICLE_CATEGORY_MODIFY';

  constructor(private articleCategoryService: ArticleCategoryService,
              private formBuilder: FormBuilder,
              private router: Router,
              private route: ActivatedRoute,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private dataSharingService:DataSharingService) {
  }

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.category_id = +params['category_id'];
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];

        if(this.readOnly) {
          this._changeToDetailMode();
        }
      }
    })
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this._getCategoryInfoDetail(this.category_id);
    this._prefillFormData();

    await this.loadingService.hide();
  }

  _changeToDetailMode() {
    this.pageTitle = 'KARTICLE_CATEGORY_DETAIL';
    this.modifyCategoryForm.get('category_status')?.disable({ onlySelf: true });
    this.modifyCategoryForm.get('category_label')?.disable({ onlySelf: true });
  }

  get fr() {
    return this.modifyCategoryForm.controls;
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST);
    if(listCache !== undefined) {
      let listCategoryInfo = listCache.data as ArticleCategoryInfo[];
      listCategoryInfo = listCategoryInfo.map((row) => {
        if(row.category_id == this.category_id) {
          row.category_id = this.category_id;
          row.category_label = payload.category_label;
          row.category_status = payload.category_status;
        }
        return row;
      });

      listCache.data = listCategoryInfo;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST, JSON.stringify(listCache));
    }
  }

  _getCategoryInfoDetail(cat_id: number | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(cat_id === undefined || cat_id === null) {
        reject();
        return;
      }

      const observer = {
        next: async (res: CustomHttpResponse<ArticleCategoryInfo>) => {
          if(res.status === "success") {
            this.article_Category_Detail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            // If the Category Info not available
            await this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });

            this.router.navigate(['/article-category/list']);

            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          reject(err);
        }
      };

      this.articleCategoryService.getCategoryDetailById(cat_id).subscribe(observer);
    })
  }

  _prefillFormData() {
    this.modifyCategoryForm.patchValue({
      category_status: this.article_Category_Detail?.category_status,
      category_label: this.article_Category_Detail?.category_label,
    })
  }

  modifyCategoryFormSubmitter() {
    if(this.readOnly) {
      return;
    }

    this.submitted = true;
    if(this.modifyCategoryForm.invalid) {
      return;
    }

    const formData = this.modifyCategoryForm.getRawValue();
    const payload = {
      category_label: formData.category_label,
      category_status: formData.category_status
    };

    this._modifyCategoryInfo(payload);
  }

  async _modifyCategoryInfo(payload: { category_label: string, category_status: number }) {
    if(this.category_id === undefined || this.category_id === null) {
      return;
    }

    const observer = {
      next: async (res: CustomHttpResponse<ArticleCategoryInfo>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/article-category/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.articleCategoryService.modifyCategoryById(this.category_id, payload).subscribe(observer);
  }

}
