import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import {PageEvent} from '@angular/material/paginator';
import { MatDialog } from '@angular/material/dialog';
import {NgxSpinnerService} from "ngx-spinner";
import {MatCheckboxChange} from "@angular/material/checkbox";
import {Article, ArticleCategories} from "../../../_models/article";
import { TranslateService } from '@ngx-translate/core';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { DialogService } from 'src/app/_services/dailog.service';
import {CustomHttpResponse, Manufacturers, StockLocationList} from "../../../_models";
import {ArticleService} from "../../../_services/article.service";
import {CommonService} from "../../../_services/common.service";
import {lastValueFrom} from "rxjs";
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-article-list',
  templateUrl: './article-list.component.html',
  styleUrls: ['./article-list.component.scss']
})
export class ArticleListComponent implements OnInit {

  @Input("displayType") displayType: "default" | "fusionner-popup" | 'bon-search-popup' = "default";
  @Input("mainArticle") currentSelectionArticle: Article | undefined;
  @Input("chosenStockLocation") chosenStockLocation: string | undefined;
  @Input("searchKeyword") searchKeyword: string | undefined;

  private _chosenFushionerArticle: any = undefined;
  get chosenFushionerArticle(): any { return this._chosenFushionerArticle }
  @Input() set chosenFushionerArticle(chosenFushionerArticle: any) {
    this.chosenFushionerArticleChange.emit(this._chosenFushionerArticle = chosenFushionerArticle);
  }
  @Output() private chosenFushionerArticleChange: EventEmitter<any> = new EventEmitter<any>();

  private _chosenSearchArticleChange: any = undefined;
  get chosenSearchArticle(): any { return this._chosenSearchArticleChange }
  @Input() set chosenSearchArticle(chosenSearchArticle: any) {
    this.chosenSearchArticleChange.emit(this._chosenSearchArticleChange = chosenSearchArticle);
  }
  @Output() private chosenSearchArticleChange: EventEmitter<any> = new EventEmitter<any>();

  submitted: boolean = false;
  searchForm: FormGroup = this.formBuilder.group({
    reference: [''],
    libelle: [''],
    serial:[''],
    category: [''],
    fabriquant: [''],
    location_id: [''],
    in_stock: [false],
    eol_only: [false],
  });

  categoriesList: ArticleCategories[] = [];
  listFabriquant: Manufacturers[] = [];
  listStockLocation: StockLocationList[] = [];
  addArticleButton: boolean = true;
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  listArticles: Article[] | undefined;

  displayedColumnsArticle: string[] = ['ARTREF', 'ARTLBL', 'ARTACATID', 'MNFLBL', 'AMATLBL', 'QTE_STOCK','actions'];

  rolePermission:any;

  constructor(private articleService: ArticleService,
              private commonService: CommonService,
              private formBuilder: FormBuilder,
              private router:Router,
              public dialog: MatDialog,
              private loadingService: NgxSpinnerService,
              private translateService:TranslateService,
              private dialogService: DialogService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    if(this.displayType === 'fusionner-popup' || this.displayType === 'bon-search-popup') {
      this.displayedColumnsArticle = [ 'ARTICLE_CHOOSE', 'ARTREF', 'ARTLBL', 'ARTACATID', 'MNFLBL', 'AMATLBL', 'QTE_STOCK' ];
    }

    if(this.displayType === 'bon-search-popup') {
      this.searchForm.patchValue({
        reference: this.searchKeyword,
        location_id: this.chosenStockLocation,
      });
      this.addArticleButton = false;
      this.searchForm.get('location_id')?.disable({ onlySelf: true });
    }

    this.checkForListCache(this.displayType === 'fusionner-popup' || this.displayType === 'bon-search-popup');
    this.loadDependencies();
  }

  checkForListCache(forceReset: boolean = false) {
    if(forceReset) {
      this.searchFormSubmitter();
      return;
    }

    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_ARTICLE_LIST_FORM);
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        reference: formCache.reference,
        libelle: formCache.libelle,
        serial: formCache.serial,
        category: formCache.category,
        fabriquant: formCache.fabriquant,
        location_id: formCache.location_id,
        in_stock: formCache.in_stock,
        eol_only: formCache.eol_only,
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ARTICLE_LIST);
      if(dataCache !== undefined) {

        this.listArticles = dataCache.data as Article[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();

    const userPermissions = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(userPermissions != null) {
      this.rolePermission = JSON.parse(userPermissions);
    }

    await this.getListCategory();
    await this.getManufacturerList();
    await this.getStorageList();

    await this.loadingService.hide();
  }

  get fr() {
    return this.searchForm.controls;
  }

  getListCategory(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticleCategories[]>) => {
          if(res.status === 'success') {
            this.categoriesList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.articleService.getArticleCategories().subscribe(observer);
    });
  }

  getManufacturerList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<Manufacturers[]>) => {
          if(res.status === 'success') {
            this.listFabriquant = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListManufacturers().subscribe(observer);
    });
  }

  getStorageList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.listStockLocation = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getStockLocationList().subscribe( observer);
    })
  }

  searchFormSubmitter() {
    this.submitted = true;
    if(this.searchForm.invalid) {
      return;
    }

    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST_FORM, JSON.stringify(formData));

    const payload = {
      reference: formData.reference,
      libelle: formData.libelle,
      serial: formData.serial,
      category: formData.category,
      fabriquant: formData.fabriquant,
      location_id: formData.location_id,
      in_stock: formData.in_stock,
      only_eol: formData.eol_only
    };

    this.currentPage = 1;
    this._searchArticles(payload, this.currentPage, this.pageSize);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ARTICLE_LIST);
    if(listCache !== undefined) {
      let listArticle: any[] = listCache.data;
      listArticle = (listArticle as Article[]).filter(r => r.id != id);

      listCache.data = listArticle;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listArticle.length === 0) {
        this.searchFormSubmitter();
      }
    }
}


  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      reference: formData.reference,
      libelle: formData.libelle,
      serial: formData.serial,
      category: formData.category,
      fabriquant: formData.fabriquant,
      location_id: formData.location_id,
      in_stock: formData.in_stock,
      only_eol: formData.eol_only
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._searchArticles(payload, this.currentPage, this.pageSize);
  }

  _searchArticles(data: any, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<Article[]>) => {
        if(res.status === "success") {
          this.listArticles = res.data;

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.articleService.searchArticleList(data, page, pageSize).subscribe(observer);
  }

  // Methods which are needed for Fusionner Articles popup
  onFusionnerSelectionChange(e: MatCheckboxChange, row: any) {
    this.chosenFushionerArticle = undefined;
    if(e.checked) {
      this.chosenFushionerArticle = row;
    }
  }

  // Methods which are needed for Search Articles popup
  onSearchArticleSelectionChange(e: MatCheckboxChange, row: any) {
    this.chosenSearchArticle = undefined;
    if(e.checked) {
      this.chosenSearchArticle = row;
    }
  }

  onTableRowClicked(row: any) {
    if(this.displayType === 'fusionner-popup') {
      if(this.currentSelectionArticle && this.currentSelectionArticle?.id === row.id) {
        return;
      }

      this.updateFusionnerOnRowClick(row);
      return;
    }
  }

  updateFusionnerOnRowClick(row: any) {
    if(this.chosenFushionerArticle === row) {
      this.chosenFushionerArticle = undefined;
    } else {
      this.chosenFushionerArticle = row;
    }
  }

  confirmRemoveArticle(article_label: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: article_label })}</h3>`;
    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeArticle(article_id: number, article_label: string) {
    if(!this.rolePermission['4_4']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })

      return;
    }

    const removeConfirmation = await this.confirmRemoveArticle(article_label);
    if(!removeConfirmation) {
      return;
    }

    this.loadingService.show();
    this._deleteArticle(article_id);
  }

  async _deleteArticle(article_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.__updateListCache(article_id);
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.articleService.removeArticle(article_id).subscribe(observer);
  }

}
