import { Component, OnInit } from '@angular/core';
import { AbstractControl, FormArray, FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {ArticleCategories, ArticleMaterialTypes, ArticleVat} from "../../../_models/article";
import {BrandInfoList, CustomHttpResponse, Manufacturers} from "../../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import * as moment from "moment";
import {CUSTOM_DATE_API_FORMAT} from "../../../_helpers/custom-date-formats";
import {ArticleService} from "../../../_services/article.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {CommonService} from "../../../_services/common.service";
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-create-article',
  templateUrl: './create-article.component.html',
  styleUrls: ['./create-article.component.scss']
})
export class CreateArticleComponent implements OnInit {

  submitted: boolean = false;
  createArticleForm: FormGroup = this.formBuilder.group({
    ARTCATID: ['', [Validators.required]],
    ARTMATID: ['', [Validators.required]],
    ARTLBL: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.ARTICLE_LABEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_LABEL_MAX_LENGTH)]],
    ARTDESC: ['', [Validators.minLength(APP_CONSTANTS.ARTICLE_DESC_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_DESC_MAX_LENGTH)]],
    ARTDTEDEBDISP: [{ value: '', disabled: true }],
    ARTDTEOBSL: [{ value: '', disabled: true }],
    ARTDERNIERDTACHAT: [{ value: '', disabled: true }],
    ARTTRACE: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.ARTICLE_TRACE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_TRACE_MAX_LENGTH)]],
    ARTDURGUART: [''],
    ARTPOID: ['', [Validators.minLength(APP_CONSTANTS.ARTICLE_POID_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_POID_MAX_LENGTH)]],
    ARTDIM1: [''],
    ARTDIM2: [''],
    ARTDIM3: [''],
    ARTMNFID: ['', [Validators.required]],
    ARTCONSREF: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.ARTICLE_CONSREF_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_CONSREF_MAX_LENGTH)]],
    ARTREFFAC: ['', [Validators.maxLength(APP_CONSTANTS.ARTICLE_REFERENCE_FACTURE_MAX_LENGTH)]],
    ARTREF: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.ARTICLE_REF_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_REF_MAX_LENGTH)]],
    ARTDELLIV: [''],
    ARTTRESHLD: [''],
    ARTCMDQTE: [''],
    ARTASSUR: [''],
    ARTPUBPRIXHT: [''],
    ARTPURPRIXHT: [''],
    ARTPRIXVENTE: ['' , [Validators.required, Validators.pattern(APP_CONSTANTS.MIN_NUM_PATTERN),Validators.minLength(APP_CONSTANTS.ARTPRIXVENTE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTPRIXVENTE_MAX_LENGTH)]],
    ARTSUPPLIER: [''],
    ARTAPR1RECP: ['', [Validators.minLength(APP_CONSTANTS.ARTICLE_APR1RECP_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_APR1RECP_MAX_LENGTH)]],
    ARTVATID: [''],
    ARTIMG: [''],
    ARTBRAND: new FormArray([])
  });

  rolePermission: any;
  PrixPub_PrixAchat_Permission: boolean = false;

  categoriesList: ArticleCategories[] = [];
  materialTypesList: ArticleMaterialTypes[] = [];
  manufacturerList: Manufacturers[] = [];
  brandlist: BrandInfoList[] = [];
  supplierBrandList: BrandInfoList[] = [];
  vatList: ArticleVat[] = [];

  uploadFile: any;

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  constructor(private articleService: ArticleService,
              private commonService: CommonService,
              private formBuilder: FormBuilder,
              private router: Router,
              private translateService: TranslateService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();

    const userPermissions = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(userPermissions != null) {
      this.rolePermission = JSON.parse(userPermissions);
      this.PrixPub_PrixAchat_Permission = this.rolePermission['11_1'];
    }

    await this.getListCategory();
    await this.getMaterialTypeList();
    await this.getVatList();
    await this.getManufacturerList();
    await this.getBrandList();
    await this.loadingService.hide();
  }

  get fr() {
    return this.createArticleForm.controls;
  }

  get brandField(){
    return (this.createArticleForm.get('ARTBRAND') as FormArray)
  }

  selectBrand(index: number){
    const selectedData = this.brandField.value[index].brand_id;
    this.brandField.value.forEach((brand:any) => {
      if(brand.brand_id){
        if(selectedData != brand.brand_id){
          this.brandlist.map((brandDetail: BrandInfoList) =>{
            if(brandDetail.BRID == brand.brand_id){
              brandDetail.disabled = true;
            }
          })
        }else{
          this.brandlist.map((brandDetail: BrandInfoList) =>{
            if(brandDetail.BRID == brand.brand_id){
              brandDetail.disabled = false;
            }
          })
        }
      }
    })
  }

  addBrand(){
    const newBrand = this.formBuilder.group({
      brand_id: ["", [Validators.required]],
      prix_de_vente: ["", [Validators.required, Validators.pattern(APP_CONSTANTS.MIN_NUM_PATTERN),Validators.minLength(APP_CONSTANTS.ARTPRIXVENTE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTPRIXVENTE_MAX_LENGTH)]]
    });

    this.brandField.push(newBrand);
  }

  deleteBrand(index:any){
    this.brandlist.forEach((brand) => {
      if(brand.BRID == this.brandField.value[index].brand_id){
        brand.disabled = false
      }
    });
    this.brandField.removeAt(index);
  }

  getListCategory(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticleCategories[]>) => {
          if(res.status === 'success') {
            this.categoriesList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.articleService.getArticleCategories().subscribe(observer);
    });
  }

  getMaterialTypeList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticleMaterialTypes[]>) => {
          if(res.status === 'success') {
            this.materialTypesList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.articleService.getArticleMaterialTypes().subscribe(observer);
    });
  }

  getManufacturerList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<Manufacturers[]>) => {
          if(res.status === 'success') {
            this.manufacturerList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListManufacturers().subscribe(observer);
    });
  }

  getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            const brandData = res.data;

            brandData.forEach((value: BrandInfoList) =>{
              value.disabled = false;
            })

            this.brandlist = brandData;

            // Filter only Supplier Brands
            this.supplierBrandList = this.brandlist.filter((row) => row.BRTYPE === 'supplier');

            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  getVatList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticleVat[]>) => {
          if(res.status === 'success') {
            this.vatList = res.data;

            const findTwenty = this.vatList.find((row) => row.VATVAL == 20);
            if(findTwenty) {
              this.createArticleForm.get('ARTVATID')?.setValue(findTwenty.VATID);
            }

            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.articleService.getArticleVatList().subscribe(observer);
    });
  }

  uploadLogo(fileInput: any) {
    if (fileInput.target.files && fileInput.target.files[0]) {
      this.uploadFile = fileInput.target.files[0];
      this.createArticleForm.get('ARTIMG')?.setValue(this.uploadFile.name, { onlySelf: true });
      this.createArticleForm.patchValue({
        ARTIMG: this.uploadFile.name
      });
      this.createArticleForm.updateValueAndValidity();
    } else {
      this.createArticleForm.get('ARTIMG')?.setValue(this.translateService.instant('KARTICLE_CHOOSE_IMAGE'), { onlySelf: true });
      this.uploadFile = undefined;
    }
  }

  createArticleFormSubmitter() {
    this.submitted = true;

    if(this.createArticleForm.invalid){
      return;
    }

    this.createArticle();
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST_FORM);
  }

  async createArticle() {
    const formData = this.createArticleForm.getRawValue();
    const payload: any = new FormData();
    payload.append("ARTREF", formData.ARTREF);
    payload.append("ARTLBL", formData.ARTLBL);
    payload.append("ARTDESC", formData.ARTDESC);
    payload.append("ARTCATID", formData.ARTCATID);
    payload.append("ARTMATID", formData.ARTMATID);
    payload.append("ARTDTEDEBDISP", formData.ARTDTEDEBDISP ? moment(formData.ARTDTEDEBDISP).format(CUSTOM_DATE_API_FORMAT) : "");
    payload.append("ARTDTEOBSL", formData.ARTDTEOBSL ? moment(formData.ARTDTEOBSL).format(CUSTOM_DATE_API_FORMAT) : "");
    payload.append("ARTDERNIERDTACHAT", formData.ARTDERNIERDTACHAT ? moment(formData.ARTDERNIERDTACHAT).format(CUSTOM_DATE_API_FORMAT) : "");
    payload.append("ARTTRACE", formData.ARTTRACE);
    payload.append("ARTDURGUART", formData.ARTDURGUART);
    payload.append("ARTPOID", formData.ARTPOID);
    payload.append("ARTDIM1", formData.ARTDIM1);
    payload.append("ARTDIM2", formData.ARTDIM2);
    payload.append("ARTDIM3", formData.ARTDIM3);
    payload.append("ARTMNFID", formData.ARTMNFID);
    payload.append("ARTCONSREF", formData.ARTCONSREF);
    payload.append("ARTREFFAC", formData.ARTREFFAC);
    payload.append("ARTTRESHLD", formData.ARTTRESHLD);
    payload.append("ARTCMDQTE", formData.ARTCMDQTE);
    payload.append("ARTSUPPLIER", formData.ARTSUPPLIER);
    payload.append("ARTASSUR", formData.ARTASSUR);
    payload.append("ARTPUBPRIXHT", formData.ARTPUBPRIXHT);
    payload.append("ARTPURPRIXHT", formData.ARTPURPRIXHT);
    payload.append("ARTVATID", formData.ARTVATID);
    payload.append("ARTIMG", this.uploadFile);
    payload.append("ARTAPR1RECP", formData.ARTAPR1RECP);
    payload.append("ARTDELLIV", formData.ARTDELLIV);
    payload.append("ARTPRIXVENTE", formData.ARTPRIXVENTE);
    payload.append("ARTBRAND", JSON.stringify(formData.ARTBRAND));
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['article/article-list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.articleService.createArticle(payload).subscribe(observer);
  }

}
