import { Component, OnInit } from '@angular/core';
import {FormBuilder, FormGroup} from "@angular/forms";
import {CustomHttpResponse} from "../../../_models";
import {MatDialog} from "@angular/material/dialog";
import {NgxSpinnerService} from "ngx-spinner";
import {TranslateService} from "@ngx-translate/core";
import {PageEvent} from "@angular/material/paginator";
import * as moment from "moment";
import {CUSTOM_DATE_API_FORMAT} from "../../../_helpers/custom-date-formats";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import { DashboardService } from 'src/app/_services/dashboard.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { AttendueDeReception } from 'src/app/_models/attendue-de-reception';
import { AttendueDeReceptionService } from 'src/app/_services/attendue-de-reception.service';

@Component({
  selector: 'app-attendue-de-reception-list',
  templateUrl: './attendue-de-reception-list.component.html',
  styleUrls: ['./attendue-de-reception-list.component.scss']
})
export class AttendueDeReceptionListComponent implements OnInit {


  searchForm: FormGroup = this.formBuilder.group({
    document_no: [''],
    start_date: [{ value: '', disabled: true }],
    end_date: [{ value: '', disabled: true }],
    status: [APP_CONSTANTS.ATTENDUE_STATUS_PENDING],
  });

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  listAttendueDeReception: AttendueDeReception[] | undefined;
  displayedColumnsAttendue: string[] = ['DOCUMENT_DATE', 'DOCUMENT_NO', 'SUPPLIER_BRANDS', 'CLIENT_BRAND', 'ITEM_REFERENCES', 'STATUS', 'ACTIONS'];
  rolePermission:any;

  statusList = APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS;

  constructor(private formBuilder: FormBuilder,
              public dialog: MatDialog,
              private attendueDeReceptionService: AttendueDeReceptionService,
              private loadingService: NgxSpinnerService,
              private translateService: TranslateService,
              private dashboardService: DashboardService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.checkForListCache();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST_FORM);
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        document_no: formCache.document_no,
        start_date: formCache.start_date,
        end_date: formCache.end_date,
        status: formCache.status
      });
      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST);
      if(dataCache !== undefined) {

        this.listAttendueDeReception = dataCache.data as AttendueDeReception[];
        this.listAttendueDeReception = this.getAttendueFormatedList(this.listAttendueDeReception);

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  getAttendueFormatedList(listAttendueDeReception: AttendueDeReception[]){
    listAttendueDeReception = listAttendueDeReception.map((list) =>{
      list.supplier_brands = list.supplier_brands.toString().split(",");
      list.item_references = list.item_references.toString().split(",");
      list.client_brands = list.client_brands.toString().split(",");
      return list;
    });
    return listAttendueDeReception;
  }

  get startDate() {
    return this.searchForm.get('start_date');
  }

  get endDate() {
    return this.searchForm.get('end_date');
  }

  isDocumentValidated(item: AttendueDeReception): boolean {
    return item.status === APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_VALIDATED;
  }

  documentStatus(item: AttendueDeReception) {
    const statusList = APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.ATTENDUE_STATUS_PENDING:
        result = `<span class='badge bg-secondary'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_INPROGRESS:
        result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_VALIDATED:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_CANCELLED:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_INVOICED:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST_FORM, JSON.stringify(formData));
    const start_date = formData.start_date ? moment(formData.start_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";
    const end_date = formData.end_date ? moment(formData.end_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";

    const payload = {
      document_no: formData.document_no,
      start_date: start_date,
      end_date: end_date,
      status: formData.status
    };

    this.currentPage = 1;

    this._getAttendueDeReceptionList(payload, this.currentPage, this.pageSize);
  }

  _getAttendueDeReceptionList(payload: { document_no: string, start_date: string, end_date: string }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<AttendueDeReception[]>) => {
        if(res.status === "success") {
          this.listAttendueDeReception = res.data;
          this.listAttendueDeReception = this.getAttendueFormatedList(this.listAttendueDeReception);
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.attendueDeReceptionService.getAttendueDeReceptionList(payload, page, pageSize).subscribe(observer);
  }

  __updateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST);
    if(listCache !== undefined) {
      let listAttendueDeReception: any[] = listCache.data;
      listAttendueDeReception = (listAttendueDeReception as AttendueDeReception[]).filter(r => r.document_no != id);

      listCache.data = listAttendueDeReception;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listAttendueDeReception.length === 0) {
        this.searchFormSubmitter();
      }
    }
}


  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();

    const start_date = formData.start_date ? moment(formData.start_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";
    const end_date = formData.end_date ? moment(formData.end_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";

    const payload = {
      document_no: formData.document_no,
      start_date: start_date,
      end_date: end_date,
      status: formData.status
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._getAttendueDeReceptionList(payload, this.currentPage, this.pageSize);
  }

}
