import {Component, ElementRef, OnInit, ViewChild} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {
  BrandInfoList, CustomHttpResponse,
  LigneInfoList,
  StoreInfoList
} from "../../../_models";
import {Article} from "../../../_models/article";
import {MatDialog} from "@angular/material/dialog";
import {CommonService} from "../../../_services/common.service";
import {ActivatedRoute, Router} from "@angular/router";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import {TranslateService} from "@ngx-translate/core";
import * as moment from "moment";
import {
  CUSTOM_DATE_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import {firstValueFrom, lastValueFrom} from "rxjs";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import { HistoriqueComponent } from 'src/app/_components/historique/historique.component';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { AttendueDeReceptionService } from 'src/app/_services/attendue-de-reception.service';
import { AttendueArticle, AttendueDeReceptionList, AttendueDetail } from 'src/app/_models/attendue-de-reception';
@Component({
  selector: 'app-edit-attendue-de-reception',
  templateUrl: './edit-attendue-de-reception.component.html',
  styleUrls: ['./edit-attendue-de-reception.component.scss']
})
export class EditAttendueDeReceptionComponent implements OnInit {

  @ViewChild('prixUnitInput') unitPrixInputEl: ElementRef | undefined;
  @ViewChild('quantiteInput') quantiteInputEl: ElementRef | undefined;
  @ViewChild('rechercheRefInput') rechercheRefInputEl: ElementRef | undefined;

  bonDocumentForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],

    LIGNE_INFO: [''], // Ligne ID
    LIGNE_CUSTOM_LABEL: [''], // Ligne Label

    SEARCH_REF: [""]
  })

  bonAttendueCommandeItemForm: FormGroup = this.formBuilder.group({
    ITEM_SUPPLIER_BRAND: ["", Validators.required],
    ITEM_CLIENT_BRAND: [""],
    ITEM_CLIENT_STORE: [""],
    ARTICLE_ID: ["", Validators.required],
    REFERENCE: [{ value: "", disabled: true }, Validators.required],
    DESCRIPTION: [{ value: "", disabled: true }, Validators.required],
    QTE: ["0", [Validators.required, Validators.min(1)]],
    PRIX_UNIT: ['', [Validators.required, ValidateGreaterThan]],
    PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
    MATERIAL_ACTIVABLE_STATUS: [null],
    ITEM_STATUS: [APP_CONSTANTS.ATTENDUE_STATUS_PENDING, Validators.required],
  });

  newArticleSelection: Article | undefined;

  bonDocumentNum: string | undefined = "*********";
  bonAttendueDeReception: AttendueDetail | undefined;
  bonAttendueArticleItems: AttendueArticle[] = [];

  totalArticles: number = 0;
  totalQte: number = 0;
  totalPT: number = 0;

  ligneList: LigneInfoList[] = [];
  clientBrandList: BrandInfoList[] = [];
  supplierBrandList: BrandInfoList[] = [];
  storeList: StoreInfoList[] = [];

  fullKeywordMatch: boolean = false;
  isKeywordSerial: boolean = false;
  searchRefArticlesList: Article[] | undefined = [];

  displayedColumns: string[] = [ 'ITEM_BRAND', 'REFERENCE', 'DESC', 'QTE', 'QTE_RECEIVED', 'ITEM_CUSTOMER', 'ITEM_STATUS', 'ACTIVE_STATUS'];

  customLigneInput: boolean = false;

  // Used to decide whether the values can be editable or not
  readOnly: boolean = false;
  pageTitle: string = "KATTENDUE_WORK_ON";

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  rolePermission:any;

  constructor(public dialog: MatDialog,
              private commonService: CommonService,
              private attendueDeReceptionService: AttendueDeReceptionService,
              private router: Router,
              private route: ActivatedRoute,
              private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    this.loadDependencies();

    this.bonDocumentForm.get("LIGNE_INFO")?.valueChanges.subscribe(
      (res) => {
        if(res === 'custom-ligne') {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue("");
          this.customLigneInput = true;
        } else {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(res.LGILBL);
          this.customLigneInput = false;
        }
      }
    );

    this.bonDocumentForm.get("SEARCH_REF")?.valueChanges.subscribe(
      (value) => {
        if(value === "" || value === null) {
          this.searchRefArticlesList = undefined;
        }
      }
    );
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.bonDocumentForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)
    });

    firstValueFrom(this.route.data).then(
      (data: any) => {
        if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
          this.readOnly = data.readOnly;
          this.__changeToDetailMode();
        }
      }
    );
    this._getRolePermission();
    await this._getLigneList();
    await this._getBrandList();

    firstValueFrom(this.route.params).then(
      (params) => {
        if(params.hasOwnProperty('attendue_no') && params["attendue_no"] !== null && params["attendue_no"] !== "") {
          this.bonDocumentNum = params["attendue_no"];

          this._getAttendueDetail(this.bonDocumentNum);
        } else {
          // If the attendue document not available
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'Invalid document'
          }).then(
            () => {
              this.router.navigate(['/attendue-de-reception/list']);
            }
          );
        }
      }
    )

    await this.loadingService.hide();
  }

  get material_activable_status() {
    return this.newArticleSelection && this.newArticleSelection?.material_activable_status;
  }

  _getRolePermission(){
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
  }

  __changeToDetailMode() {
    this.pageTitle = "KATTENDUE_WORK_ON";

    this.bonDocumentForm.get('LIGNE_INFO')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('LIGNE_CUSTOM_LABEL')?.disable({ onlySelf: true });
  }

  AttendueItemStatus(status: number): string {
    const statusList = APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS;

    const statusData = statusList.find((r) => r.value == status);
    let result = "";
    switch (statusData?.value) {
      case APP_CONSTANTS.ATTENDUE_STATUS_PENDING:
        result = `<span class='badge bg-secondary'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_INPROGRESS:
        result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_VALIDATED:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_CANCELLED:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  async onCloseDocument() {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    this.returnBack();
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST);
    if(listCache !== undefined) {
      let listAttendue = listCache.data as AttendueDeReceptionList[];
      listAttendue = listAttendue.map((row) => {
        if(row.document_no == payload.document_no) {
          row.achatcommande_id = payload.achatcommande_id;
          row.created_at = payload.created_at;
          row.created_userid = payload.created_userid;
          row.created_username = payload.created_username;
          row.dated = payload.dated;
          row.document_no = payload.document_no;
          row.ligne_id = payload.ligne_id;
          row.ligne_lbl = payload.ligne_lbl;
          row.modified_at = payload.modified_at;
          row.modified_userid = payload.modified_userid;
          row.modified_username = payload.modified_username;
          row.reception_document_no = payload.reception_document_no;
          row.reception_id = payload.reception_id;
          row.status = payload.status;
          row.store_id = payload.customer?.store_id;
          row.store_name = payload.customer?.store_nom;
          row.store_type = payload.customer?.store_type;
        }
        return row;
      });

      listCache.data = listAttendue;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST, JSON.stringify(listCache));
    }
  }

  async validateAttendueDocument() {
    if(!this.isAttendueModifiable) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: this.translateService.instant('KATTENDUE_NO_ARTICLE_ADDED')
      })

      return;
    }
    if(!this.rolePermission['1_9']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }

    const type = APP_CONSTANTS.SELECTION_ACHAT_TO_BR;
    this.dataSharingService.selectionItems.next({
      type: type,
      items: this.bonAttendueArticleItems,
      extras: {
        document_type: "AC",
        document_no: this.bonAttendueDeReception ? this.bonAttendueDeReception.linked_document_no : '',
        documentDetail: this.bonAttendueDeReception
      }
    });
    this.router.navigate(['/bon-de-reception/create']);
  }

  returnBack() {
    this.router.navigate(['/attendue-de-reception/list']);
  }

  _getLigneList(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.commonService.getAllLigneList().subscribe({
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  _getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            const brandList = res.data;

            // Filter only Client Brands
            this.clientBrandList = brandList.filter((row) => row.BRTYPE === 'client');

            // Filter only Supplier Brands
            this.supplierBrandList = brandList.filter((row) => row.BRTYPE === 'supplier');
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  _getAttendueDetail(document_no: string | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined || document_no === null || document_no === '') {
        reject();
        return;
      }

      this.attendueDeReceptionService.getAttendueDetail(document_no).subscribe( {
        next: (res: CustomHttpResponse<AttendueDetail>) => {
          if(res.status === "success") {
            const attendueData = res.data;
            this.bonAttendueDeReception = attendueData;
            if(!this.rolePermission['1_6'] && this.bonAttendueDeReception.status === APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_VALIDATED) {
              this.dialogService.displayAlert({
                title: this.translateService.instant('KERROR'),
                message: this.translateService.instant('KACCESS_DENIED')
              }).then(() => {
                this.returnBack();
              });
              return;
            }
            this.__refreshBonDetails(attendueData);
          } else {
            // If the bon attendue de reception not available
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/attendue-de-reception/list']);
              }
            );
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  __refreshBonDetails(attendueData: AttendueDetail) {
    if(attendueData !== undefined && attendueData !== null) {
      this.bonDocumentNum = attendueData.document_no;
      this.bonAttendueArticleItems = attendueData.articles;
      this.totalArticles = 0;
      this.totalQte = 0;
      this.totalPT = 0;

      this.bonAttendueArticleItems.forEach((row) => {
        this.totalArticles += 1;
        this.totalQte += +row.qte;
        // this.totalPT += +row.total_price;
      })

      this.bonDocumentForm.get("NUMBON")?.setValue(attendueData.document_no);
      this.bonDocumentForm.get("DATE_DU_BON")?.setValue(moment(attendueData.dated).format(CUSTOM_DATE_FORMAT));

      if(attendueData.ligne_lbl !== undefined && attendueData.ligne_lbl !== null && attendueData.ligne_lbl !== "") {
        if(attendueData.ligne_id === undefined || attendueData.ligne_id === null || attendueData.ligne_id === "") {
          // Custom Ligne Input
          this.customLigneInput = true;
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue('custom-ligne');
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(attendueData.ligne_lbl);
        } else {
          // Chosen Ligne Input from the dropdown
          const currentLigneItem = this.ligneList.find(r => r.LGIID == attendueData.ligne_id)
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue(currentLigneItem);
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(attendueData.ligne_lbl);
        }
      }
    }
  }

  __serialsListDisplay(serialsArr: string[]) {
    return serialsArr.join(", ");
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  get isAttendueModifiable() {
    return this.bonAttendueDeReception !== undefined;
  }

  get isDocumentInvoiced(): boolean {
    return this.bonAttendueDeReception !== undefined && this.bonAttendueDeReception !== null && this.bonAttendueDeReception.status === APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_INVOICED;
  }


  get isDocumentValidated(): boolean {
    return this.bonAttendueDeReception !== undefined && this.bonAttendueDeReception !== null && this.bonAttendueDeReception.status === APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_VALIDATED;
  }

  get isDocumentCancelled(): boolean {
    return this.bonAttendueDeReception !== undefined && this.bonAttendueDeReception !== null && this.bonAttendueDeReception.status === APP_CONSTANTS.ATTENDUE_DOCUMENT_STATUS_CANCELLED;
  }

  __resetNewLineItemForm() {
    this.newArticleSelection = undefined;
    this.bonAttendueCommandeItemForm.patchValue({
      ITEM_SUPPLIER_BRAND: "",
      ITEM_CLIENT_BRAND: "",
      ITEM_CLIENT_STORE: "",

      ARTICLE_ID: "",
      REFERENCE: "",
      DESCRIPTION: "",
      QTE: "0",
      PRIX_UNIT: "",
      PRIX_TOTAL: "0",
      MATERIAL_ACTIVABLE_STATUS: null,
      ITEM_STATUS: APP_CONSTANTS.ATTENDUE_STATUS_PENDING,
    })

    this.searchRefArticlesList = undefined;

    this.bonDocumentForm.patchValue({
      SEARCH_REF: ""
    });
  }

  confirmResetNewArticle(): Promise<any> {
    const message = `<h3>${this.translateService.instant('KCONFIRM_DISCARD_NEW_ITEM')}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  openHistorique(){
    const formData = this.bonDocumentForm.getRawValue();
    const data = {
      document_no: formData.NUMBON,
      document_type: APP_CONSTANTS.DOCUMENT_TYPE_ATTENDUE
    };
    this.dialog.open(HistoriqueComponent,
      {
        minWidth: '80%',
        maxHeight: '90vh',
        data: data || {}
      }
    );
  }
}
