import {Component, Inject, OnInit} from '@angular/core';
import {FormArray, FormBuilder, FormGroup, Validators} from "@angular/forms";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import {BonDeLivraisonService} from "../../../_services/bon-de-livraison.service";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import {TranslateService} from "@ngx-translate/core";
import {BonLivraison, BonLivraisonCustomer} from "../../../_models/livraison";
import {
  CustomHttpResponse,
  LineColis,
  ListDepotTNT, ListServiceExpedition,
  LivraisonExpedition
} from "../../../_models";
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material/dialog";
import {CommonService} from "../../../_services/common.service";
import { APP_CONSTANTS } from 'src/app/_helpers/constants';

@Component({
  selector: 'app-saisie-expedition',
  templateUrl: './saisie-expedition.component.html',
  styleUrls: ['./saisie-expedition.component.scss']
})
export class SaisieExpeditionComponent implements OnInit {

  expeditionForm: FormGroup = this.formBuilder.group({
    NUM_TICKET: [''],
    LIVRAISON_ID: ['', [Validators.required, Validators.minLength(1)]],
    NUM_BL: ['', [Validators.required, Validators.minLength(4), Validators.maxLength(255)]],
    ADDR_TNT: [''],
    COMPTE_DE_EXPEDITION: ['', [Validators.maxLength(255)]],
    RAISON_SOCIALE: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(255)]],
    ADDRES_1: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(255)]],
    ADDRES_2: ['', [Validators.minLength(3), Validators.maxLength(255)]],
    CODE_POSTAL: ['', [Validators.required, Validators.minLength(4), Validators.maxLength(255)]],
    COMMUNE: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(255)]],
    CONTACT: ['', [Validators.required, Validators.minLength(5), Validators.maxLength(255)]],
    TELEPHONE: [''],

    INSTRUCT_1: [''],
    INSTRUCT_2: [''],

    SERVICE_EXP: [''],
    LIVRAISON_SAMEDI: [false],
    REF_COLIS: ['', [Validators.required, Validators.minLength(3), Validators.maxLength(255)]],
    ETIQUETTE_RETOUR: [false],

    EXP_TYPE: ['ENTERPRICE', [Validators.required, Validators.minLength(3), Validators.maxLength(255)]],
    TICKET_GLPI_1: [''],
    TICKET_GLPI_2: [''],
    TICKET_GLPI_3: [''],

    LIST_COLIS: new FormArray([])
  });

  colisItemForm: FormGroup = this.formBuilder.group({
    NUM_COLIS: [''],
    POID_COLIS: ['', [Validators.required, ValidateGreaterThan]],
    ASSURANCE_COLIS: [''],
  });

  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  listDepotTnt: ListDepotTNT[] = [];
  listServiceExpedition: ListServiceExpedition[] = [];

  bonDocumentNum: string | undefined = "*********";
  bonLivraison: BonLivraison | undefined;
  bonCustomer: BonLivraisonCustomer | undefined;

  expeditionDetail: LivraisonExpedition | undefined;

  constructor(public dialogRef: MatDialogRef<SaisieExpeditionComponent>,
              @Inject(MAT_DIALOG_DATA) public data: any,
              private formBuilder: FormBuilder,
              private livraisonService: BonDeLivraisonService,
              private commonService: CommonService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService
              ) { }

  ngOnInit(): void {
    if(this.data !== undefined && this.data !== null) {
      this.bonLivraison = this.data.document;
      this.bonDocumentNum = this.data.document_no;

      if(this.bonLivraison?.customer !== undefined && this.bonLivraison?.customer !== null) {
        this.bonCustomer = this.bonLivraison?.customer;
      }
    }

    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.expeditionForm.patchValue({
      LIVRAISON_ID: this.bonLivraison?.livraison_id,
      NUM_BL: this.bonDocumentNum,
      REF_COLIS: this.bonDocumentNum
    });

    await this._getDepotTntList();
    await this._getServiceExpeditionList();

    if(this.bonDocumentNum !== undefined && this.bonDocumentNum !== null) {
      await this._getLivraisonExpeditionDetail(this.bonDocumentNum);

      if(this.expeditionDetail === undefined || this.expeditionDetail === null) {
        this.expeditionForm.patchValue({
          RAISON_SOCIALE: this.bonCustomer ? this.bonCustomer.store_nom : '',
          ADDRES_1: this.bonCustomer ? this.bonCustomer.store_address : '',
          ADDRES_2: '',
          CODE_POSTAL: this.bonCustomer ? this.bonCustomer.store_code_postal : '',
          COMMUNE: this.bonCustomer ? this.bonCustomer.store_ville : '',
          CONTACT: '',
          TELEPHONE: '',
        });
      }
    } else {
      this.expeditionForm.patchValue({
        RAISON_SOCIALE: this.bonCustomer ? this.bonCustomer.store_nom : '',
        ADDRES_1: this.bonCustomer ? this.bonCustomer.store_address : '',
        ADDRES_2: '',
        CODE_POSTAL: this.bonCustomer ? this.bonCustomer.store_code_postal : '',
        COMMUNE: this.bonCustomer ? this.bonCustomer.store_ville : '',
        CONTACT: '',
        TELEPHONE: '',
      });
    }

    await this.loadingService.hide();
  }

  _getDepotTntList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ListDepotTNT[]>) => {
          if(res.status === 'success') {
            this.listDepotTnt = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getDepotTNTList().subscribe(observer);
    });
  }

  _getServiceExpeditionList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ListServiceExpedition[]>) => {
          if(res.status === 'success') {
            this.listServiceExpedition = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getServiceExpeditionList().subscribe(observer);
    });
  }

  _getLivraisonExpeditionDetail(document_no: string): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<LivraisonExpedition>) => {
          if(res.status === 'success') {
            this.expeditionDetail = res.data;

            this.__refreshExpeditionDetails(this.expeditionDetail);
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.livraisonService.getLivraisonExpedition(document_no).subscribe(observer);
    });
  }

  colisItemFormSubmitter() {
    this.colisItemForm.get('POID_COLIS')?.addValidators([Validators.required]);
    this.colisItemForm.get('POID_COLIS')?.updateValueAndValidity();

    if(this.colisItemForm.invalid) {
      return;
    }

    const colisNum = `Num Colis ${(this.listColis.controls.length || 0) + 1}`
    this.colisItemForm.get('NUM_COLIS')?.setValue(colisNum);

    const formData = this.colisItemForm.getRawValue();

    const payload = {
      NUM_COLIS: formData.NUM_COLIS,
      POID_COLIS: formData.POID_COLIS,
      ASSURANCE_COLIS: formData.ASSURANCE_COLIS
    };

    this.addNewColis(payload);
    this.__resetColisItemForm();
  }

  __resetColisItemForm() {
    this.colisItemForm.patchValue({
      NUM_COLIS: '',
      POID_COLIS: '',
      ASSURANCE_COLIS: ''
    });
    this.colisItemForm.get('POID_COLIS')?.clearValidators();
    this.colisItemForm.get('POID_COLIS')?.updateValueAndValidity();
    this.colisItemForm.reset();
  }

  onAdresseTntChange() {
    const depotTntId = this.expeditionForm.get('ADDR_TNT')?.value;
    if(depotTntId !== undefined && depotTntId !== null && depotTntId !== '') {
      const depotTntDetail = this.listDepotTnt.find((row) => row.depot_id == depotTntId);
      this.expeditionForm.get('EXP_TYPE')?.setValue('AGENCE_TNT');
      this.expeditionForm.patchValue({
        ADDRES_1: depotTntDetail ? depotTntDetail.depot_address : '',
        ADDRES_2: '',
        CODE_POSTAL: depotTntDetail ? depotTntDetail.depot_code_postal : '',
        COMMUNE: depotTntDetail ? depotTntDetail.depot_ville : '',
        CONTACT: '',
        TELEPHONE: '',
      });
    } else {
      this.expeditionForm.patchValue({
        ADDRES_1: this.bonCustomer ? this.bonCustomer.store_address : '',
        ADDRES_2: '',
        CODE_POSTAL: this.bonCustomer ? this.bonCustomer.store_code_postal : '',
        COMMUNE: this.bonCustomer ? this.bonCustomer.store_ville : '',
        CONTACT: '',
        TELEPHONE: '',
      });
    }
  }

  get listColis(): FormArray {
    return this.expeditionForm.get('LIST_COLIS') as FormArray;
  }

  get isExpeditionModifiable() {
    return this.expeditionDetail !== undefined && this.expeditionDetail !== null;
  }

  addNewColis(payload: { NUM_COLIS: string, POID_COLIS: number, ASSURANCE_COLIS: number }) {
    const itemForm: FormGroup = this.formBuilder.group({
      NUM_COLIS: [payload.NUM_COLIS, Validators.required],
      POID_COLIS: [payload.POID_COLIS, [Validators.required, ValidateGreaterThan]],
      ASSURANCE_COLIS: [payload.ASSURANCE_COLIS]
    });

    this.listColis.push(itemForm);
  }

  removeColis(i: number) {
    this.listColis.removeAt(i);
  }

  expeditionFormSubmitter() {
    if(this.expeditionForm.invalid) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }

    if(this.listColis.length === 0) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KEXP_NO_COLIS_PROVIDED'
      });

      return;
    }

    const formData = this.expeditionForm.getRawValue();

    let totalColis = 0,
        totalPoids = 0,
        total_assurance = 0;

    let colisItems: any = [...formData.LIST_COLIS];
    colisItems = colisItems.map((row: any) => {
      const item: LineColis = {
        number: row.NUM_COLIS,
        poid: row.POID_COLIS,
        assurance: row.ASSURANCE_COLIS,
      };

      totalColis += 1;
      totalPoids += +row.POID_COLIS;
      total_assurance += +row.ASSURANCE_COLIS;

      return item;
    });

    const ticketGlpi = [];

    if(formData.TICKET_GLPI_1 !== undefined && formData.TICKET_GLPI_1 !== null && formData.TICKET_GLPI_1 !== "") {
      ticketGlpi.push(formData.TICKET_GLPI_1);
    }

    if(formData.TICKET_GLPI_2 !== undefined && formData.TICKET_GLPI_2 !== null && formData.TICKET_GLPI_2 !== "") {
      ticketGlpi.push(formData.TICKET_GLPI_2);
    }

    if(formData.TICKET_GLPI_3 !== undefined && formData.TICKET_GLPI_3 !== null && formData.TICKET_GLPI_3 !== "") {
      ticketGlpi.push(formData.TICKET_GLPI_3);
    }

    const payload: LivraisonExpedition = {
      ticket_number: formData.NUM_TICKET,
      livraison_id: formData.LIVRAISON_ID,
      livraison_document_no: formData.NUM_BL,
      depot_id: formData.ADDR_TNT,
      customer: {
        name: formData.RAISON_SOCIALE,
        address1: formData.ADDRES_1,
        address2: formData.ADDRES_2,
        code_postal: formData.CODE_POSTAL,
        ville: formData.COMMUNE,
        instruction1: formData.INSTRUCT_1,
        instruction2: formData.INSTRUCT_2,
        contact: formData.CONTACT,
        telephone: formData.TELEPHONE,
      },
      compte_de_expedition: formData.COMPTE_DE_EXPEDITION,
      service_expedition_id: formData.SERVICE_EXP,
      livraison_samdi_status: formData.LIVRAISON_SAMEDI,
      reference_number: formData.REF_COLIS,
      expedition_type: formData.EXP_TYPE,
      ticket_glp: ticketGlpi.join(','),
      return_status: formData.ETIQUETTE_RETOUR,
      colis: colisItems,
      total_colis: totalColis,
      total_poids: totalPoids,
      total_assurance: total_assurance
    };

    this.loadingService.show();
    if(this.isExpeditionModifiable && this.expeditionDetail?.livraison_expedition_id) {
      this._modifyExpedition(this.expeditionDetail?.livraison_expedition_id, payload);
      return;
    }

    this._createExpedition(payload);
  }

  _createExpedition(payload: LivraisonExpedition): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.livraisonService.createLivraisonExpedition(payload).subscribe( {
        next: (res: CustomHttpResponse<LivraisonExpedition>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const expeditionDetail = res.data;
            this.expeditionDetail = expeditionDetail;
            this.__refreshExpeditionDetails(expeditionDetail);
            this.dialogService.displayAlert({
              title: 'KSUCESS',
              message: res.message
            });
            this.dialogRef.close(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  _modifyExpedition(expedition_id: number, payload: LivraisonExpedition): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.livraisonService.modifyLivraisonExpedition(expedition_id, payload).subscribe( {
        next: (res: CustomHttpResponse<LivraisonExpedition>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const expeditionDetail = res.data;
            this.expeditionDetail = expeditionDetail;
            this.__refreshExpeditionDetails(expeditionDetail);
            this.dialogService.displayAlert({
              title: 'KSUCESS',
              message: res.message
            });
            this.dialogRef.close(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  __refreshExpeditionDetails(expeditionDetail: LivraisonExpedition) {
    if(expeditionDetail === undefined || expeditionDetail === null) {
      return;
    }
    let ticketGlp1 = '',
        ticketGlp2 = '',
        ticketGlp3 = '';
    if(expeditionDetail.ticket_glp !== undefined && expeditionDetail.ticket_glp !== null && expeditionDetail.ticket_glp !== '') {
      [ticketGlp1, ticketGlp2, ticketGlp3] = expeditionDetail.ticket_glp.split(',');
    }

    this.expeditionForm.patchValue({
      NUM_TICKET: expeditionDetail.ticket_number,
      ADDR_TNT: expeditionDetail.depot_id,

      RAISON_SOCIALE: expeditionDetail.customer.name,
      COMPTE_DE_EXPEDITION: expeditionDetail.compte_de_expedition,
      ADDRES_1: expeditionDetail.customer.address1,
      ADDRES_2: expeditionDetail.customer.address2,
      CODE_POSTAL: expeditionDetail.customer.code_postal,
      COMMUNE: expeditionDetail.customer.ville,
      CONTACT: expeditionDetail.customer.contact,
      TELEPHONE: expeditionDetail.customer.telephone,

      INSTRUCT_1: expeditionDetail.customer.instruction1,
      INSTRUCT_2: expeditionDetail.customer.instruction2,

      SERVICE_EXP: expeditionDetail.service_expedition_id,
      LIVRAISON_SAMEDI: expeditionDetail.livraison_samdi_status,
      REF_COLIS: expeditionDetail.reference_number,
      ETIQUETTE_RETOUR: expeditionDetail.return_status,

      EXP_TYPE: expeditionDetail.expedition_type,
      TICKET_GLPI_1: ticketGlp1 ? ticketGlp1 : '',
      TICKET_GLPI_2: ticketGlp2 ? ticketGlp2 : '',
      TICKET_GLPI_3: ticketGlp3 ? ticketGlp3 : '',
    });

    this.listColis.clear();
    if(expeditionDetail.colis !== undefined && expeditionDetail.colis !== null) {
      expeditionDetail.colis.forEach((row) => {
        const payload = {
          NUM_COLIS: row.number,
          POID_COLIS: row.poid,
          ASSURANCE_COLIS: row.assurance
        };
        this.addNewColis(payload);
      });
    }
  }

}
