import {Component, ElementRef, Inject, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {
  BonAchatCommande,
  BonAchatCommandeItem,
  BrandInfoList,
  CustomHttpResponse,
  StockLocationList
} from "../../../_models";
import {NumeroSerieComponent} from "../../inventory/numero-serie/numero-serie.component";
import {AbstractControl, FormArray, FormBuilder, FormControl, FormGroup, Validators} from "@angular/forms";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import {CommonService} from "../../../_services/common.service";
import {MAT_DIALOG_DATA, MatDialog, MatDialogRef} from "@angular/material/dialog";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {Article} from "../../../_models/article";
import {Subscription} from "rxjs";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import {TranslateService} from "@ngx-translate/core";

@Component({
  selector: 'app-add-bulk-items',
  templateUrl: './add-bulk-items.component.html',
  styleUrls: ['./add-bulk-items.component.scss']
})
export class AddBulkItemsComponent implements OnInit, OnDestroy {
  @ViewChild('prixUnitInput') prixUnitInput: ElementRef<HTMLElement> | undefined;
  linkedDocumentNo: string | undefined;
  linkedDocumentType: string | undefined;

  documentDetail: BonAchatCommande | undefined;
  bonItems: BonAchatCommandeItem[] | undefined;
  bonArticles: Article[] | undefined;
  brandList: BrandInfoList[] = [];
  stockLocationList: StockLocationList[] = [];

  bonReceptionBulkItemsForm: FormGroup = this.formBuilder.group({
    ITEMS: this.formBuilder.array([])
  });

  submitter: boolean = false;
  rolePermission:any;
  //PRIX_UNIT AND PRIX_TOTAL Permission
  PU_PT_Permission: boolean = false;
  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  returnDaysLength = APP_CONSTANTS.RETURN_DAYS_MAX_LENGTH;

  subscriptions: Subscription[] | undefined = [];

  constructor(@Inject(MAT_DIALOG_DATA) public data: any,
              public dialogRef: MatDialogRef<AddBulkItemsComponent>,
              private formBuilder: FormBuilder,
              private commonService: CommonService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService,
              private dialog: MatDialog) {

  }

  ngOnInit(): void {
    this.loadDependencies();

    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }

    this.PU_PT_Permission = this.rolePermission['11_1'];
    if(this.data !== undefined && this.data !== null) {
      if(this.data.hasOwnProperty('items')) {
        this.bonItems = this.data['items'];
        if(this.bonItems !== undefined && this.bonItems !== null) {
          this.processBonItems(this.bonItems);
        }
      }

      if(this.data.hasOwnProperty('linked_document_no')) {
        this.linkedDocumentNo = this.data['linked_document_no'];
      }

      if(this.data.hasOwnProperty('linked_document_type')) {
        this.linkedDocumentType = this.data['linked_document_type'];
      }

      if(this.data.hasOwnProperty('documentDetail')) {
        this.documentDetail = this.data['documentDetail'];
        this.bonArticles = this.documentDetail?.custom_articles;
      }
    }
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this._getStockLocationList();
    await this._getBrandList();

    await this.loadingService.hide();
  }

  ngOnDestroy() {
    if(this.subscriptions && this.subscriptions.length > 0) {
      this.subscriptions.forEach((_s) => {
        if(_s) {
          _s.unsubscribe();
        }
      });
    }
  }

  get lineItems(): FormArray {
    return this.bonReceptionBulkItemsForm.get("ITEMS") as FormArray;
  }

  getSerialList(rowIndex: number){
    const serials = this.lineItems.at(rowIndex).get('SERIALS_LIST')?.value;
    if(serials){
      return true;
    }
    return false;
  }

  qteOrigin(rowIndex: number): number {
    return this.lineItems.at(rowIndex).get('QTE_ORIGIN')?.value;
  }

  qteReceived(rowIndex: number): number {
    return this.lineItems.at(rowIndex).get('QTE_RECEIVED')?.value;
  }

  qteBlocked(rowIndex: number): number {
    return this.lineItems.at(rowIndex).get('QTE_BLOCKED')?.value;
  }

  qteRemaining(rowIndex: number): number {
    return this.lineItems.at(rowIndex).get('QTE_REMAINING')?.value;
  }

  itemModifiable(rowIndex: number) {
    return this.lineItems.at(rowIndex).get('ITEM_MODIFIABLE')?.value;
  }

  processBonItems(items: BonAchatCommandeItem[]) {
    for(let item of items) {
      this._newLineItem(item);
    }
  }

  _newLineItem(data?: BonAchatCommandeItem) {
    const newItem = this.formBuilder.group({
      ACHAT_ITEM_ID: ["", Validators.required],
      STOCK_SOURCE_ID: ["", Validators.required],
      ARTICLE_ID: ["", Validators.required],
      REFERENCE: [{ value: "", disabled: true }, Validators.required],
      LABEL: [{ value: "", disabled: true }, Validators.required],
      QTE: [{ value: "0", disabled: true }, [Validators.required, Validators.min(1)]],
      QTE_ORIGIN: [''],
      QTE_RECEIVED: [''],
      QTE_BLOCKED: [''],
      QTE_REMAINING: [''],
      ITEM_BRAND:[''],
      PRIX_UNIT: ['', [Validators.required, ValidateGreaterThan]],
      PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
      AVAIL_STOCK: [{ value: "0", disabled: true }],
      RETURN_STATUS: [0],
      RETURN_DAYS: [''],
      MATERIAL_ACTIVABLE_STATUS: [null],
      SERIALS_LIST: ['', Validators.required],
      ITEM_MODIFIABLE: [false]
    });

    if(data !== undefined && data !== null) {
      const originQte = data.qte;
      const receivedQte = data.qte_received || 0;
      const blockedQte = data.qte_blocked || 0;
      const remainingQte = originQte - receivedQte - blockedQte;

      const itemCancelled = data.item_status == APP_CONSTANTS.ACHAT_ITEM_STATUS_CANCELLED;

      newItem.get('ITEM_MODIFIABLE')?.setValue(false);
      if(remainingQte > 0 && !itemCancelled) {
        if(!this.PU_PT_Permission){
          newItem.get('PRIX_UNIT')?.clearValidators();
          newItem.get('PRIX_TOTAL')?.clearValidators();
        }
        newItem.get('QTE')?.setValidators([Validators.required, Validators.min(1), Validators.max(remainingQte)]);
        newItem.get('STOCK_SOURCE_ID')?.setValidators([Validators.required]);
        newItem.get('SERIALS_LIST')?.setValidators([Validators.required]);
        newItem.get('ITEM_MODIFIABLE')?.setValue(true);

        newItem.get('STOCK_SOURCE_ID')?.enable({ onlySelf: true });
        newItem.get('RETURN_STATUS')?.enable({ onlySelf: true });
        newItem.get('PRIX_UNIT')?.enable({ onlySelf: true });
        newItem.get('MATERIAL_ACTIVABLE_STATUS')?.enable({ onlySelf: true });
      } else {
        newItem.get('QTE')?.setValidators(null);
        newItem.get('STOCK_SOURCE_ID')?.setValidators(null);
        newItem.get('SERIALS_LIST')?.setValidators(null);
        newItem.get('ITEM_BRAND')?.disable({ onlySelf: true });
        newItem.get('STOCK_SOURCE_ID')?.disable({ onlySelf: true });
        newItem.get('RETURN_STATUS')?.disable({ onlySelf: true });
        newItem.get('PRIX_UNIT')?.disable({ onlySelf: true });
        newItem.get('MATERIAL_ACTIVABLE_STATUS')?.disable({ onlySelf: true });
      }
      newItem.updateValueAndValidity();
      newItem.patchValue({
        ACHAT_ITEM_ID: data.achat_item_id,
        ARTICLE_ID: data.article_id,
        REFERENCE: data.article_reference,
        LABEL: data.article_desc,
        ITEM_BRAND: data.supplier_brand_id,
        QTE: 0,
        QTE_ORIGIN: originQte, // Original full quantity
        QTE_RECEIVED: receivedQte, // Received quantity so far
        QTE_BLOCKED: blockedQte, // Added quantity Reception and yet to be validated
        QTE_REMAINING: remainingQte, // Received quantity so far
        PRIX_UNIT: data.unit_price,
        PRIX_TOTAL: data.total_price,
        AVAIL_STOCK: 0
      });
    }

    this.lineItems.push(newItem);
  }

  bonReceptionBulkItemsFormSubmitter() {
    this.submitter= true;
    const itemStatus = (this.bonReceptionBulkItemsForm.get('ITEMS') as FormArray).controls.some((r: any, index: number) => r.valid && this.itemModifiable(index));
    if(!itemStatus) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }

    const validItems: any = [];
    (this.bonReceptionBulkItemsForm.get('ITEMS') as FormArray).controls.filter((r: any) => r.valid && r.get('ITEM_MODIFIABLE').value).forEach((row: any) =>{
      validItems.push(row.getRawValue());
    });

    this.dialogRef.close({ status: 'success', data: validItems, extraDetail: this.data });
  }

  _getStockLocationList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.subscriptions?.push(this.commonService.getAllStockList().subscribe({
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.stockLocationList = res.data;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    })
  }

  async onStockSourceChange(rowIndex: number) {
    const currentRowItem = this.lineItems.at(rowIndex);

    const article_id = currentRowItem.get('ARTICLE_ID')?.value;
    const location_id = currentRowItem.get('STOCK_SOURCE_ID')?.value;

    const res: CustomHttpResponse<Article> = await this._getArticleDetail(article_id, location_id);
    const currentArticleDetail: Article = res.data;

    if(currentArticleDetail === undefined || currentArticleDetail === null) {
      return;
    }

    currentRowItem.patchValue({
      REFERENCE: currentArticleDetail.reference,
      LABEL: currentArticleDetail.label,
      AVAIL_STOCK: ((currentArticleDetail.avail_stock === undefined || currentArticleDetail.avail_stock === null) ? 0 : +currentArticleDetail.avail_stock).toString()
    });

    this.calculatePrixTotal(rowIndex);
  }


  _getBrandList(): Promise<any> { // get both Client and Supplier Brands
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }


  _getArticleDetail(article_id: number, location_id: number): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.commonService.getArticleDetail(article_id, location_id).subscribe({
        next: (res: CustomHttpResponse<Article>) => {
          if(res.status === 'success') {
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res.message);
          }
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  changeQuantity(rowIndex: number, unitPrixInput: any) {
    console.log("unitPrixInput", unitPrixInput)
    let oldListSerials = [];

    const currentRowItem = this.lineItems.at(rowIndex);

    const itemSerials = currentRowItem.get("SERIALS_LIST")?.value;
    if(itemSerials !== "") {
      oldListSerials = itemSerials.split(",");
    }

    oldListSerials = this.commonService.formatSerialNumbers(oldListSerials);

    const maximumDoableQte = this.qteRemaining(rowIndex);

    // Avoid S/N duplication across multiple items in the popup by
    // Combining all the S/Ns from all the items except the current item
    let cumulatedSerials: string[] = [];
    this.lineItems.controls.forEach((row, index) => {
      if(index !== rowIndex) {
        const itemSerials = row.get('SERIALS_LIST')?.value;

        let formattedSerials = [];
        if(itemSerials !== "") {
          formattedSerials = itemSerials.split(",");
        }

        cumulatedSerials = [
          ...cumulatedSerials,
          ...formattedSerials
        ];
      }
    });

    this.openUpSerialNumPopup(rowIndex, unitPrixInput, {
      oldListSerials,
      maxSerialsLimit: maximumDoableQte,
      block_in_serials: (cumulatedSerials.length > 0) ? cumulatedSerials.join(',') : undefined
    });
  }

  openUpSerialNumPopup(rowIndex: number, unitPrixInput: any, data?: any) {
    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: {
          numeroType:'reception-numero-series',
          ...data
        }
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        if(result.data && result.data !== '') {
          if(data.readonly === true) {
            return;
          }

          const currentRowItem = this.lineItems.at(rowIndex);

          currentRowItem.get('QTE')?.setValue(result.data.length.toString());
          currentRowItem.get('SERIALS_LIST')?.setValue(result.data.join(','));

          this.calculatePrixTotal(rowIndex);
          if(unitPrixInput){
            unitPrixInput?.nativeElement.select();
            unitPrixInput?.nativeElement.focus();
          }
        }
      }
    });
  }

  calculatePrixTotal(rowIndex: number) {
    const currentRowItem = this.lineItems.at(rowIndex) as FormGroup;

    const formData = currentRowItem.getRawValue();

    let totalPrix = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      currentRowItem.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      currentRowItem.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    currentRowItem.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
  }

  material_activable_status(rowIndex: number): number {
    const currentRowItem = this.lineItems.at(rowIndex);
    const article_id = currentRowItem.get('ARTICLE_ID')?.value;

    if(this.bonArticles) {
      const articleDetail = this.bonArticles.find(r => r.id == +article_id);

      if(articleDetail) {
        return articleDetail.material_activable_status;
      }
    }

    return 0;
  }

  itemReturnStatus(rowIndex: number): AbstractControl {
    const currentRowItem = this.lineItems.at(rowIndex);
    return currentRowItem.get("RETURN_STATUS") as AbstractControl;
  }

  onChangeReturnStatus(rowIndex: number) {
    const currentRowItem = this.lineItems.at(rowIndex);

    const returnStatus = currentRowItem.get('RETURN_STATUS')?.value;
    if(returnStatus === -1){
      currentRowItem.get('RETURN_DAYS')?.setValidators([Validators.required,Validators.min(1)])
      currentRowItem.get('RETURN_DAYS')?.updateValueAndValidity();
    }else{
      currentRowItem.get('RETURN_DAYS')?.clearValidators();
      currentRowItem.get('RETURN_DAYS')?.updateValueAndValidity();
    }
  }

}
