import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import {PageEvent} from '@angular/material/paginator';
import { MatDialog } from '@angular/material/dialog';
import { BonDeReceptionService } from '../../../_services/bon-de-reception.service';
import { TranslateService } from '@ngx-translate/core';
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {CustomHttpResponse, ListReception} from "../../../_models";
import {lastValueFrom} from "rxjs";
import { DashboardService } from 'src/app/_services/dashboard.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';

@Component({
  selector: 'app-bon-de-reception-list',
  templateUrl: './bon-de-reception-list.component.html',
  styleUrls: ['./bon-de-reception-list.component.scss']
})
export class BonDeReceptionListComponent implements OnInit {

  searchForm: FormGroup = this.formBuilder.group({
    document_no: [''],
    start_date: [{ value: '', disabled: true }],
    end_date: [{ value: '', disabled: true }],
    status: [''],
  });

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  listReception: ListReception[] | undefined;
  displayedColumnsReception: string[] = ['DOCUMENT_DATE', 'DOCUMENT_NO', 'LIGNE', 'STORE_NOM','STORE_TYPE','AFFILIATED_ACHAT', 'STATUS', 'ACTIONS'];
  rolePermission:any;
  statusList = APP_CONSTANTS.RECEPTION_STATUS;

  constructor(private formBuilder: FormBuilder,
              private router: Router,
              public dialog: MatDialog,
              private bonDeReceptionService: BonDeReceptionService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private translateService:TranslateService,
              private dashboardService:DashboardService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    const statusValue = this.dashboardService.dashboardStatusValue;
    this.dashboardService.dashboardStatusValue = '';
    if(statusValue){
      this.searchForm.get('status')?.setValue(parseInt(statusValue));
      this.searchFormSubmitter()
      return;
    }
    this.checkForListCache();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_RECEPTION_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        document_no: formCache.document_no,
        start_date: formCache.start_date,
        end_date: formCache.end_date,
        status: formCache.status,
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_RECEPTION_LIST);
      if(dataCache !== undefined) {

        this.listReception = dataCache.data as ListReception[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  documentStatus(item: ListReception) {
    
    let result = "";
    const statusList = APP_CONSTANTS.RECEPTION_STATUS;
    const statusData = statusList.find((r) => r.value == item.status);
    if(statusData == undefined) {
      return result;
    }
    switch (item.status) {
      case APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO:
        result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }
    
    return result;
  }

  isDocumentValidated(item: ListReception): boolean {
    return item.status === APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST_FORM, JSON.stringify(formData));
    const payload = {
      document_no: formData.document_no,
      start_date: formData.start_date,
      end_date: formData.end_date,
      status: formData.status
    };

    this.currentPage = 1;

    this._getReceptionList(payload, this.currentPage, this.pageSize);
  }

  _getReceptionList(payload: { document_no: string, start_date: string, end_date: string, status: string }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<ListReception[]>) => {
        if(res.status === "success") {
          this.listReception = res.data;

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.bonDeReceptionService.getReceptionList(payload, page, pageSize).subscribe(observer);
  }

  __updateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_RECEPTION_LIST);
    if(listCache !== undefined) {
      let listReception: any[] = listCache.data;
      listReception = (listReception as ListReception[]).filter(r => r.document_no != id);

      listCache.data = listReception;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listReception.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      document_no: formData.document_no,
      start_date: formData.start_date,
      end_date: formData.end_date,
      status: formData.status
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._getReceptionList(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveReception(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeReception(document_no: string, item:ListReception) {
    if((this.isDocumentValidated(item) && !this.rolePermission['2_27']) || (!this.isDocumentValidated(item) && !this.rolePermission['2_24'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveReception(document_no);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteReception(document_no);
    }
  }

  async _deleteReception(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(document_no);

          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.bonDeReceptionService.deleteReception(document_no).subscribe(observer);
  }

}
