import {DatePipe} from '@angular/common';
import {Component, ElementRef, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from '@angular/forms';
import {MatDialog} from '@angular/material/dialog';
import {ActivatedRoute, Params, Router} from '@angular/router';
import {firstValueFrom, lastValueFrom, Observable, Subscription} from 'rxjs';
import {BonDeReceptionService} from 'src/app/_services/bon-de-reception.service';
import {CommonService} from 'src/app/_services/common.service';
import {NumeroSerieComponent} from '../../inventory/numero-serie/numero-serie.component';
import * as moment from "moment";
import {
  BonReception, BonReceptionCustomer,
  BonReceptionItem,
  BrandInfoList,
  CustomHttpResponse,
  LigneInfoList,
  StockLocationList
} from "../../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {DateAdapter, MAT_DATE_FORMATS, MAT_DATE_LOCALE} from "@angular/material/core";
import {
  CUSTOM_DATE_API_FORMAT, CUSTOM_DATE_FORMAT,
  CUSTOM_DATE_FORMATS,
  CUSTOM_DATE_TIME_API_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import {MomentDateAdapter} from "@angular/material-moment-adapter";
import {MatSelect} from "@angular/material/select";
import {DialogService} from "../../../_services/dailog.service";
import {ModifyItemComponent} from "../modify-item/modify-item.component";
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {TranslateService} from "@ngx-translate/core";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {Article, SearchArticle} from "../../../_models/article";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import {DataSharingService} from "../../../_services/data-sharing.service";
import {BonLivraisonItem} from "../../../_models/livraison";
import { HistoriqueComponent } from 'src/app/_components/historique/historique.component';
import {AddBulkItemsComponent} from "../add-bulk-items/add-bulk-items.component";
import { IfStmt } from '@angular/compiler';
import {SearchArticleComponent} from "../../article/search-article/search-article.component";

@Component({
  selector: 'app-create-bon-de-reception',
  templateUrl: './create-bon-de-reception.component.html',
  styleUrls: ['./create-bon-de-reception.component.scss'],
  providers: [
    {provide: DateAdapter, useClass: MomentDateAdapter, deps: [MAT_DATE_LOCALE]},
    {provide: MAT_DATE_FORMATS, useValue: CUSTOM_DATE_FORMATS}
  ]
})

export class CreateBonDeReceptionComponent implements OnInit, OnDestroy {

  @ViewChild('prixUnitInput') unitPrixInputEl: ElementRef | undefined;
  @ViewChild('quantiteInput') quantiteInputEl: ElementRef | undefined;
  @ViewChild('stockLocationInput') stockLocationInputEl: MatSelect | undefined;
  @ViewChild('rechercheRefInput') rechercheRefInputEl: ElementRef | undefined;

  bonDocumentForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],
    RECEPTION_DATE: [{ value: new Date(), disabled: true }],
    AXESITE: [''],
    DOCUMENT_ASSOC: [''],

    LIGNE_INFO: [''], // Ligne ID
    LIGNE_CUSTOM_LABEL: [''], // Ligne Label

    SEARCH_STOCK_LOCATION: [""],
    SEARCH_REF: [""]
  })

  bonReceptionItemForm: FormGroup = this.formBuilder.group({
    STOCK_SOURCE_ID: ["", Validators.required],
    STOCK_SOURCE_LBL: [{ value: "", disabled: true }, Validators.required],
    ARTICLE_ID: ["", Validators.required],
    REFERENCE: [{ value: "", disabled: true }, Validators.required],
    LABEL: [{ value: "", disabled: true }, Validators.required],
    QTE: [{ value: "0", disabled: true }, [Validators.required, Validators.min(1)]],
    PRIX_UNIT: ['', [Validators.required, ValidateGreaterThan]],
    PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
    AVAIL_STOCK: [{ value: "0", disabled: true }],
    MATERIAL_ACTIVABLE_STATUS: [null],
    SERIALS_LIST: ['', Validators.required],
    ITEM_BRAND: ['']
  });

  newArticleSelection: Article | undefined;

  bonDocumentNum: string | undefined = "*********";
  bonReception: BonReception | undefined;
  bonReceptionItems: BonReceptionItem[] = [];
  bonReceptionArticles: Article[] = [];
  brandList: BrandInfoList[] = [];
  linkDocumentNo: string | undefined;
  linkDocumentType: string | undefined;

  totalArticles: number = 0;
  totalQte: number = 0;
  totalPT: number = 0;

  ligneList: LigneInfoList[] = [];
  stockLocationList: StockLocationList[] = [];
  newLineItemInsertable: boolean = true;
  fullKeywordMatch: boolean = false;
  isKeywordSerial: boolean = false;
  searchRefArticlesList: Article[] | undefined = [];
  // searchLblArticlesList: Article[] | undefined = [];

  displayedColumns: string[] = ['BRAND', 'REFERENCE', 'LABEL', 'STOCK_SOURCE', 'QTE', 'AVAIL_STOCK', 'ACTIVE_STATUS', 'ACTIONS'];
  expandedItem: Article | undefined;

  chosenCustomer: BonReceptionCustomer | undefined;

  customLigneInput: boolean = false;

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  //PRIX_UNIT AND PRIX_TOTAL Permission
  PU_PT_Permission: boolean = false;

  subscriptions: Subscription[] | undefined = [];

  rolePermission:any;

  constructor(public dialog: MatDialog,
              private commonService: CommonService,
              private receptionService: BonDeReceptionService,
              private datePipe: DatePipe,
              private router: Router,
              private route: ActivatedRoute,
              private formBuilder: FormBuilder,
              private dataSharingService: DataSharingService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService) {
  }

  ngOnInit(): void {
    this.loadDependencies();

    const ligneInfoSubscription = this.bonDocumentForm.get("LIGNE_INFO")?.valueChanges.subscribe(
      (res) => {
        if(res === 'custom-ligne') {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue("");
          this.customLigneInput = true;
        } else {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(res.LGILBL);
          this.customLigneInput = false;
        }
      }
    );

    if(ligneInfoSubscription) {
      this.subscriptions?.push(ligneInfoSubscription);
    }

    const searchRefSubscription = this.bonDocumentForm.get("SEARCH_REF")?.valueChanges.subscribe(
      (value) => {
        if(value === "" || value === null) {
          this.searchRefArticlesList = undefined;
        }
      }
    );

    if(searchRefSubscription) {
      this.subscriptions?.push(searchRefSubscription);
    }
  }

  ngOnDestroy() {
    if(this.subscriptions && this.subscriptions.length > 0) {
      this.subscriptions.forEach((_s) => {
        if(_s) {
          _s.unsubscribe();
        }
      });
    }
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.bonDocumentForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)
    });
    this._getRolePermission();
    await this._getLigneList();
    await this._getBrandList();
    await this._getStockLocationList();

    firstValueFrom(this.route.queryParams).then(
      (params) => {
        if(params.hasOwnProperty('document_no') && params["document_no"] !== null && params["document_no"] !== "") {
          this.bonDocumentNum = params["document_no"];

          this._getBonReceptionDetail(this.bonDocumentNum);
        }

        if(params.hasOwnProperty('_doc') && params["_doc"] !== null && params["_doc"] !== "") {
          this.linkDocumentNo = params["_doc"];
          this.linkDocumentType = params["_doc_t"];
        }
      }
    )

    const selectionSubscription = this.dataSharingService.selectionItems.subscribe((res) => {
      // res = JSON.parse(localStorage.getItem('test_achat') || "");
      console.log(res);
      if(res !== undefined && res !== null) {
        const { type, items, extras } = res;
        const { document_type, document_no, documentDetail } = extras;

        if(type === APP_CONSTANTS.SELECTION_ACHAT_TO_BR) {
          // Open Up the ACHAT-TO-RECEPTION Items popup to fill multiple items at once and update
          const data = {
            items: items,
            linked_document_no: document_no,
            linked_document_type: document_type,
            documentDetail: documentDetail
          };

          this.openUpBulkItemsPopup(document_no, document_type, data);
          // and once all the items have filled, we should add them as a new Bon Reception document
          this.dataSharingService.selectionItems.next(undefined);
        }

        if(type === APP_CONSTANTS.SELECTION_BL_TO_BR) {
          let livraisonItems: any[] = [...items];

          livraisonItems = livraisonItems.map((row: BonLivraisonItem) => {
            return {
              stock_source: +row.stock_source,
              article_id: +row.article_id,
              article_reference: row.article_reference,
              article_desc: row.article_desc,
              qte: +row.qte,
              qte_stock: +row.qte_stock,
              unit_price: +row.unit_price,
              total_price: +row.total_price,
              material_activable_status: +row.material_activable_status,
              article_serials: row.article_serials
            }
          });

          this._addCopiedBonReceptionItems(livraisonItems as BonReceptionItem[], document_no, document_type);
        }
      }
    })

    this.subscriptions?.push(selectionSubscription);

    await this.loadingService.hide();
  }

  _getRolePermission(){
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.PU_PT_Permission = this.rolePermission['11_1'];
    if(this.PU_PT_Permission){
      this.displayedColumns = ['BRAND', 'REFERENCE', 'LABEL', 'STOCK_SOURCE', 'QTE', 'PRIX_UNIT', 'PRIX_TOTAL', 'AVAIL_STOCK', 'ACTIVE_STATUS', 'ACTIONS'];
    }else{
      this.bonReceptionItemForm.get('PRIX_UNIT')?.clearValidators();
    }
  }

  async onCloseDocument() {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    if(this.isBonReceptionModifiable && (this.chosenCustomer === undefined || this.chosenCustomer === null)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KRECEPTION_CLIENT_SUPPLIER_NOT_ASSIGNED'
      })

      return;
    }

    this.returnBack();
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST_FORM);
  }

  async onSubmitDocument(type: 'ok' | 'validate' = 'ok') {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    if(type === 'validate'){
      if(!this.rolePermission['2_25']) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KACCESS_DENIED'
        })
        return;
      }
    }

    if(!this.isBonReceptionModifiable) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ARTICLE_ADDED'
      })

      return;
    }

    if(this.chosenCustomer === undefined || this.chosenCustomer === null) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KRECEPTION_CLIENT_SUPPLIER_NOT_ASSIGNED'
      })

      return;
    }

    const documentFormData = this.bonDocumentForm.getRawValue();

    const bonArticles: BonReceptionItem[] = [...this.bonReceptionItems];
    const bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    const bonStatus = type === 'validate' ? APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES : (this.bonReception?.status ? this.bonReception?.status : APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO);

    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonReception = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,

      customer: bonCustomer,
      articles: bonArticles,

      total_price: bonTotalPrix,
      status: bonStatus,
    };
    this.__resetListCache();

    this._modifyReception(this.bonDocumentNum, bonDocument).then(
      () => {
          this.returnBack(type === 'validate' ? 'consultation' : 'list');
      }
    )
  }

  generateReceptionPdf(print?: boolean) {
    if(!this.isBonReceptionModifiable || (this.bonDocumentNum === undefined)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KRECEPTION_NO_CONTENT'
      })
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.receptionService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      }
    });
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_BON_DE_RECEPTION_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  returnBack(type: 'list' | 'consultation' = 'list') {
    if(type === 'list') {
      this.router.navigate(['/bon-de-reception/list']);
    } else if (type === 'consultation') {
      this.router.navigate(['/bon-de-reception/detail', this.bonDocumentNum]);
    }
  }

  _getLigneList(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.subscriptions?.push(this.commonService.getAllLigneList().subscribe({
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    });
  }

  _getBrandList(): Promise<any> { // get both Client and Supplier Brands
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  _getStockLocationList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.subscriptions?.push(this.commonService.getAllStockList().subscribe({
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.stockLocationList = res.data;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    })
  }

  _searchArticles(data: {keyword: string, location_id: number }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      let searchObs: Observable<CustomHttpResponse<SearchArticle>> | undefined;

      searchObs = this.commonService.searchArticlesListByRef(data);

      if(searchObs === undefined) {
        return;
      }

      this.subscriptions?.push(searchObs.subscribe({
        next: (res: CustomHttpResponse<SearchArticle>) => {
          if(res.status === 'success') {
            this.fullKeywordMatch = res.data.fullMatch || false;
            this.isKeywordSerial = res.data.isSerial || false;
            this.searchRefArticlesList = res.data.rows;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    })
  }

  _getBonReceptionDetail(document_no: string | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined || document_no === null || document_no === '') {
        reject();
        return;
      }

      this.subscriptions?.push(this.receptionService.getReceptionDetail(document_no).subscribe( {
        next: (res: CustomHttpResponse<BonReception>) => {
          if(res.status === "success") {
            const bonReceptionData = res.data;
            if(bonReceptionData.linked_document_type == "AC"){
              this.newLineItemInsertable = false;
            }
            this.bonReception = bonReceptionData;
            if(this.bonReception.status === APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES) {
              this.dialogService.displayAlert({
                title: 'KERROR',
                message: 'KDOCUMENT_VALIDATED_NO_FURTHER_CHANGES',
                messageTranslateParam: { DOCUMENT_NO: document_no }
              }).then(() => {
                this.returnBack();
              });

              return;
            }

            this.__refreshBonDetails(bonReceptionData);
          } else {
            // If the bon reception not available
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/bon-de-reception/create']);
              }
            );
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    })
  }

  __refreshBonDetails(bonReception: BonReception) {
    if(bonReception !== undefined && bonReception !== null) {
      this.bonDocumentNum = bonReception.document_no;
      this.bonReceptionItems = bonReception.articles;
      this.bonReceptionArticles = bonReception?.custom_articles ? bonReception?.custom_articles : [];

      this.chosenCustomer = bonReception.customer;

      this.totalArticles = 0;
      this.totalQte = 0;
      this.totalPT = 0;

      this.bonReceptionItems.forEach((row) => {
        this.totalArticles += 1;
        this.totalQte += +row.qte;
        this.totalPT += +row.total_price;
      })

      this.bonDocumentForm.get("NUMBON")?.setValue(bonReception.document_no);
      this.bonDocumentForm.get("DATE_DU_BON")?.setValue(moment(bonReception.dated).format(CUSTOM_DATE_FORMAT));
      this.bonDocumentForm.get("RECEPTION_DATE")?.setValue((bonReception.reception_date !== undefined && bonReception.reception_date !== null) ? moment(bonReception.reception_date) : '');
      this.bonDocumentForm.get("AXESITE")?.setValue(bonReception.axesite);
      this.bonDocumentForm.get("DOCUMENT_ASSOC")?.setValue(bonReception.docs_assoc);

      if(bonReception.ligne_lbl !== undefined && bonReception.ligne_lbl !== null && bonReception.ligne_lbl !== "") {
        if(bonReception.ligne_id === undefined || bonReception.ligne_id === null || bonReception.ligne_id === "") {
          // Custom Ligne Input
          this.customLigneInput = true;
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue('custom-ligne');
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonReception.ligne_lbl);
        } else {
          // Chosen Ligne Input from the dropdown
          const currentLigneItem = this.ligneList.find(r => r.LGIID == bonReception.ligne_id)
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue(currentLigneItem);
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonReception.ligne_lbl);
        }
      }
    }
  }

  __serialsListDisplay(serialsArr: string[]) {
    return serialsArr.join(", ");
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  get documentAssocie() {
    return this.bonDocumentForm.get("DOCUMENT_ASSOC");
  }

  get material_activable_status() {
    return this.newArticleSelection && this.newArticleSelection?.material_activable_status;
  }

  get isBonReceptionModifiable() {
    return this.bonReception !== undefined;
  }

  get __isDocumentValidated() {
    return this.bonReception && this.bonReception.status === APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES;
  }

  get QTE_VALUE(){
    return this.bonReceptionItemForm.get('QTE')?.value;
  }

  async addBonReceptionItemFormSubmitter() {
    if(this.bonReceptionItemForm.invalid || this.QTE_VALUE === '0') {
      this.dialogService.displayAlert({
        title: "Error!",
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }

    this.loadingService.show();
    const documentFormData = this.bonDocumentForm.getRawValue();
    const itemFormData = this.bonReceptionItemForm.getRawValue();
    console.log(itemFormData);
    let bonArticles: BonReceptionItem[] = [];

    if(this.isBonReceptionModifiable) {
      bonArticles = [...this.bonReceptionItems];
    }

    const itemSerials = itemFormData.SERIALS_LIST.split(',');

    bonArticles.push({
      stock_source: +itemFormData.STOCK_SOURCE_ID,
      article_id: +itemFormData.ARTICLE_ID,
      article_reference: itemFormData.REFERENCE,
      article_desc: itemFormData.LABEL,
      qte: +itemFormData.QTE,
      qte_stock: +itemFormData.AVAIL_STOCK,
      unit_price: +itemFormData.PRIX_UNIT,
      total_price: +itemFormData.PRIX_TOTAL,
      material_activable_status: itemFormData.MATERIAL_ACTIVABLE_STATUS,
      article_serials: itemSerials,
      brand_id: itemFormData.ITEM_BRAND,
    })
    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
    let bonStatus = APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO;

    if(this.isBonReceptionModifiable) {
      bonStatus = this.bonReception?.status ? this.bonReception?.status : APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO;
    }

    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonReception = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,

      customer: bonCustomer,
      articles: bonArticles,

      total_price: bonTotalPrix,
      status: bonStatus,

      linked_document_no: this.linkDocumentNo ? this.linkDocumentNo : '',
      linked_document_type: this.linkDocumentType ? this.linkDocumentType : '',
    };

    console.log('payload: ', bonDocument);
    if(this.isBonReceptionModifiable) {
      delete bonDocument.linked_document_no;
      delete bonDocument.linked_document_type;

      this._modifyReception(this.bonDocumentNum, bonDocument).then(
        () => {
          this.__resetNewLineItemForm();
        }
      )

      return;
    }

    try {
      await this._addNewLineItem(bonDocument);

      this.__resetNewLineItemForm();
      // After adding a new Line item.
      this.__updateCurrentRouteDocumentNum(this.bonDocumentNum);
    } catch (e: any) {
      // Catch Block
    }
  }

  _addCopiedBonReceptionItems(items: BonReceptionItem[], link_document_no: string, link_document_type: string, achat_commande_id?:number, attendue_reception_id?: number) {
    if(items === undefined || items === null || items.length === 0) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_SELECTION_ITEM_AVAILABLE'
      });

      return;
    }

    this.loadingService.show();
    const documentFormData = this.bonDocumentForm.getRawValue();
    let bonArticles: BonReceptionItem[] = [...items];

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
    let bonStatus = APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO;

    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonReception = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,
      customer: bonCustomer,
      articles: bonArticles,
      total_price: bonTotalPrix,
      status: bonStatus,
      achat_commande_id: achat_commande_id,
      attendue_reception_id: attendue_reception_id,
      linked_document_no: link_document_no,
      linked_document_type: link_document_type
    };

    this._addNewLineItem(bonDocument).then(
      () => {
        if(link_document_type === "AC"){
          this.newLineItemInsertable = false;
        }
        this.__resetNewLineItemForm();
        // After adding a new Line item.
        this.__updateCurrentRouteDocumentNum(this.bonDocumentNum);
      }
    )
  }

  removeLineItem(currentItem: BonReceptionItem) {
    this.loadingService.show();

    let bonArticles: BonReceptionItem[] = [...this.bonReceptionItems];
    bonArticles = bonArticles.filter((row) => row.item_id !== currentItem.item_id);

    const documentFormData = this.bonDocumentForm.getRawValue();

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    const bonStatus = this.bonReception?.status ? this.bonReception?.status : APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO;

    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    let bonDocument: BonReception = {
      ...this.bonReception,

      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',

      customer: bonCustomer,

      articles: bonArticles,
      total_price: bonTotalPrix,
      status: bonStatus
    };

    delete bonDocument.reception_id;

    this._modifyReception(this.bonDocumentNum, bonDocument);
  }

  _addNewLineItem(bonDocument: BonReception): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.subscriptions?.push(this.receptionService.createReception(bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonReception>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonReceptionData = res.data;
            this.bonReception = bonReceptionData;

            // Emptying the Selection items once the items are added successfully.
            this.dataSharingService.selectionItems.next(undefined);

            this.__refreshBonDetails(bonReceptionData);
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });

            // Emptying the Selection items if we got any errors
            this.dataSharingService.selectionItems.next(undefined);

            reject(res);
          }
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });

          // Emptying the Selection items if we got any errors
          this.dataSharingService.selectionItems.next(undefined);

          reject(err);
        }
      }));
    })
  }

  _modifyReception(document_no: string | undefined, bonDocument: BonReception ): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined) {
        reject("INVALID_DOCUMENT_NO");
        return;
      }

      this.subscriptions?.push(this.receptionService.modifyReception(document_no, bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonReception>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonReceptionData = res.data;
            this.bonReception = bonReceptionData;
            this.__refreshBonDetails(bonReceptionData);
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }));
    })
  }

  __updateCurrentRouteDocumentNum(document_no: string | undefined) {
    if(document_no === undefined) {
      return;
    }

    const queryParams: Params = { document_no };

    this.router.navigate(
      [],
      {
        relativeTo: this.route,
        queryParams: queryParams
      });
  }

  __resetNewLineItemForm() {
    this.newArticleSelection = undefined;
    this.bonReceptionItemForm.patchValue({
      STOCK_SOURCE_ID: "",
      STOCK_SOURCE_LBL: "",
      ARTICLE_ID: "",
      REFERENCE: "",
      LABEL: "",
      QTE: "0",
      PRIX_UNIT: "",
      PRIX_TOTAL: "0",
      AVAIL_STOCK: "",
      MATERIAL_ACTIVABLE_STATUS: null,
      SERIALS_LIST: ""
    })

    this.searchRefArticlesList = undefined;
    // this.searchLblArticlesList = undefined;

    this.bonDocumentForm.patchValue({
      SEARCH_STOCK_LOCATION: "",
      SEARCH_REF: "",
    });
  }

  onStockLocationChange(event: any) {
    this.stockLocationInputEl?.close();

    if(this.newArticleSelection) {
      // Changing the Stock location when already an article selected
      const referenceValue = this.bonDocumentForm.get("SEARCH_REF")?.value;

      if(referenceValue !== '') {
        this.searchArticles(undefined).then(
          (res) => {
            const currentArticleMatch = this.searchRefArticlesList?.find((r) => r.id === this.newArticleSelection?.id);
            if(currentArticleMatch) {
              this.newArticleSelection = currentArticleMatch;

              this.__fillNewLineItemForm('existing');
            }
          }
        );
      }
    } else {
      // Changing the Stock location at first
      this.rechercheRefInputEl?.nativeElement.focus();

      this.searchRefArticlesList = undefined;

      // Changing the Stock location when already an article selected
      const referenceValue = this.bonDocumentForm.get("SEARCH_REF")?.value;

      if(referenceValue !== '') {
        this.searchArticles(undefined).then(
          (res) => {
            const currentArticleMatch = this.searchRefArticlesList?.find((r) => r.id === this.newArticleSelection?.id);
            if(currentArticleMatch) {
              this.newArticleSelection = currentArticleMatch;

              this.__fillNewLineItemForm('existing');
            }
          }
        );
      }
    }
  }

  async searchArticles(event: any | undefined): Promise<any> {
    return new Promise(async (resolve, reject) => {
      if(event !== undefined && (event.keyCode == 38 || event.keyCode == 40 || event.keyCode == 13)) {
        return;
      }

      const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
      if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
        this.searchRefArticlesList = undefined;

        this.stockLocationInputEl?.focus();
        this.stockLocationInputEl?.open();
        return;
      }

      let payload = {
        location_id: +searchStockLocation,
        keyword: ''
      };

        const searchRefKeyword = this.bonDocumentForm.get("SEARCH_REF")?.value;

        if(searchRefKeyword === '' || searchRefKeyword === null) return;

        payload = {
          ...payload,
          keyword: searchRefKeyword
        };
      const searchResult = await this._searchArticles(payload);

      resolve(searchResult);
    })
  }

  async onScannerSearchArticle() {
    const searchRef = this.bonDocumentForm.get('SEARCH_REF')?.value;
    if(searchRef == undefined || searchRef == "") {
      return;
    }

    await this.searchArticles(searchRef);

    if(this.searchRefArticlesList === undefined || this.searchRefArticlesList === null) {
      return;
    }

    if(this.searchRefArticlesList.length > 0 && this.fullKeywordMatch) {
      if(this.isKeywordSerial) {
        const firstArticle = this.searchRefArticlesList[0];
        this.searchSelected(firstArticle, true);
      } else {
        const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
        if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
          this.searchRefArticlesList = undefined;

          this.stockLocationInputEl?.focus();
          this.stockLocationInputEl?.open();
          return;
        }

        const firstArticle = this.searchRefArticlesList[0];
        this.searchSelected(firstArticle, true);
      }
    } else {
      const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
      if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
        this.searchRefArticlesList = undefined;

        this.stockLocationInputEl?.focus();
        this.stockLocationInputEl?.open();
        return;
      }

      this.openSearchArticlePopup();
    }
  }

  async openSearchArticlePopup() {
    const searchKeyword = this.bonDocumentForm.get('SEARCH_REF')?.value;
    const stockLocation = this.bonDocumentForm.get('SEARCH_STOCK_LOCATION')?.value;

    const data = {
      currentArticle: this.newArticleSelection,
      stockLocation: stockLocation,
      searchKeyword: searchKeyword
    };

    const dialog = this.dialog.open(SearchArticleComponent, {
      data,
      maxHeight: "90vh"
    });

    dialog.afterClosed().subscribe((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        this.newArticleSelection = res.data;

        this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);
        this.bonReceptionItemForm.get("SERIALS_LIST")?.setValue(this.newArticleSelection?.serial_list);
        console.log(this.bonReceptionItemForm.getRawValue());

        this.__fillNewLineItemForm('new');
        console.log('SEARCH RES: ', res);

        this.changeQuantity('creation');
      }
    });
  }

  searchSelected(event: any, directCall: boolean = false) {
    if(directCall) {
      this.newArticleSelection = event;
    } else {
      this.newArticleSelection = event.option.value;
    }

    this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);

    this.__fillNewLineItemForm('new');

    this.changeQuantity('creation');
  }

  __fillNewLineItemForm(type: 'new' | 'existing') {
    const stockLocationId = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
    const stockLocationDetail = this.stockLocationList.find((_r) => _r.STLID == stockLocationId);

    let unitPrix = 0;
    if(this.newArticleSelection?.public_prix !== undefined && this.newArticleSelection?.public_prix !== null) {
      unitPrix = this.newArticleSelection?.public_prix;
    }

    this.bonReceptionItemForm.patchValue({
      STOCK_SOURCE_ID: stockLocationDetail?.STLID,
      STOCK_SOURCE_LBL: stockLocationDetail?.STLLBL,
      ARTICLE_ID: this.newArticleSelection?.id,
      REFERENCE: this.newArticleSelection?.reference,
      LABEL: this.newArticleSelection?.label,
      AVAIL_STOCK: (this.newArticleSelection?.avail_stock === null ? 0 : +this.newArticleSelection?.avail_stock).toString(),
    });

    if(type === "new") {
      this.bonReceptionItemForm.patchValue({
        QTE: "0",
        PRIX_UNIT: unitPrix.toString(),
        PRIX_TOTAL: "0",
        SERIALS_LIST: ""
      });
    }
  }

  changeQuantity(type: 'creation' | 'modification') {
    let oldListSerials = [];

    const itemSerials = this.bonReceptionItemForm.get("SERIALS_LIST")?.value;
    if(itemSerials !== "") {
      oldListSerials = itemSerials.split(",");
    }

    oldListSerials = this.commonService.formatSerialNumbers(oldListSerials);

    this.openUpSerialNumPopup({
      oldListSerials
    });
  }

  viewItemSerials(item: BonReceptionItem) {
    const oldListSerials = this.commonService.formatSerialNumbers(item.article_serials);

    this.openUpSerialNumPopup({
      oldListSerials,
      readonly: true
    });
  }

  openUpSerialNumPopup(data?: any) {
    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: {
          numeroType:'reception-numero-series',
          ...data
        }
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        if(result.data && result.data !== '') {
          if(data.readonly === true) {
            return;
          }

          this.bonReceptionItemForm.get('QTE')?.setValue(result.data.length.toString());
          this.bonReceptionItemForm.get('SERIALS_LIST')?.setValue(result.data.join(','));

          this.unitPrixInputEl?.nativeElement.select();
          this.unitPrixInputEl?.nativeElement.focus();
          this.quantiteInputEl?.nativeElement.dispatchEvent(new Event('change'));
        }
      }
    });
  }

  confirmResetNewArticle(): Promise<any> {
    const message = `<h3>${this.translateService.instant('KCONFIRM_DISCARD_NEW_ITEM')}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async modifyLineItem(item: BonReceptionItem) {

    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    const data = {
      item: item,
      numeroType: "reception-numero-series",
      document_no: this.bonDocumentNum,
      document: this.bonReception,
    };

    this.openUpModifyItemPopup(data);
  }

  openUpModifyItemPopup(data?: {}) {
    const dialogRef = this.dialog.open(ModifyItemComponent,
      {
        maxWidth: '35vw',
        data: data || {}
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        console.log(result.data);
        if(result.data && result.data !== '') {
          this.loadingService.show();
          const bonDocument = result.data;
          this._modifyReception(this.bonDocumentNum, bonDocument);
        }
      }
    });
  }

  openUpBulkItemsPopup(document_no: string, document_type: string, data?: {}) {
    const dialogRef = this.dialog.open(AddBulkItemsComponent,
      {
        width: '90vw',
        data: data || {}
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        if(result.data && result.data !== '') {
          this.loadingService.show();
          let achat_commande_id = result.extraDetail.documentDetail.achat_commande_id;
          let attendue_reception_id = result.extraDetail.documentDetail.attendue_reception_id;
          let bonReceptionItems = [];
          const lineItems = result.data;
          for(let row of lineItems) {
            const itemSerials = row.SERIALS_LIST ? row.SERIALS_LIST.split(',') : [];
            const attendue_item = result.extraDetail?.documentDetail?.articles.find((article:any) => row.ACHAT_ITEM_ID === article.achat_item_id);
            bonReceptionItems.push({
              achat_item_id: row.ACHAT_ITEM_ID,
              stock_source: +row.STOCK_SOURCE_ID,
              article_id: +row.ARTICLE_ID,
              article_reference: row.REFERENCE,
              article_desc: row.LABEL,
              qte: +row.QTE,
              qte_stock: +row.AVAIL_STOCK,
              unit_price: +row.PRIX_UNIT,
              total_price: +row.PRIX_TOTAL,
              material_activable_status: row.MATERIAL_ACTIVABLE_STATUS ? +row.MATERIAL_ACTIVABLE_STATUS : 0,
              article_serials: itemSerials,
              return_status: +row.RETURN_STATUS,
              return_days: row.RETURN_DAYS,
              attendue_item_id : attendue_item.item_id,
              brand_id: row.ITEM_BRAND
            });
          }
          this.bonReceptionItems = bonReceptionItems;
          if(bonReceptionItems.length === 0) {
            this.loadingService.hide();
            return;
          }

          this._addCopiedBonReceptionItems(bonReceptionItems as BonReceptionItem[], document_no, document_type, achat_commande_id, attendue_reception_id);
        }
      }
    });
  }

  onBonDocumentAssocieChange(documentAss: string) {
    if(this.documentAssocie?.value !== documentAss) {
      this.documentAssocie?.setValue(documentAss);
    }
  }

  onBonCustomerChange(customer: BonReceptionCustomer) {
    if(this.chosenCustomer !== customer) {
      this.chosenCustomer = customer;

      if(this.isBonReceptionModifiable) {
        this._updateBonCustomer();
      }
    }
  }

  _updateBonCustomer() {
    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : ""
    };

    const documentFormData = this.bonDocumentForm.getRawValue();
    const bonStatus = this.bonReception?.status ? this.bonReception?.status : APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO;
    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonReception = {
      ...this.bonReception,

      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,

      customer: bonCustomer,
      articles: this.bonReceptionItems,

      total_price: this.bonReception?.total_price ? this.bonReception?.total_price : 0,
      status: bonStatus,
    };

    this._modifyReception(this.bonDocumentNum, bonDocument);
  }

  calculatePrixTotal(){
    const formData = this.bonReceptionItemForm.getRawValue();

    let totalPrix = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      this.bonReceptionItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      this.bonReceptionItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    this.bonReceptionItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
  }

  openHistorique(){
    const formData = this.bonDocumentForm.getRawValue();
    const data = {
      document_no: formData.NUMBON,
      document_type: APP_CONSTANTS.DOCUMENT_TYPE_RECEPTION
    };
    this.dialog.open(HistoriqueComponent,
      {
        minWidth: '80%',
        maxHeight: '90vh',
        data: data || {}
      }
    );
  }

  confirmRemoveReception(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeReception() {
    if(this.bonDocumentNum == undefined){
      return
    }
    if((this.__isDocumentValidated && !this.rolePermission['2_27']) || (!this.__isDocumentValidated && !this.rolePermission['2_24'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveReception(this.bonDocumentNum);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteReception(this.bonDocumentNum);
    }
  }

  async _deleteReception(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/bon-de-reception/list']);
          return;
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.receptionService.deleteReception(document_no).subscribe(observer);
  }

}
