import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import {DatePipe} from '@angular/common';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import {ActivatedRoute, Router} from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import * as moment  from 'moment';
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {MatDialog} from "@angular/material/dialog";
import {NgxSpinnerService} from "ngx-spinner";
import { BonDeReceptionService } from 'src/app/_services/bon-de-reception.service';
import { NumeroSerieComponent } from '../../inventory/numero-serie/numero-serie.component';
import {MatSelect} from "@angular/material/select";
import {
  BonReception,
  BonReceptionCustomer,
  BonReceptionItem, BrandInfoList, CustomHttpResponse,
  LigneInfoList,
  ListReception,
  StockLocationList, Store
} from "../../../_models";
import {CommonService} from "../../../_services/common.service";
import {DialogService} from "../../../_services/dailog.service";
import {
  CUSTOM_DATE_API_FORMAT,
  CUSTOM_DATE_FORMAT, CUSTOM_DATE_FORMATS,
  CUSTOM_DATE_TIME_API_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import {firstValueFrom, lastValueFrom, Observable} from "rxjs";
import {ModifyItemComponent} from "../modify-item/modify-item.component";
import {DateAdapter, MAT_DATE_FORMATS, MAT_DATE_LOCALE} from "@angular/material/core";
import {MomentDateAdapter} from "@angular/material-moment-adapter";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {Article, SearchArticle} from "../../../_models/article";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import {MatCheckboxChange} from "@angular/material/checkbox";
import {DataSharingService} from "../../../_services/data-sharing.service";
import {FusionnerTypeComponent} from "../../../_components/fusionner-type/fusionner-type.component";
import {SelectionCopyTypeComponent} from "../../../_components/selection-copy-type/selection-copy-type.component";
import { HistoriqueComponent } from 'src/app/_components/historique/historique.component';
import {cloneDeep} from "lodash";
import {SearchArticleComponent} from "../../article/search-article/search-article.component";

@Component({
  selector: 'app-edit-bon-de-reception',
  templateUrl: './edit-bon-de-reception.component.html',
  styleUrls: ['./edit-bon-de-reception.component.scss'],
  providers: [
    {provide: DateAdapter, useClass: MomentDateAdapter, deps: [MAT_DATE_LOCALE]},
    {provide: MAT_DATE_FORMATS, useValue: CUSTOM_DATE_FORMATS}
  ]
})
export class EditBonDeReceptionComponent implements OnInit {

  @ViewChild('prixUnitInput') unitPrixInputEl: ElementRef | undefined;
  @ViewChild('quantiteInput') quantiteInputEl: ElementRef | undefined;
  @ViewChild('stockLocationInput') stockLocationInputEl: MatSelect | undefined;
  @ViewChild('rechercheRefInput') rechercheRefInputEl: ElementRef | undefined;

  bonDocumentForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],
    RECEPTION_DATE: [{ value: "", disabled: true }],
    AXESITE: [''],
    DOCUMENT_ASSOC: [''],

    LIGNE_INFO: [''], // Ligne ID
    LIGNE_CUSTOM_LABEL: [''], // Ligne Label

    SEARCH_STOCK_LOCATION: [""],
    SEARCH_REF: [""]
  })

  bonReceptionItemForm: FormGroup = this.formBuilder.group({
    STOCK_SOURCE_ID: ["", Validators.required],
    STOCK_SOURCE_LBL: [{ value: "", disabled: true }, Validators.required],
    ARTICLE_ID: ["", Validators.required],
    REFERENCE: [{ value: "", disabled: true }, Validators.required],
    LABEL: [{ value: "", disabled: true }, Validators.required],
    QTE: [{ value: "0", disabled: true }, [Validators.required, Validators.min(1)]],
    PRIX_UNIT: ['', [Validators.required, ValidateGreaterThan]],
    PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
    AVAIL_STOCK: [{ value: "0", disabled: true }],
    MATERIAL_ACTIVABLE_STATUS: [null],
    SERIALS_LIST: ['', Validators.required],
    ITEM_BRAND: ['']
  });

  newArticleSelection: Article | undefined;

  bonDocumentNum: string | undefined = "*********";
  bonReception: BonReception | undefined;
  bonReceptionItems: BonReceptionItem[] = [];
  bonReceptionArticles: Article[] = [];
  brandList: BrandInfoList[] = [];
  totalArticles: number = 0;
  totalQte: number = 0;
  totalPT: number = 0;
  newLineItemInsertable: boolean = true;
  ligneList: LigneInfoList[] = [];
  stockLocationList: StockLocationList[] = [];

  fullKeywordMatch: boolean = false;
  isKeywordSerial: boolean = false;
  searchRefArticlesList: Article[] | undefined = [];

  displayedColumns: string[] = [ 'BRAND', 'REFERENCE', 'LABEL', 'STOCK_SOURCE', 'QTE', 'AVAIL_STOCK', 'ACTIVE_STATUS', 'ACTIONS'];

  chosenCustomer: BonReceptionCustomer | undefined;
  customLigneInput: boolean = false;

  // Used to decide whether the values can be editable or not
  readOnly: boolean = false;

  pageTitle: string = "KMODIFY_BON_DE_RECEPTION";

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  //PRIX_UNIT AND PRIX_TOTAL Permission
  PU_PT_Permission: boolean = false;

  chosenSelectionArticle: BonReceptionItem[] = [];

  rolePermission:any;

  constructor(public dialog: MatDialog,
              private commonService: CommonService,
              private receptionService: BonDeReceptionService,
              private datePipe: DatePipe,
              private router: Router,
              private route: ActivatedRoute,
              private formBuilder: FormBuilder,
              private dataSharingService: DataSharingService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService) {
  }

  ngOnInit(): void {
    this.loadDependencies();

    this.bonDocumentForm.get("LIGNE_INFO")?.valueChanges.subscribe(
      (res) => {
        if(res === 'custom-ligne') {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue("");
          this.customLigneInput = true;
        } else {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(res.LGILBL);
          this.customLigneInput = false;
        }
      }
    );


    this.bonDocumentForm.get("SEARCH_REF")?.valueChanges.subscribe(
      (value) => {
        if(value === "" || value === null) {
          this.searchRefArticlesList = undefined;
        }
      }
    );
  }

  async loadDependencies() {
    await this.loadingService.show();

    firstValueFrom(this.route.data).then(
      (data: any) => {
        if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
          this.readOnly = data.readOnly;
          if(this.readOnly) {
            this.__changeToDetailMode();
          }
        }
      }
    );

    this.bonDocumentForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)
    });
    this._getRolePermission();
    await this._getLigneList();
    await this._getBrandList();
    await this._getStockLocationList();

    firstValueFrom(this.route.params).then(
      (params) => {
        if(params.hasOwnProperty('document_no') && params["document_no"] !== null && params["document_no"] !== "") {
          this.bonDocumentNum = params["document_no"];

          this._getBonReceptionDetail(this.bonDocumentNum);
        } else {
          // If the bon reception not available
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KRECEPTION_INVALID_DOCUMENT'
          }).then(
            () => {
              this.router.navigate(['/bon-de-reception/list']);
            }
          );
        }
      }
    )

    await this.loadingService.hide();
  }

  _getRolePermission(){
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.PU_PT_Permission = this.rolePermission['11_1'];
    if(this.PU_PT_Permission){
      this.displayedColumns = [ 'BRAND', 'REFERENCE', 'LABEL', 'STOCK_SOURCE', 'QTE', 'PRIX_UNIT', 'PRIX_TOTAL', 'AVAIL_STOCK', 'ACTIVE_STATUS', 'ACTIONS'];
    }else{
      this.bonReceptionItemForm.get('PRIX_UNIT')?.clearValidators();
    }
  }

  get __isDocumentValidated() {
    return this.bonReception && this.bonReception.status === APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES;
  }

  get material_activable_status() {
    return this.newArticleSelection?.material_activable_status;
  }

  __changeToDetailMode() {
    this.pageTitle = "KDETAIL_BON_DE_RECEPTION";

    // For Selection button
    if(this.__isDocumentValidated) {
      this.displayedColumns.unshift('ITEM_SELECTION');
    }

    this.bonDocumentForm.get('RECEPTION_DATE')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('AXESITE')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('DOCUMENT_ASSOC')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('LIGNE_INFO')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('LIGNE_CUSTOM_LABEL')?.disable({ onlySelf: true });
  }

  __selectionItemExists(row: BonReceptionItem) {
    if(this.chosenSelectionArticle === undefined || this.chosenSelectionArticle === null) {
      return false;
    }

    return this.chosenSelectionArticle.find(r => r.item_id == row.item_id);
  }

  onArticleSelectionChange(e: MatCheckboxChange, row: BonReceptionItem) {
    if(e.checked) {
      const itemExists = this.__selectionItemExists(row);
      if(!itemExists) {
        this.chosenSelectionArticle.push(row);
      }
    } else {
      this.chosenSelectionArticle = this.chosenSelectionArticle.filter(r => r.item_id != row.item_id);
    }
  }

  async openUpSelectionsPopup(): Promise<any> {
    return new Promise<any>((resolve) => {
      const dialog = this.dialog.open(SelectionCopyTypeComponent, {
        data: {
          documentType: 'BR'
        },
        width: '400px',
        disableClose: true,
      });

      dialog.afterClosed().subscribe((res) => {
        resolve(res);
      })
    })
  }

  onSelectionItemsClick() {
    if(!this.readOnly) {
      return;
    }

    if(this.chosenSelectionArticle === undefined || this.chosenSelectionArticle === null || this.chosenSelectionArticle.length === 0) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ITEMS_CHOSE_TO_COPY'
      });

      return;
    }

    this.openUpSelectionsPopup().then((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        const type = res.data;
        this.dataSharingService.selectionItems.next({
          type: type,
          items: this.chosenSelectionArticle,
          extras: {
            document_type: "BR",
            document_no: this.bonDocumentNum || '',
            documentDetail: this.bonReception
          }
        });

        let route
        if(type === APP_CONSTANTS.SELECTION_BR_TO_BL) {
          route = '/bon-de-livraison/create';
        } else if(type === APP_CONSTANTS.SELECTION_BR_TO_TR) {
          if(this.chosenSelectionArticle.length > 1) {
            // Only if more than 1 items chosen
            let allItemLocationsAreSame = true;
            let tmpItem;
            for(let j=0; j < this.chosenSelectionArticle.length; j++) {
              if(tmpItem === undefined) {
                tmpItem = this.chosenSelectionArticle[j];
              } else {
                if(tmpItem !== this.chosenSelectionArticle[j]) {
                  allItemLocationsAreSame = false;
                  break;
                }
              }
            }

            if(!allItemLocationsAreSame) {
              this.dialogService.displayAlert({
                title: 'KERROR',
                message: 'KRECEPTION_ITEMS_SAME_LOCATION_TO_COPY_TR'
              });

              return;
            }
          }

          route = '/transfer-de-stock/create';
        }

        if(route) {
          this.router.navigate([route]);
        }
      }
    });
  }

  async onCloseDocument() {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    if(this.isBonReceptionModifiable && !this.readOnly && (this.chosenCustomer === undefined || this.chosenCustomer === null)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KRECEPTION_CLIENT_SUPPLIER_NOT_ASSIGNED'
      })

      return;
    }

    this.returnBack();
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_RECEPTION_LIST);
    if(listCache !== undefined) {
      let listLivraison = listCache.data as ListReception[];
      listLivraison = listLivraison.map((row) => {
        if(row.document_no == payload.document_no) {
          row.achatcommande_document_no = payload.achatcommande_document_no;
          row.achatcommande_id = payload.achatcommande_id;
          row.axesite = payload.axesite;
          row.created_at = payload.created_at;
          row.dated = payload.dated;
          row.docs_assoc = payload.docs_assoc;
          row.document_no = payload.document_no;
          row.ligne_id = payload.ligne_id;
          row.ligne_lbl = payload.ligne_lbl;
          row.modified_at = payload.modified_at;
          row.reception_date = payload.reception_date;
          row.reception_id = payload.reception_id;
          row.status = payload.status;
          row.store_id = payload.customer.store_id;
          row.store_name = payload.customer.store_nom;
          row.store_type = payload.customer.store_type;
          row.total_price = payload.total_price;
        }
        return row;
      });
      listCache.data = listLivraison;
      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST, JSON.stringify(listCache));
    }
  }

  __deleteUpdateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_RECEPTION_LIST);
    if(listCache !== undefined) {
      let listReception: any[] = listCache.data;
      listReception = (listReception as ListReception[]).filter(r => r.document_no != id);

      listCache.data = listReception;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST, JSON.stringify(listCache));

      this.__updateListCache(listCache.data);
    }
}

  async onSubmitDocument(type: 'ok' | 'validate' = 'ok') {
    if(this.readOnly) {
      return;
    }

    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    if(type === 'validate'){
      if(!this.rolePermission['2_25']) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KACCESS_DENIED'
        })
        return;
      }
    }

    if(!this.isBonReceptionModifiable) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ARTICLE_ADDED'
      })

      return;
    }

    if(this.chosenCustomer === undefined || this.chosenCustomer === null) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KRECEPTION_CLIENT_SUPPLIER_NOT_ASSIGNED'
      })

      return;
    }

    const documentFormData = this.bonDocumentForm.getRawValue();

    const bonArticles: BonReceptionItem[] = [...this.bonReceptionItems];
    const bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    const bonStatus = type === 'validate' ? APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES : (this.bonReception?.status ? this.bonReception?.status : APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO);

    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonReception = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,

      customer: bonCustomer,
      articles: bonArticles,

      total_price: bonTotalPrix,
      status: bonStatus,
    };

    this._modifyReception(this.bonDocumentNum, bonDocument).then(
      () => {
        this.returnBack(type === 'validate' ? 'consultation' : 'list');
      }
    )
  }

  generateReceptionPdf(print?: boolean) {
    if(!this.isBonReceptionModifiable || (this.bonDocumentNum === undefined)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KRECEPTION_NO_CONTENT'
      })
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.receptionService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      }
    });
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_BON_DE_RECEPTION_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  returnBack(type: 'list' | 'consultation' = 'list') {
    if(type === 'list') {
      this.router.navigate(['/bon-de-reception/list']);
    } else if (type === 'consultation') {
      this.router.navigate(['/bon-de-reception/detail', this.bonDocumentNum]);
    }
  }

  _getLigneList(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.commonService.getAllLigneList().subscribe({
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  _getBrandList(): Promise<any> { // get both Client and Supplier Brands
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  _getStockLocationList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.commonService.getAllStockList().subscribe({
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.stockLocationList = res.data;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  _searchArticles(data: {keyword: string, location_id: number }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      let searchObs: Observable<CustomHttpResponse<SearchArticle>> | undefined;

      searchObs = this.commonService.searchArticlesListByRef(data);

      if(searchObs === undefined) {
        return;
      }

      searchObs.subscribe({
        next: (res: CustomHttpResponse<SearchArticle>) => {
          if(res.status === 'success') {
            this.fullKeywordMatch = res.data.fullMatch || false;
            this.isKeywordSerial = res.data.isSerial || false;
            this.searchRefArticlesList = res.data.rows;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  _getBonReceptionDetail(document_no: string | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined || document_no === null || document_no === '') {
        reject();
        return;
      }

      this.receptionService.getReceptionDetail(document_no).subscribe( {
        next: (res: CustomHttpResponse<BonReception>) => {
          if(res.status === "success") {
            const bonReceptionData = res.data;
            if(bonReceptionData.linked_document_type == "AC"){
              this.newLineItemInsertable = false;
            }
            this.bonReception = bonReceptionData;
            if(!this.rolePermission['2_26'] && this.bonReception.status === APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES) {
              this.dialogService.displayAlert({
                title: this.translateService.instant('KERROR'),
                message: this.translateService.instant('KACCESS_DENIED')
              }).then(() => {
                this.returnBack();
              });

              return;
            }
            this.__refreshBonDetails(bonReceptionData);
          } else {
            // If the bon reception not available
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/bon-de-reception/list']);
              }
            );
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  __refreshBonDetails(bonReception: BonReception) {
    if(bonReception !== undefined && bonReception !== null) {
      this.bonDocumentNum = bonReception.document_no;
      this.bonReceptionItems = bonReception.articles;
      this.bonReceptionArticles = bonReception?.custom_articles ? bonReception?.custom_articles : [];

      this.chosenCustomer = bonReception.customer;

      this.totalArticles = 0;
      this.totalQte = 0;
      this.totalPT = 0;

      this.bonReceptionItems.forEach((row) => {
        this.totalArticles += 1;
        this.totalQte += +row.qte;
        this.totalPT += +row.total_price;
      })

      this.bonDocumentForm.get("NUMBON")?.setValue(bonReception.document_no);
      this.bonDocumentForm.get("DATE_DU_BON")?.setValue(moment(bonReception.dated).format(CUSTOM_DATE_FORMAT));
      this.bonDocumentForm.get("RECEPTION_DATE")?.setValue((bonReception.reception_date !== undefined && bonReception.reception_date !== null) ? moment(bonReception.reception_date) : '');
      this.bonDocumentForm.get("AXESITE")?.setValue(bonReception.axesite);
      this.bonDocumentForm.get("DOCUMENT_ASSOC")?.setValue(bonReception.docs_assoc);

      if(bonReception.ligne_lbl !== undefined && bonReception.ligne_lbl !== null && bonReception.ligne_lbl !== "") {
        if(bonReception.ligne_id === undefined || bonReception.ligne_id === null || bonReception.ligne_id === "") {
          // Custom Ligne Input
          this.customLigneInput = true;
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue('custom-ligne');
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonReception.ligne_lbl);
        } else {
          // Chosen Ligne Input from the dropdown
          const currentLigneItem = this.ligneList.find(r => r.LGIID == bonReception.ligne_id)
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue(currentLigneItem);
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonReception.ligne_lbl);
        }
      }

      if(this.readOnly) {
        this.__changeToDetailMode();
      }
    }
  }

  __serialsListDisplay(serialsArr: string[]) {
    return serialsArr.join(", ");
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  get documentAssocie() {
    return this.bonDocumentForm.get("DOCUMENT_ASSOC");
  }

  get isBonReceptionModifiable() {
    return this.bonReception !== undefined;
  }

  get QTE_VALUE(){
    return this.bonReceptionItemForm.get('QTE')?.value;
  }

  addBonReceptionItemFormSubmitter() {
    if(this.bonReceptionItemForm.invalid || this.QTE_VALUE === '0') {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }

    this.loadingService.show();
    const documentFormData = this.bonDocumentForm.getRawValue();
    const itemFormData = this.bonReceptionItemForm.getRawValue();

    let bonArticles: BonReceptionItem[] = [];

    if(this.isBonReceptionModifiable) {
      bonArticles = [...this.bonReceptionItems];
    }

    const itemSerials = itemFormData.SERIALS_LIST.split(',');

    bonArticles.push({
      stock_source: +itemFormData.STOCK_SOURCE_ID,
      article_id: +itemFormData.ARTICLE_ID,
      article_reference: itemFormData.REFERENCE,
      article_desc: itemFormData.LABEL,
      qte: +itemFormData.QTE,
      qte_stock: +itemFormData.AVAIL_STOCK,
      unit_price: +itemFormData.PRIX_UNIT,
      total_price: +itemFormData.PRIX_TOTAL,
      material_activable_status: itemFormData.MATERIAL_ACTIVABLE_STATUS,
      article_serials: itemSerials,
      brand_id: itemFormData.ITEM_BRAND
    })

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
    let bonStatus = this.bonReception?.status ? this.bonReception?.status : APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO;

    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonReception = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,

      customer: bonCustomer,
      articles: bonArticles,

      total_price: bonTotalPrix,
      status: bonStatus,
    };

    this._modifyReception(this.bonDocumentNum, bonDocument).then(
      () => {
        this.__resetNewLineItemForm();
      }
    )
  }

  removeLineItem(currentItem: BonReceptionItem) {
    if(this.readOnly) {
      return;
    }

    this.loadingService.show();

    let bonArticles: BonReceptionItem[] = [...this.bonReceptionItems];
    bonArticles = bonArticles.filter((row) => row.item_id !== currentItem.item_id);

    const documentFormData = this.bonDocumentForm.getRawValue();

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    const bonStatus = this.bonReception?.status ? this.bonReception?.status : APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO;

    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    let bonDocument: BonReception = {
      ...this.bonReception,

      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',

      customer: bonCustomer,

      articles: bonArticles,
      total_price: bonTotalPrix,
      status: bonStatus
    };

    delete bonDocument.reception_id;

    this._modifyReception(this.bonDocumentNum, bonDocument);
  }

  _modifyReception(document_no: string | undefined, bonDocument: BonReception): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined) {
        reject("INVALID_DOCUMENT_NO");
        return;
      }

      this.receptionService.modifyReception(document_no, bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonReception>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonReceptionData = res.data;
            this.bonReception = bonReceptionData;
            this.__updateListCache(bonReceptionData);
            this.__refreshBonDetails(bonReceptionData);
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  __resetNewLineItemForm() {
    this.newArticleSelection = undefined;
    this.bonReceptionItemForm.patchValue({
      STOCK_SOURCE_ID: "",
      STOCK_SOURCE_LBL: "",
      ARTICLE_ID: "",
      REFERENCE: "",
      LABEL: "",
      QTE: "0",
      PRIX_UNIT: "",
      PRIX_TOTAL: "0",
      AVAIL_STOCK: "",
      MATERIAL_ACTIVABLE_STATUS: null,
      SERIALS_LIST: "",
      ITEM_BRAND: ""
    })

    this.searchRefArticlesList = undefined;

    this.bonDocumentForm.patchValue({
      SEARCH_STOCK_LOCATION: "",
      SEARCH_REF: ""
    });
  }

  onStockLocationChange(event: any) {
    this.stockLocationInputEl?.close();

    if(this.newArticleSelection) {
      // Changing the Stock location when already an article selected
      const referenceValue = this.bonDocumentForm.get("SEARCH_REF")?.value;
      // const labelValue = this.bonDocumentForm.get("SEARCH_LBL")?.value;

      if(referenceValue !== '') {
        this.searchArticles(undefined).then(
          (res) => {
            const currentArticleMatch = this.searchRefArticlesList?.find((r) => r.id === this.newArticleSelection?.id);
            if(currentArticleMatch) {
              this.newArticleSelection = currentArticleMatch;

              this.__fillNewLineItemForm('existing');
            }
          }
        );
      }
    } else {
      // Changing the Stock location at first
      this.rechercheRefInputEl?.nativeElement.focus();

      this.searchRefArticlesList = undefined;

      // Changing the Stock location when already an article selected
      const referenceValue = this.bonDocumentForm.get("SEARCH_REF")?.value;

      if(referenceValue !== '') {
        this.searchArticles(undefined).then(
          (res) => {
            const currentArticleMatch = this.searchRefArticlesList?.find((r) => r.id === this.newArticleSelection?.id);
            if(currentArticleMatch) {
              this.newArticleSelection = currentArticleMatch;

              this.__fillNewLineItemForm('existing');
            }
          }
        );
      }
    }
  }

  async searchArticles(event: any | undefined): Promise<any> {
    return new Promise(async (resolve, reject) => {
      if(event !== undefined && (event.keyCode == 38 || event.keyCode == 40 || event.keyCode == 13)) {
        return;
      }

      const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
      if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
        this.searchRefArticlesList = undefined;

        this.stockLocationInputEl?.focus();
        this.stockLocationInputEl?.open();
        return;
      }

      let payload = {
        location_id: +searchStockLocation,
        keyword: ''
      };

      const searchRefKeyword = this.bonDocumentForm.get("SEARCH_REF")?.value;
      if(searchRefKeyword === '' || searchRefKeyword === null) return;
      payload = {
        ...payload,
        keyword: searchRefKeyword
      };

      const searchResult = await this._searchArticles(payload);

      resolve(searchResult);
    })
  }

  async onScannerSearchArticle() {
    const searchRef = this.bonDocumentForm.get('SEARCH_REF')?.value;
    if(searchRef == undefined || searchRef == "") {
      return;
    }

    await this.searchArticles(searchRef);

    if(this.searchRefArticlesList === undefined || this.searchRefArticlesList === null) {
      return;
    }

    if(this.searchRefArticlesList.length > 0 && this.fullKeywordMatch) {
      if(this.isKeywordSerial) {
        const firstArticle = this.searchRefArticlesList[0];
        this.searchSelected(firstArticle, true);
      } else {
        const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
        if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
          this.searchRefArticlesList = undefined;

          this.stockLocationInputEl?.focus();
          this.stockLocationInputEl?.open();
          return;
        }

        const firstArticle = this.searchRefArticlesList[0];
        this.searchSelected(firstArticle, true);
      }
    } else {
      const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
      if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
        this.searchRefArticlesList = undefined;

        this.stockLocationInputEl?.focus();
        this.stockLocationInputEl?.open();
        return;
      }

      this.openSearchArticlePopup();
    }
  }

  async openSearchArticlePopup() {
    const searchKeyword = this.bonDocumentForm.get('SEARCH_REF')?.value;
    const stockLocation = this.bonDocumentForm.get('SEARCH_STOCK_LOCATION')?.value;

    const data = {
      currentArticle: this.newArticleSelection,
      stockLocation: stockLocation,
      searchKeyword: searchKeyword
    };

    const dialog = this.dialog.open(SearchArticleComponent, {
      data,
      maxHeight: "90vh"
    });

    dialog.afterClosed().subscribe((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        this.newArticleSelection = res.data;

        this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);
        this.bonReceptionItemForm.get("SERIALS_LIST")?.setValue(this.newArticleSelection?.serial_list);
        console.log(this.bonReceptionItemForm.getRawValue());

        this.__fillNewLineItemForm('new');
        console.log('SEARCH RES: ', res);

        this.changeQuantity('creation');
      }
    });
  }

  searchSelected(event: any, directCall: boolean = false) {
    if(directCall) {
      this.newArticleSelection = event;
    } else {
      this.newArticleSelection = event.option.value;
    }

    this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);

    this.__fillNewLineItemForm('new');

    this.changeQuantity('creation');
  }

  __fillNewLineItemForm(type: 'new' | 'existing') {
    const stockLocationId = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
    const stockLocationDetail = this.stockLocationList.find((_r) => _r.STLID == stockLocationId);

    let unitPrix = 0;
    if(this.newArticleSelection?.public_prix !== undefined && this.newArticleSelection?.public_prix !== null) {
      unitPrix = this.newArticleSelection?.public_prix;
    }

    this.bonReceptionItemForm.patchValue({
      STOCK_SOURCE_ID: stockLocationDetail?.STLID,
      STOCK_SOURCE_LBL: stockLocationDetail?.STLLBL,
      ARTICLE_ID: this.newArticleSelection?.id,
      REFERENCE: this.newArticleSelection?.reference,
      LABEL: this.newArticleSelection?.label,
      AVAIL_STOCK: (this.newArticleSelection?.avail_stock === null ? 0 : +this.newArticleSelection?.avail_stock).toString(),
    });

    if(type === "new") {
      this.bonReceptionItemForm.patchValue({
        QTE: "0",
        PRIX_UNIT: unitPrix.toString(),
        PRIX_TOTAL: "0",
        SERIALS_LIST: ""
      });
    }
  }

  changeQuantity(type: 'creation' | 'modification') {
    let oldListSerials = [];

    const itemSerials = this.bonReceptionItemForm.get("SERIALS_LIST")?.value;
    if(itemSerials !== "") {
      oldListSerials = itemSerials.split(",");
    }

    oldListSerials = this.commonService.formatSerialNumbers(oldListSerials);

    let blockedInSerials = this.newArticleSelection?.blocked_in_serials;
    if(this.__isDocumentValidated) {
      const documentItems = cloneDeep(this.bonReceptionItems);
      const documentSerials = documentItems.map(r => {
        return r.article_serials; // ['SN1', 'SN2']
      }).reduce((acc, current) => {
        current = current.map(row => `${row}${APP_CONSTANTS.BLOCK_SERIALS_SEPARATOR}${this.bonDocumentNum}${APP_CONSTANTS.BLOCK_SERIALS_SEPARATOR}BR`)
        acc = [...acc, ...current];

        return acc;
      }, []).join(",");

      if(blockedInSerials !== undefined && blockedInSerials !== null && blockedInSerials !== "") {
        blockedInSerials += "," + documentSerials;
      } else {
        blockedInSerials = documentSerials;
      }
    }

    this.openUpSerialNumPopup({
      oldListSerials,
      block_in_serials: blockedInSerials,
    });
  }

  viewItemSerials(item: BonReceptionItem) {
    const oldListSerials = this.commonService.formatSerialNumbers(item.article_serials);

    this.openUpSerialNumPopup({
      oldListSerials,
      readonly: true
    });
  }

  openUpSerialNumPopup(data?: any) {
    data.numeroType = "reception-numero-series";

    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: data || {}
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        if(result.data && result.data !== '') {
          if(data.readonly === true) {
            return;
          }

          this.bonReceptionItemForm.get('QTE')?.setValue(result.data.length.toString());
          this.bonReceptionItemForm.get('SERIALS_LIST')?.setValue(result.data.join(','));

          this.unitPrixInputEl?.nativeElement.select();
          this.unitPrixInputEl?.nativeElement.focus();
          this.quantiteInputEl?.nativeElement.dispatchEvent(new Event('change'));
        }
      }
    });
  }

  confirmResetNewArticle(): Promise<any> {
    const message = `<h3>${this.translateService.instant('KCONFIRM_DISCARD_NEW_ITEM')}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async modifyLineItem(item: BonReceptionItem) {
    if(this.readOnly) {
      return;
    }

    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    const data = {
      item: item,
      numeroType: "reception-numero-series",
      document_no: this.bonDocumentNum,
      document: this.bonReception,
    };

    this.openUpModifyItemPopup(data);
  }

  openUpModifyItemPopup(data?: {}) {
    const dialogRef = this.dialog.open(ModifyItemComponent,
      {
        maxWidth: '40vw',
        data: data || {}
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        console.log(result.data);
        if(result.data && result.data !== '') {
          this.loadingService.show();
          const bonDocument = result.data;
          this._modifyReception(this.bonDocumentNum, bonDocument);
        }
      }
    });
  }

  onBonDocumentAssocieChange(documentAss: string) {
    if(this.documentAssocie?.value !== documentAss) {
      this.documentAssocie?.setValue(documentAss);
    }
  }

  onBonCustomerChange(customer: BonReceptionCustomer) {
    if(this.chosenCustomer !== customer) {
      this.chosenCustomer = customer;

      if(this.isBonReceptionModifiable) {
        this._updateBonCustomer();
      }
    }
  }

  _updateBonCustomer() {
    const bonCustomer: BonReceptionCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : ""
    };

    const documentFormData = this.bonDocumentForm.getRawValue();
    const bonStatus = this.bonReception?.status ? this.bonReception?.status : APP_CONSTANTS.RECEPTION_VALIDATION_STAT_NO;
    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonReception = {
      ...this.bonReception,

      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      reception_date: (documentFormData.RECEPTION_DATE !== undefined && documentFormData.RECEPTION_DATE !== null && documentFormData.RECEPTION_DATE !== "" && documentFormData.RECEPTION_DATE.value !== "") ? moment(documentFormData.RECEPTION_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,

      customer: bonCustomer,
      articles: this.bonReceptionItems,

      total_price: this.bonReception?.total_price ? this.bonReception?.total_price : 0,
      status: bonStatus,
    };

    this._modifyReception(this.bonDocumentNum, bonDocument);
  }

  calculatePrixTotal(){
    const formData = this.bonReceptionItemForm.getRawValue();

    let totalPrix = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      this.bonReceptionItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      this.bonReceptionItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    this.bonReceptionItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
  }

  openHistorique(){
    const formData = this.bonDocumentForm.getRawValue();
    const data = {
      document_no: formData.NUMBON,
      document_type: APP_CONSTANTS.DOCUMENT_TYPE_RECEPTION
    };
    this.dialog.open(HistoriqueComponent,
      {
        minWidth: '80%',
        maxHeight: '90vh',
        data: data || {}
      }
    );
  }

  generateBonDeLivraison() {
    if(!this.readOnly) {
      return;
    }

    this.router.navigate(['/bon-de-livraison/create'], { queryParams: { _doc: this.bonDocumentNum, _doc_t: 'BR' } });
  }

  confirmRemoveReception(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeReception() {
    if(this.bonDocumentNum == undefined){
      return
    }
    if((this.__isDocumentValidated && !this.rolePermission['2_27']) || (!this.__isDocumentValidated && !this.rolePermission['2_24'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveReception(this.bonDocumentNum);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteReception(this.bonDocumentNum);
    }
  }

  async _deleteReception(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__deleteUpdateListCache(document_no);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/bon-de-reception/list']);
          return;
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.receptionService.deleteReception(document_no).subscribe(observer);
  }

}
