import {Component, EventEmitter, Input, OnDestroy, OnInit, Output} from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import {NgxSpinnerService} from "ngx-spinner";
import {BrandInfoList, CustomHttpResponse} from "../../../_models";
import {CommonService} from "../../../_services/common.service";
import {DialogService} from "../../../_services/dailog.service";
import {ContactService} from "../../../_services/contact.service";
import {TranslateService} from "@ngx-translate/core";
import {Router} from "@angular/router";
import {debounceTime, distinctUntilChanged, Subscription} from "rxjs";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {DataSharingService} from "../../../_services/data-sharing.service";

@Component({
  selector: 'app-add-contact',
  templateUrl: './add-contact.component.html',
  styleUrls: ['./add-contact.component.scss']
})
export class AddContactComponent implements OnInit, OnDestroy {

  @Input() flowType: 'store-create' | 'default' = 'default';
  @Input() defaultFormData: any | undefined;

  @Output() onCreateContactFormChanges: EventEmitter<any> = new EventEmitter<any>();
  formChangeSubscription: Subscription | undefined;

  submitted: boolean = false;
  createContactForm: FormGroup = this.formBuilder.group({
    CTNOMDU: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.CONTACT_NOM_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.CONTACT_NOM_MAX_LENGTH)]],
    CTTEL: ['', [Validators.minLength(APP_CONSTANTS.CONTACT_TEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.CONTACT_TEL_MAX_LENGTH)]],
    CTMAIL: ['', [Validators.pattern(APP_CONSTANTS.EMAIL_PATTERN)]],
    CTPOSTE: ['', [Validators.minLength(APP_CONSTANTS.CONTACT_POSTE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.CONTACT_POSTE_MAX_LENGTH)]],
    CTSTID: ['', [Validators.required]],
    CTBRID: ['', [Validators.required]],
  });

  brandList: BrandInfoList[] = [];
  storeList: any = [];

  preferredCountries = APP_CONSTANTS.PREFERRED_TEL_COUNTRIES;

  constructor(private commonService: CommonService,
              private contactService: ContactService,
              private dataSharingService: DataSharingService,
              private formBuilder: FormBuilder,
              private translateService:TranslateService,
              private router: Router,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService) {
  }

  ngOnInit(): void {
    this.loadDependencies();

    this.formChangeSubscription = this.createContactForm.valueChanges.pipe(
      debounceTime(500),
      distinctUntilChanged()
    ).subscribe(() => {
      // whenever the form changes
      this.onCreateContactFormChanges.emit(this.createContactForm);
    })
  }

  ngOnDestroy() {
    if(this.formChangeSubscription) {
      this.formChangeSubscription.unsubscribe();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getBrandList();

    if(this.flowType === 'store-create') {
      if(this.defaultFormData !== undefined && this.defaultFormData !== null) {
        this.createContactForm.get('CTNOMDU')?.setValue(this.defaultFormData.CTNOMDU);
        this.createContactForm.get('CTTEL')?.setValue(this.defaultFormData.CTTEL);
        this.createContactForm.get('CTMAIL')?.setValue(this.defaultFormData.CTMAIL);
        this.createContactForm.get('CTPOSTE')?.setValue(this.defaultFormData.CTPOSTE);
        this.createContactForm.get('CTBRID')?.setValue(this.defaultFormData.CTBRID);
        this.createContactForm.get('CTBRID')?.disable({ onlySelf: true });
        this.createContactForm.get('CTSTID')?.disable({ onlySelf: true });
        this.createContactForm.updateValueAndValidity();
      }
    }

    await this.loadingService.hide();
  }

  get fr() {
    return this.createContactForm.controls;
  }

  getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  getStoreListByBrand(brand_id: number): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const payload = {
        brand_id
      };

      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.storeList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.contactService.getStoreListByBrand(payload).subscribe(observer);
    });
  }

  createContactFormSubmitter() {
    this.submitted = true;
    if(this.createContactForm.invalid){
      return;
    }

    this.createContact();
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST_FORM);
  }

  createContact() {
    const formData = this.createContactForm.getRawValue();

    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.submitted = false;
          this.__resetListCache();
          this.router.navigate(['/contact/contact-list']);
        } else {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        console.log(err);
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }

    this.contactService.createContact(formData).subscribe(observer);
  }

}

