import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {CommonService} from "../../../_services/common.service";
import {CustomHttpResponse, StockLocationList} from "../../../_models";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {ContactService} from "../../../_services/contact.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {DataSharingService} from "../../../_services/data-sharing.service";

@Component({
  selector: 'app-create-brand',
  templateUrl: './create-brand.component.html',
  styleUrls: ['./create-brand.component.scss']
})
export class CreateBrandComponent implements OnInit {

  submitted: boolean = false;
  createBrandForm: FormGroup = this.formBuilder.group({
    BRLAB: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.BRAND_NOM_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.BRAND_NOM_MAX_LENGTH)]],
    BRTEL: ['', [Validators.minLength(APP_CONSTANTS.BRAND_TEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.BRAND_TEL_MAX_LENGTH)]],
    BRMAIL: ['', [Validators.pattern(APP_CONSTANTS.EMAIL_PATTERN)]],
    BRLOGO: ['', [Validators.required]],
    BRSTOCKID: ['', [Validators.required]],
    BRTYPE: ['', [Validators.required]],
    BRCONTRACT: ['', [Validators.maxLength(APP_CONSTANTS.BRAND_CONTRACT_MAX_LENGTH)]],
  });

  storageList: StockLocationList[] = [];
  uploadFile: any;

  preferredCountries = APP_CONSTANTS.PREFERRED_TEL_COUNTRIES;

  constructor(private contactService: ContactService,
              private dataSharingService: DataSharingService,
              private formBuilder: FormBuilder,
              private router: Router,
              private translateService: TranslateService,
              private commonService: CommonService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService) {
   }

  ngOnInit(): void {
    this.loadDependencies();
  }

  get fr() {
    return this.createBrandForm.controls;
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getStorageList();

    await this.loadingService.hide();
  }

  getStorageList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.storageList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getStockLocationList().subscribe( observer);
    })
  }

  createBrandFormSubmitter() {
    this.submitted = true;

    if(this.createBrandForm.invalid) {
      return;
    }

    const formData = this.createBrandForm.getRawValue();

    const payload = new FormData();
    payload.append("BRLAB", formData.BRLAB);
    payload.append("BRTEL", formData.BRTEL);
    payload.append("BRMAIL", formData.BRMAIL);
    payload.append("BRLOGO", this.uploadFile);
    payload.append("BRSTOCKID", formData.BRSTOCKID);
    payload.append("BRTYPE", formData.BRTYPE);
    payload.append("BRCONTRACT", formData.BRCONTRACT);

    this.createBrand(payload);
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST_FORM);
  }

  createBrand(payload: any) {
    this.loadingService.show();

    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.submitted = false;
          this.__resetListCache();

          const brandId = res.data.brand_id;

          this.router.navigate(['/contact/store/create'], { queryParams: { brand_id: brandId } });
        } else {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }

    this.contactService.createBrand(payload).subscribe(observer);
  }

  uploadLogo(fileInput: any) {
    if (fileInput.target.files && fileInput.target.files[0]) {
      const file = fileInput.target.files[0];
      this.uploadFile = file;

      this.createBrandForm.get('BRLOGO')?.setValue(file.name, { onlySelf: true });
      this.createBrandForm.updateValueAndValidity();
    } else {
      this.uploadFile = undefined;
    }
  }

}
