import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {CustomHttpResponse, StockLocationList} from "../../../_models";
import {Brand, BrandList} from "../../../_models/brand";
import {ContactService} from "../../../_services/contact.service";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {CommonService} from "../../../_services/common.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {DataSharingService} from "../../../_services/data-sharing.service";

@Component({
  selector: 'app-edit-brand',
  templateUrl: './edit-brand.component.html',
  styleUrls: ['./edit-brand.component.scss']
})
export class EditBrandComponent implements OnInit {

  submitted = false;
  modifyBrandForm: FormGroup = this.formBuilder.group({
    BRLAB: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.BRAND_NOM_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.BRAND_NOM_MAX_LENGTH)]],
    BRTEL: ['', [Validators.minLength(APP_CONSTANTS.BRAND_TEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.BRAND_TEL_MAX_LENGTH)]],
    BRMAIL: ['', [Validators.pattern(APP_CONSTANTS.EMAIL_PATTERN)]],
    BRLOGO: ['', [Validators.required]],
    BRSTOCKID: ['', [Validators.required]],
    BRTYPE: ['', [Validators.required]],
    BRCONTRACT: ['', [Validators.maxLength(APP_CONSTANTS.BRAND_CONTRACT_MAX_LENGTH)]],
  });

  storageList: StockLocationList[] = [];
  uploadFile: any;

  brandId: number | undefined;
  brandDetail: Brand | undefined;
  imgURL: string | undefined;

  preferredCountries = APP_CONSTANTS.PREFERRED_TEL_COUNTRIES;

  constructor(private commonService: CommonService,
              private dataSharingService: DataSharingService,
              private contactService: ContactService,
              private formBuilder: FormBuilder,
              private route: ActivatedRoute,
              private router: Router,
              private translateService:TranslateService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService) {
   }

  ngOnInit(): void {
    this.route.params.subscribe(params => {
      this.brandId = +params["brand_id"];

      this.loadDependencies();
    });
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getStorageList();
    await this.getBrandDetail(this.brandId);
    this._preFillBrandDetails(this.brandDetail);

    await this.loadingService.hide();
  }

  getBrandDetail(id: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const payload = {
        brand_id: id
      };

      const observer = {
        next: (res: CustomHttpResponse<Brand>) => {
          if(res.status === 'success') {
            this.imgURL = res.imageurl;
            this.brandDetail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.contactService.getBrandDetail(payload).subscribe(observer);
    })
  }

  _preFillBrandDetails(brandDetail?: Brand) {
    if(brandDetail === undefined || brandDetail === null) {
      return;
    }

    this.modifyBrandForm.get('BRLAB')?.setValue(brandDetail.brand_lbl);
    this.modifyBrandForm.get('BRTEL')?.setValue(brandDetail.brand_tel || '');
    this.modifyBrandForm.get('BRMAIL')?.setValue(brandDetail.brand_mail || '');
    this.modifyBrandForm.get('BRTYPE')?.setValue(brandDetail.brand_type);
    this.modifyBrandForm.get('BRLOGO')?.setValue(brandDetail.brand_logo);
    this.modifyBrandForm.get('BRCONTRACT')?.setValue(brandDetail.brand_contract);
    this.modifyBrandForm.get('BRSTOCKID')?.setValue(brandDetail.stock_location_id);
  }

  get fr() {
    return this.modifyBrandForm.controls;
  }

  getStorageList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.storageList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getStockLocationList().subscribe( observer);
    })
  }

  uploadLogo(fileInput:any) {
    if (fileInput.target.files && fileInput.target.files[0]) {
      this.uploadFile = fileInput.target.files[0];

      this.modifyBrandForm.get('BRLOGO')?.setValue(this.uploadFile.name, { onlySelf: true });
      this.modifyBrandForm.updateValueAndValidity();
    } else {
      this.uploadFile = undefined;
    }
  }

  modifyBrandFormSubmitter() {
    this.submitted = true;
    if(this.modifyBrandForm.invalid){
      return;
    }

    if(this.brandId === undefined) {
      return;
    }

    const formData = this.modifyBrandForm.getRawValue();

    const payload = new FormData();
    payload.append("BRID", this.brandId.toString());
    payload.append("BRLAB", formData.BRLAB);
    payload.append("BRTEL", formData.BRTEL);
    payload.append("BRMAIL", formData.BRMAIL);
    payload.append("BRLOGO", this.uploadFile);
    payload.append("BRSTOCKID", formData.BRSTOCKID);
    payload.append("BRTYPE", formData.BRTYPE);
    payload.append("BRCONTRACT", formData.BRCONTRACT);

    this.updateBrand(payload);
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
    if(listCache !== undefined) {
      let listContacts = listCache.data as BrandList[];
      listContacts = listContacts.map((row) => {
        if(row.brand_id == payload.get('BRID')) {
          row.brand_lbl = payload.get('BRLAB');
          row.brand_email = payload.get('BRMAIL');
          row.brand_phone = payload.get('BRTEL');
          // TODO: Brand Logo updation
          row.brand_type = payload.get('BRTYPE');
        }

        return row;
      });

      listCache.data = listContacts;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST, JSON.stringify(listCache));
    }
  }

  updateBrand(data: any) {
    this.loadingService.show();

    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status == 'success') {
          this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          }).then(() => {
            this.modifyBrandForm.reset();
            this.modifyBrandForm.get('BRTEL')?.setValue('');

            this.submitted = false;
            this.__updateListCache(data);
            this.router.navigate(['/contact/contact-list']);
          });
        } else {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.contactService.updateBrand(data).subscribe(observer);
  }

}
