import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {BrandInfoList, Contact, ContactList, CustomHttpResponse} from "../../../_models";
import {ContactService} from "../../../_services/contact.service";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {CommonService} from "../../../_services/common.service";
import {DataSharingService} from "../../../_services/data-sharing.service";

@Component({
  selector: 'app-edit-contact',
  templateUrl: './edit-contact.component.html',
  styleUrls: ['./edit-contact.component.scss']
})
export class EditContactComponent implements OnInit {

  submitted: boolean = false;
  modifyContactForm: FormGroup = this.formBuilder.group({
    CTID: ['', Validators.required],
    CTNOMDU: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.CONTACT_NOM_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.CONTACT_NOM_MAX_LENGTH)]],
    CTTEL: ['', [Validators.minLength(APP_CONSTANTS.CONTACT_TEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.CONTACT_TEL_MAX_LENGTH)]],
    CTMAIL: ['', [Validators.pattern(APP_CONSTANTS.EMAIL_PATTERN)]],
    CTPOSTE: ['', [Validators.minLength(APP_CONSTANTS.CONTACT_POSTE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.CONTACT_POSTE_MAX_LENGTH)]],
    CTSTID: ['', [Validators.required]],
    CTBRID: ['', [Validators.required]],
  });

  brandList: BrandInfoList[] = [];
  storeList: any = [];

  contact_id: number | undefined;
  contactDetail: Contact | undefined;

  preferredCountries = APP_CONSTANTS.PREFERRED_TEL_COUNTRIES;

  constructor(private contactService: ContactService,
              private commonService: CommonService,
              private dataSharingService: DataSharingService,
              private formBuilder: FormBuilder,
              private route: ActivatedRoute,
              private router: Router,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private translateService: TranslateService) {
   }

   ngOnInit(): void {
    this.getBrandList();
    this.route.params.subscribe(params => {
      this.contact_id = params["contact_id"];

      this.modifyContactForm.get('CTID')?.setValue(this.contact_id);
      this.loadDependencies();
    })
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getBrandList();
    await this.getContactDetail(this.contact_id);
    await this.getStoresByBrand(this.contactDetail?.CTBRID);

    this._preFillContactDetails(this.contactDetail);

    await this.loadingService.hide();
  }

  get fr() {
    return this.modifyContactForm.controls;
  }

  getContactDetail(id: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const payload = {
        CTID: id
      };

      const observer = {
        next: (res: CustomHttpResponse<Contact>) => {
          if(res.status === 'success') {
            this.contactDetail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.contactService.getContactDetail(payload).subscribe(observer);
    })
  }

  _preFillContactDetails(contactDetail?: Contact) {
    if(contactDetail === undefined || contactDetail === null) {
      return;
    }

    this.modifyContactForm.get('CTNOMDU')?.setValue(contactDetail.CTNOMDU);
    this.modifyContactForm.get('CTBRID')?.setValue(contactDetail.CTBRID);
    this.modifyContactForm.get('CTPOSTE')?.setValue(contactDetail.CTPOSTE);
    this.modifyContactForm.get('CTMAIL')?.setValue(contactDetail.CTMAIL);
    this.modifyContactForm.get('CTTEL')?.setValue(contactDetail.CTTEL);

    if(contactDetail.BRTYPE === 'supplier') {
      this.modifyContactForm.get('CTSTID')?.setValue(contactDetail.CTSSTID);
    } else {
      this.modifyContactForm.get('CTSTID')?.setValue(contactDetail.CTCSTID);
    }
  }

  getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  getStoresByBrand(brand?: number): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const brand_id = brand || this.modifyContactForm.get('CTBRID')?.value;
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.storeList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.contactService.getStoreListByBrand({ brand_id }).subscribe(observer);
    });
  }

  modifyContactFormSubmitter() {
    this.submitted = true;
    if(this.modifyContactForm.invalid){
      return;
    }

    this.updateContact();
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
    if(listCache !== undefined) {
      let listContacts = listCache.data as ContactList[];
      listContacts = listContacts.map((row) => {
        if(row.contact_id == payload.CTID) {
          row.contact_name = payload.CTNOMDU;
          row.contact_email = payload.CTMAIL;
          row.contact_phone = payload.CTTEL;
        }

        return row;
      });

      listCache.data = listContacts;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST, JSON.stringify(listCache));
    }
  }

  updateContact() {
    const formData = this.modifyContactForm.getRawValue();

    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        if(res.status === 'success') {
          this.submitted = false;
          this.__updateListCache(formData);
          this.router.navigate(['/contact/contact-list']);
        } else {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        console.log(err);
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }

    this.contactService.updateContact(formData).subscribe(observer);
  }

}
