import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {
  ArticlesList,
  CustomHttpResponse,
  Store,
  StoreCategories,
  StoreClientCategories, StoreList,
  StoreSupplierCategories
} from "../../../_models";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {ContactService} from "../../../_services/contact.service";
import {CommonService} from "../../../_services/common.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {DataSharingService} from "../../../_services/data-sharing.service";

@Component({
  selector: 'app-edit-store',
  templateUrl: './edit-store.component.html',
  styleUrls: ['./edit-store.component.scss']
})
export class EditStoreComponent implements OnInit {

  filteredVillesFacturation: any = [];
  filteredVillesExpedition: any = [];

  filteredPostalCodesFacturation: any = [];
  filteredPostalCodesExpedition: any = [];

  submitted: boolean = false;

  articleList: ArticlesList[] = [];
  paysList: any = [];
  defriementList: any = [];

  _filteredArticles: ArticlesList[] = [];
  _filterArticleKey: string = "";

  modifyStoreForm: FormGroup = this.formBuilder.group({
    store_brand: [{ value: '', disabled: true }, Validators.required],
    store_nom: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_NOM_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_NOM_MAX_LENGTH)]],
    store_category: ['', Validators.required],

    store_defriement: [''], // Only for clients

    store_delay_livraison: [''], // Only for supplier
    store_cout_commande: [''], // Only for supplier

    store_address: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_ADDRESS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_ADDRESS_MAX_LENGTH)]],
    store_code_postal: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_CP_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_CP_MAX_LENGTH)]],
    store_pays: ['FR', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_PAYS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_PAYS_MAX_LENGTH)]],
    store_ville: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_VILLE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_VILLE_MAX_LENGTH)]],

    storefac_address: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_ADDRESS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_ADDRESS_MAX_LENGTH)]],
    storefac_code_postal: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_CP_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_CP_MAX_LENGTH)]],
    storefac_ville: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_PAYS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_PAYS_MAX_LENGTH)]],
    storefac_pays: ['FR', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_VILLE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_VILLE_MAX_LENGTH)]],
    store_articles: [[]],

    SAME_AS_SHIPPING: [false],
    store_headquarters: [false]
  });

  store_id: number | undefined;
  store_type: string | undefined;
  storeDetail: Store | undefined;
  imgURL: string | undefined;

  storeCategories: StoreCategories[] = [];

  sameAsShipping = false;

  constructor(private commonService: CommonService,
              private dataSharingService: DataSharingService,
              private contactService: ContactService,
              private formBuilder: FormBuilder,
              private route: ActivatedRoute,
              private router: Router,
              private translateService:TranslateService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService) {
  }

  ngOnInit(): void {
    this.route.params.subscribe(params => {
      this.store_id = +params["store_id"];
      this.store_type = params["store_type"];

      this.loadDependencies();
    });
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getPaysList();
    await this.getArticleList();
    await this.getDefriementList();

    await this.getStoreCategories(this.store_type);

    await this.getStoreDetail(this.store_id, this.store_type);
    this._preFillStoreDetails(this.storeDetail);
    this._checkBillingShippingSame();

    await this.loadingService.hide();
  }

  getPaysList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.paysList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListPays().subscribe(observer);
    });
  }

  getArticleList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticlesList[]>) => {
          if(res.status === 'success') {
            this.articleList = res.data;
            this._filteredArticles = this.articleList;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListArticles().subscribe(observer);
    });
  }

  getDefriementList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.defriementList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListDefriement().subscribe(observer);
    });
  }

  getStoreCategories(type: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<StoreSupplierCategories[] | StoreClientCategories[]>) => {
          if(res.status === 'success') {
            const storeCategories = res.data;

            if(type === 'client') {
              this.storeCategories = (storeCategories as StoreClientCategories[]).map((row: any) => {
                return { category_id: row.CCTID, category_label: row.CCTLBL };
              });
            }

            if(type === 'supplier') {
              this.storeCategories = (storeCategories as StoreSupplierCategories[]).map((row: any) => {
                return { category_id: row.SCTID, category_label: row.SCTLBL };
              });
            }

            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListStoreCategories(type).subscribe(observer);
    });
  }

  getStoreDetail(id: any, type: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const payload = {
        store_id: id,
        store_type: type
      };

      const observer = {
        next: (res: CustomHttpResponse<Store>) => {
          if(res.status === 'success') {
            this.imgURL = res.imageurl;
            this.storeDetail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.contactService.getStoreDetail(payload).subscribe(observer);
    })
  }

  _preFillStoreDetails(storeDetail?: Store) {
    if(storeDetail === undefined || storeDetail === null) {
      return;
    }

    let chosenArticles: any = storeDetail.article_list ? [...storeDetail.article_list] : [];
    chosenArticles = chosenArticles.map((row: any) => row.article_id);

    this.modifyStoreForm.get('store_brand')?.setValue(storeDetail.store_brand);
    this.modifyStoreForm.get('store_nom')?.setValue(storeDetail.store_nom);
    this.modifyStoreForm.get('store_category')?.setValue(+storeDetail.store_category_id);

    this.modifyStoreForm.get('store_address')?.setValue(storeDetail.store_address);
    this.modifyStoreForm.get('store_code_postal')?.setValue(storeDetail.store_code_postal);
    this.modifyStoreForm.get('store_pays')?.setValue(storeDetail.store_pays);
    this.modifyStoreForm.get('store_ville')?.setValue(storeDetail.store_ville);

    this.modifyStoreForm.get('storefac_address')?.setValue(storeDetail.storefac_address);
    this.modifyStoreForm.get('storefac_code_postal')?.setValue(storeDetail.storefac_code_postal);
    this.modifyStoreForm.get('storefac_ville')?.setValue(storeDetail.storefac_ville);
    this.modifyStoreForm.get('storefac_pays')?.setValue(storeDetail.storefac_pays);
    this.modifyStoreForm.get('store_headquarters')?.setValue(storeDetail.store_hq == APP_CONSTANTS.STORE_HQ_STAT_YES);

    this.modifyStoreForm.get('store_articles')?.setValue(chosenArticles);

    if(this.store_type === 'client') {
      this.modifyStoreForm.get('store_defriement')?.setValue(+storeDetail.defriement_id);
    }

    if(this.store_type === 'supplier') {
      this.modifyStoreForm.get('store_delay_livraison')?.setValue(storeDetail.store_de_livraison);
      this.modifyStoreForm.get('store_cout_commande')?.setValue(storeDetail.store_de_commande);
    }
  }

  modifyStoreFormSubmitter() {
    this.submitted = true;
    if(this.modifyStoreForm.invalid){
      return;
    }

    this.updateStore();
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
    if(listCache !== undefined) {
      let listContacts = listCache.data as StoreList[];
      listContacts = listContacts.map((row) => {
        if(row.store_id == this.store_id) {
          row.store_nom = payload.store_nom;
          row.store_category = payload.store_category;
          row.store_address = payload.store_address;
          row.store_code_postal = payload.store_code_postal;
          row.store_pays = payload.store_pays;
          row.store_ville = payload.store_ville;
          row.storefac_address = payload.storefac_address;
          row.storefac_code_postal = payload.storefac_code_postal;
          row.storefac_ville = payload.storefac_ville;
          row.storefac_pays = payload.storefac_pays;

          row.store_hq = payload.store_headquarters ? APP_CONSTANTS.STORE_HQ_STAT_YES : APP_CONSTANTS.STORE_HQ_STAT_NO;
        }

        return row;
      });

      listCache.data = listContacts;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST, JSON.stringify(listCache));
    }
  }

  updateStore() {
    this.submitted = true;

    this.modifyStoreForm.updateValueAndValidity();
    if(this.modifyStoreForm.invalid){
      return;
    }

    const formData = this.modifyStoreForm.getRawValue();

    let articlesList = formData.store_articles ? [...formData.store_articles.filter((_r: any) => _r != null)] : [];
    let payload: any = {};

    if(this.store_type === 'client') {
      payload = {
        STID: this.store_id,
        BRID : this.storeDetail?.store_brand_id,

        CSTNOM : formData.store_nom,
        CSTCCTID: formData.store_category,

        CSTEXPADDR : formData.store_address,
        CSTEXPCODPOS : formData.store_code_postal,
        CSTEXPPAYCOD : formData.store_pays,
        CSTEXPVILLE : formData.store_ville,

        CSTFACADDR : formData.storefac_address,
        CSTFACCODPOS : formData.storefac_code_postal,
        CSTFACVILLE : formData.storefac_ville,
        CSTFACPAYCOD :formData.storefac_pays,

        CSTDEFID : formData.store_defriement,
        CSTHQSTAT : formData.store_headquarters,

        CSTARTID : articlesList,
      };
    }

    if(this.store_type === 'supplier') {
      payload = {
        STID: this.store_id,
        BRID : this.storeDetail?.store_brand_id,

        SSTNOM : formData.store_nom,
        SSTCCTID: formData.store_category,

        SSTEXPADDR : formData.store_address,
        SSTEXPCODPOS : formData.store_code_postal,
        SSTEXPPAYCOD : formData.store_pays,
        SSTEXPVILLE : formData.store_ville,

        SSTFACADDR : formData.storefac_address,
        SSTFACCODPOS : formData.storefac_code_postal,
        SSTFACVILLE : formData.storefac_ville,
        SSTFACPAYCOD : formData.storefac_pays,

        SSTCOMM: formData.store_cout_commande,
        SSTDELIV: formData.store_delay_livraison,
        SSTHQSTAT : formData.store_headquarters,

        SSTARTID : articlesList,
      };
    }

    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        if(res.status === 'success') {
          this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          }).then(() => {
            this.__updateListCache(formData);
            this.router.navigate(['/contact/contact-list']);
          });
        } else {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        console.log(err);
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }

    this.contactService.updateStore(payload).subscribe(observer);
  }

  get isClient(): boolean {
    return (this.store_type !== undefined && this.store_type === 'client');
  }

  get isSupplier(): boolean {
    return (this.store_type !== undefined && this.store_type === 'supplier');
  }

  get expeditionCodePostal() {
    return this.modifyStoreForm.get("store_code_postal")?.value;
  }

  get expeditionVille() {
    return this.modifyStoreForm.get("store_ville")?.value;
  }

  get facturationCodePostal() {
    return this.modifyStoreForm.get("store_code_postal")?.value;
  }

  get facturationVille() {
    return this.modifyStoreForm.get("store_ville")?.value;
  }

  get expeditionPays() {
    return this.modifyStoreForm.get("store_pays")?.value;
  }

  get facturationPays() {
    return this.modifyStoreForm.get("storefac_pays")?.value;
  }

  get fr() {
    return this.modifyStoreForm.controls;
  }

  _checkBillingShippingSame() {
    if(this.isClient) {
      const EXP_ADDR = this.modifyStoreForm.get('store_address')?.value;
      const EXP_VILLE = this.modifyStoreForm.get('store_ville')?.value;
      const EXP_PAYS = this.modifyStoreForm.get('store_pays')?.value;
      const EXP_CODPOS = this.modifyStoreForm.get('store_code_postal')?.value;

      const FAC_ADDR = this.modifyStoreForm.get('storefac_address')?.value;
      const FAC_VILLE = this.modifyStoreForm.get('storefac_ville')?.value;
      const FAC_PAYS = this.modifyStoreForm.get('storefac_pays')?.value;
      const FAC_CODPOS = this.modifyStoreForm.get('storefac_code_postal')?.value;

      if((EXP_ADDR === FAC_ADDR)
        && (EXP_VILLE === FAC_VILLE)
        && (EXP_PAYS === FAC_PAYS)
        && (EXP_CODPOS === FAC_CODPOS)) {
        this.modifyStoreForm.get('SAME_AS_SHIPPING')?.setValue(true);
      }
    } else {
      const EXP_ADDR = this.modifyStoreForm.get('store_address')?.value;
      const EXP_VILLE = this.modifyStoreForm.get('store_ville')?.value;
      const EXP_PAYS = this.modifyStoreForm.get('store_pays')?.value;
      const EXP_CODPOS = this.modifyStoreForm.get('store_code_postal')?.value;

      const FAC_ADDR = this.modifyStoreForm.get('storefac_address')?.value;
      const FAC_VILLE = this.modifyStoreForm.get('storefac_ville')?.value;
      const FAC_PAYS = this.modifyStoreForm.get('storefac_pays')?.value;
      const FAC_CODPOS = this.modifyStoreForm.get('storefac_code_postal')?.value;

      if((EXP_ADDR === FAC_ADDR)
        && (EXP_VILLE === FAC_VILLE)
        && (EXP_PAYS === FAC_PAYS)
        && (EXP_CODPOS === FAC_CODPOS)) {
        this.modifyStoreForm.get('SAME_AS_SHIPPING')?.setValue(true);
      }
    }
  }

  _onSameBillingChange(status: boolean) {
    if (status) {
      this.modifyStoreForm.patchValue({
        storefac_address: this.modifyStoreForm.get('store_address')?.value,
        storefac_code_postal: this.modifyStoreForm.get('store_code_postal')?.value,
        storefac_pays: this.modifyStoreForm.get('store_pays')?.value,
        storefac_ville: this.modifyStoreForm.get('store_ville')?.value,
      });
    } else {
      this.modifyStoreForm.patchValue({
        storefac_address: "",
        storefac_code_postal: "",
        storefac_pays: "",
        storefac_ville: "",
      });
    }
  }

  _onArticleFilterChange(search: string) {
    this._filteredArticles = this.articleList.filter((row: ArticlesList) => row.article_label.toLowerCase().indexOf(search.toLowerCase()) > -1);
  }

  onCodePostalChanged(type: 'facturation' | 'expedition') {
    let postalCode = undefined,
        pays = undefined;

    if(type === 'facturation') {
      postalCode = this.modifyStoreForm.get('storefac_code_postal')?.value;
      pays = this.facturationPays;
    }

    if(type === 'expedition') {
      postalCode = this.modifyStoreForm.get('store_code_postal')?.value;
      pays = this.expeditionPays;
    }

    if(postalCode !== '' && postalCode != null) {
      const payload = {
        codpos: postalCode,
        codpays: pays
      };

      const observer = {
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
            if(type === 'facturation') {
              this.filteredPostalCodesFacturation = res.data;
            }

            if (type === 'expedition') {
              this.filteredPostalCodesExpedition = res.data;
            }
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };

      this.commonService.getListCodPosVille(payload).subscribe(observer);
    }
  }

  resetCodePostal(type: 'facturation' | 'expedition') {
    if(type === 'facturation') {
      this.modifyStoreForm.get('storefac_code_postal')?.reset()
    }

    if(type === 'expedition') {
      this.modifyStoreForm.get('store_code_postal')?.reset()
    }
  }

  onVilleChanged(type: 'facturation' | 'expedition') {
    let ville = undefined,
        pays = undefined;

    if(type === 'facturation') {
      ville = this.modifyStoreForm.get('storefac_ville')?.value;
      pays = this.facturationPays;
    }

    if(type === 'expedition') {
      ville = this.modifyStoreForm.get('store_ville')?.value;
      pays = this.expeditionPays;
    }

    if(ville !== '' && ville != null) {
      const payload = {
        ville,
        codpays: pays
      };

      const observer = {
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
            if(type === 'facturation') {
              this.filteredVillesFacturation = res.data;
            }

            if (type === 'expedition') {
              this.filteredVillesExpedition = res.data;
            }
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };

      this.commonService.getListCodPosVille(payload).subscribe(observer);
    }
  }

  resetVille(type: 'facturation' | 'expedition') {
    if(type =='facturation') {
      this.modifyStoreForm.get('storefac_ville')?.reset()
    }

    if(type == 'expedition') {
      this.modifyStoreForm.get('store_ville')?.reset()
    }
  }

  onCodeOptionSelected(event:any, type: 'facturation' | 'expedition') {
    if(type === 'facturation') {
      this.modifyStoreForm.get('storefac_code_postal')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.modifyStoreForm.get('storefac_ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
    }

    if(type === 'expedition') {
      this.modifyStoreForm.get('store_code_postal')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.modifyStoreForm.get('store_ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
    }
  }

  onVilleOptionSelected(event:any, type: 'facturation' | 'expedition') {
    if(type === 'facturation') {
      this.modifyStoreForm.get('storefac_code_postal')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.modifyStoreForm.get('storefac_ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
    }

    if(type === 'expedition') {
      this.modifyStoreForm.get('store_code_postal')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.modifyStoreForm.get('store_ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
    }
  }

}
