import {Component, Inject, OnInit} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {MAT_DIALOG_DATA, MatDialog, MatDialogRef} from "@angular/material/dialog";
import {TranslateService} from "@ngx-translate/core";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {lastValueFrom} from "rxjs";
import {Store} from "../../../_models";
import {ContactService} from "../../../_services/contact.service";
import {ConfirmComponent} from "../../../dialogs/confirm/confirm.component";
import {FusionnerTypeComponent} from "../../../_components/fusionner-type/fusionner-type.component";
import {APP_CONSTANTS} from "../../../_helpers/constants";

@Component({
  selector: 'app-fusioner-store',
  templateUrl: './fusioner-store.component.html',
  styleUrls: ['./fusioner-store.component.scss']
})
export class FusionerStoreComponent implements OnInit {

  currentStore: Store | undefined;
  fusionnerForm: FormGroup = this.formBuilder.group({
    CHOSEN_FUSIONNER_STORE: [undefined, Validators.required]
  });

  defaultFilterParams = {
    FILTER_TYPE: "store",
    FILTER_KEYWORD: "",

    FILTER_STORE_TYPE: "supplier",
  };

  constructor(public dialogRef: MatDialogRef<FusionerStoreComponent>,
              @Inject(MAT_DIALOG_DATA) public data: any,
              private translateService: TranslateService,
              private formBuilder: FormBuilder,
              private dialogService: DialogService,
              private contactService: ContactService,
              private loadingService: NgxSpinnerService,
              private dialog: MatDialog) { }

  ngOnInit(): void {
    if(this.data !== undefined && this.data !== null) {
      if(this.data.hasOwnProperty('currentStore')) {
        this.currentStore = this.data.currentStore;

        if(this.currentStore) {
          this.defaultFilterParams = {
            ...this.defaultFilterParams,
            FILTER_STORE_TYPE: this.currentStore.store_type
          }
        }
      }
    }
  }

  onFusionnerStoreChange(event: any) {
    this.fusionnerForm.get('CHOSEN_FUSIONNER_STORE')?.setValue(event);
  }

  get _getChosenFusionnerStore() {
    return this.fusionnerForm.get('CHOSEN_FUSIONNER_STORE');
  }

  confirmFusionnerStore(parentStore: Store, chosenStore: Store, type: string): Promise<any> {
    let message = "";

    if(type === APP_CONSTANTS.FUSIONNER_Y_TO_X) {
      message = `<h3>${this.translateService.instant('KFUSIONNER_STORE_CONFIRMATION_CHILD_PARENT', { CHOSEN_STORE: chosenStore.store_nom, PARENT_STORE: parentStore.store_nom})}</h3>`;
    }

    if(type === APP_CONSTANTS.FUSIONNER_X_TO_Y) {
      message = `<h3>${this.translateService.instant('KFUSIONNER_STORE_CONFIRMATION_PARENT_CHILD', { PARENT_STORE: parentStore.store_nom, CHOSEN_STORE: chosenStore.store_nom})}</h3>`;
    }

    if(type === APP_CONSTANTS.FUSIONNER_BOTH) {
      message = this.translateService.instant('KFUSIONNER_STORE_NOTHING_CHANGES');
    }

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KFUSIONNER_STORE_CONTINUE_MERGE',
      message
    }));
  }

  async confirmFusionnerType(parentStore: Store, chosenStore: Store): Promise<any> {
    return new Promise<any>((resolve) => {
      const dialog = this.dialog.open(FusionnerTypeComponent, {
        data: {
          x: parentStore.store_nom,
          y: chosenStore.store_nom
        },
        width: '400px',
        disableClose: true,
      });

      dialog.afterClosed().subscribe((res) => {
        resolve(res);
      })
    })
  }

  async onFusionnerSubmitter() {
    if(this.currentStore === undefined || this.currentStore === null) {
      return;
    }

    if(this.fusionnerForm.invalid) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KFUSIONNER_STORE_NO_ARTICLE_CHOOSEN'
      });
      return;
    }

    const formData = this.fusionnerForm.getRawValue();

    const confirmationTypeRes = await this.confirmFusionnerType(this.currentStore, formData.CHOSEN_FUSIONNER_STORE);

    if(confirmationTypeRes === undefined || confirmationTypeRes === null || confirmationTypeRes === '') {
      return;
    }

    const confirmationType = confirmationTypeRes.data;
    if(confirmationType === APP_CONSTANTS.FUSIONNER_BOTH) {
      this.dialogRef.close({ status: "success", fusionnerType: confirmationType, data: [] });
      return;
    }

    const confirmation = await this.confirmFusionnerStore(this.currentStore, formData.CHOSEN_FUSIONNER_STORE, confirmationType);

    if(confirmation) {
      this.loadingService.show();

      let x = undefined;
      let y = undefined;

      if(confirmationType === APP_CONSTANTS.FUSIONNER_Y_TO_X) {
        x = this.currentStore.store_id;
        y = formData.CHOSEN_FUSIONNER_STORE.store_id;
      }

      if(confirmationType === APP_CONSTANTS.FUSIONNER_X_TO_Y) {
        x = formData.CHOSEN_FUSIONNER_STORE.store_id;
        y = this.currentStore.store_id;
      }

      const payload = {
        store_type: this.currentStore.store_type,
        main_store_id: x,
        child_store_id: y
      };

      this.callFusionnerStoreAPI(payload, confirmationType);
      return;
    }

    return;
  }

  async callFusionnerStoreAPI(payload: any, fusionnerType: string) {
    this.contactService.mergeTwoStores(payload).subscribe(
      async (res: any) => {
        this.loadingService.hide();

        if(res.status === 'success') {
          const mergedMainStore = res.data;

          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });

          this.dialogRef.close({ status: "success", data: mergedMainStore, fusionnerType });
          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      err => {
        this.loadingService.hide();
      }
    )
  }

}
