import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { MatDatepicker } from '@angular/material/datepicker';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import * as Highcharts from 'highcharts';
import * as moment from 'moment';
import { NgxSpinnerService } from 'ngx-spinner';
import { CustomHttpResponse, Dashboard } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { DashboardService } from 'src/app/_services/dashboard.service';
import { APP_CONSTANTS } from '../../_helpers/constants';

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.scss']
})
export class DashboardComponent implements OnInit {

  Highcharts: typeof Highcharts = Highcharts;
  updateFlag = false;
  dashboardData:any = [];
  pendingCount:any=[];
  alertArticleData:any=[];
  chartForm:FormGroup = this.formBuilder.group({
    livraison_reception_statistics_type: ['week', [Validators.required]],
    livraison_reception_period: [''],
    achat_statistics_type: ['week', [Validators.required]],
    achat_period: [''],
  });

  statisticsTypeLR = APP_CONSTANTS.STATISTICSTYPE;

  AchatmonthDate = new FormControl(moment());
  AchatyearDate = new FormControl(moment());
  monthDate = new FormControl(moment());
  yearDate = new FormControl(moment());
  maxDate = new Date();

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private loadingService: NgxSpinnerService,
    private translateService: TranslateService,
    private dialogService: DialogService,
    private dashboardService:DashboardService) {}

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();
    this.getDashboardData();
    await this.loadingService.hide();
  }

  get itemFilterBLBRStatus() {
    return this.chartForm.get("livraison_reception_statistics_type");
  }

  get itemFilterAchatStatus() {
    return this.chartForm.get("achat_statistics_type");
  }

  getDashboardData(){
    const formData = this.chartForm.getRawValue();
    if(this.itemFilterBLBRStatus && this.itemFilterBLBRStatus.value == 'month'){
      formData.livraison_reception_period = moment(this.monthDate.value).format('MM.YYYY');
      formData.livraison_reception_statistics_type  = this.itemFilterBLBRStatus.value;
    }else if(this.itemFilterBLBRStatus && this.itemFilterBLBRStatus.value == 'year'){
      formData.livraison_reception_period = moment(this.yearDate.value).format('YYYY');
      formData.livraison_reception_statistics_type  = this.itemFilterBLBRStatus.value;
    }
    if(this.itemFilterAchatStatus && this.itemFilterAchatStatus.value == 'month'){
      formData.achat_period = moment(this.AchatmonthDate.value).format('MM.YYYY');
      formData.achat_statistics_type  = this.itemFilterAchatStatus.value;
    }else if(this.itemFilterAchatStatus && this.itemFilterAchatStatus.value == 'year'){
      formData.achat_period = moment(this.AchatyearDate.value).format('YYYY');
      formData.achat_statistics_type  = this.itemFilterAchatStatus.value;
    }
    this.getDashboardDataAPI(formData)
  }

  getDashboardDataAPI(payload:Dashboard){
    return new Promise((resolve, reject) => {
      this.dashboardService.getDashboardData(payload).subscribe({
        next: (res: CustomHttpResponse<Dashboard>) => {
          if(res.status === 'success') {
            this.dashboardData = res.data;
            this.pendingCount = this.dashboardData.pending;
            this.alertArticleData = this.dashboardData.alert_articles;
            this.updateChartData();
            resolve(res);
          }else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  updateChartData(){
    let xAxisLR:any= [];
    let xAxisachat:any= [];
    let livData:any= [];
    let RecData:any= [];
    let AchatData:any= [];
    this.dashboardData.livraison.map((data:any)=>{
      xAxisLR.push(data.label);
      livData.push(data.value);
    })
    this.dashboardData.reception.map((data:any)=>{
      RecData.push(data.value);
    })
    this.dashboardData.achat.map((data:any)=>{
      xAxisachat.push(data.label);
      AchatData.push(data.value);
    })

    this.chartOptions.xAxis= {
      categories: xAxisLR
    }
    this.chartOptionsColumn.xAxis= {
      categories: xAxisachat
    }
    this.chartOptionsColumn.series=[
      {
        name: this.translateService.instant('KDOCUMENT_ACHAT'),
        type:'column',
        data: AchatData
     }
    ]
    this.chartOptions.series=[
      {
        name: this.translateService.instant('KDOCUMENT_BL'),
        type:'line',
        data: livData
     },
     {
        name: this.translateService.instant('KDOCUMENT_BR'),
        type:'line',
        data: RecData
     }
    ]
    this.updateFlag = true;
  }

  setMonthAndYear(type:string, normalizedMonthAndYear: moment.Moment, datepicker: MatDatepicker<moment.Moment>) {
    const ctrlValue = this.monthDate.value!;
    ctrlValue.month(normalizedMonthAndYear.month());
    ctrlValue.year(normalizedMonthAndYear.year());
    if(type === 'achat'){
      this.AchatmonthDate.setValue(ctrlValue);
    }else{
      this.monthDate.setValue(ctrlValue);
    }
    datepicker.close();
    this.getDashboardData();
  }

  setYear(type:string, normalizedMonthAndYear: moment.Moment, datepicker: MatDatepicker<moment.Moment>) {
    const ctrlValue = this.yearDate.value!;
    ctrlValue.year(normalizedMonthAndYear.year());
    if(type === 'achat'){
      this.AchatyearDate.setValue(ctrlValue);
    }else{
      this.yearDate.setValue(ctrlValue);
    }
    datepicker.close();
    this.getDashboardData();
  }

  viewPendingData(statusValue: any, document: string){

    if(statusValue === "0"){
      this.dashboardService.dashboardStatusValue = statusValue;
    }else if(statusValue == true){
      this.dashboardService.returnPendingStatus = statusValue;
    }

    if(document === 'livraison'){
      this.router.navigate(['/bon-de-livraison/list'])
    }
    if(document === 'reception'){
      this.router.navigate(['/bon-de-reception/list'])
    }
    if(document === 'achat'){
      this.router.navigate(['/achat-commande/list'])
    }
    if(document === 'inventory'){
      this.router.navigate(['/inventory/inventory-list'])
    }
  }

  chartOptions: Highcharts.Options = {
    title :  {
      text: this.translateService.instant('KBLBR_CHART')
    },
    yAxis: {
      title: {
        text: this.translateService.instant('KBLBR_VALUES')
      }
    },
    xAxis: {
      categories: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
    },
    chart: {
      height: 400
    },
    series: [
      {
         name: this.translateService.instant('KDOCUMENT_BL'),
         type:'line',
         data: []
      },
      {
         name: this.translateService.instant('KDOCUMENT_BR'),
         type:'line',
         data: []
      }
     ]
  }

  chartOptionsColumn: Highcharts.Options = {
    title :  {
      text: this.translateService.instant('KACHAT_CHART')
    },
    xAxis: {
      categories: []
    },
    yAxis: {
      title: {
        text: this.translateService.instant('KACHAT_VALUES')
      }
    },
    chart: {
      height: 400
    },
    series: [
      {
         name: this.translateService.instant('KDOCUMENT_ACHAT'),
         type:'column',
         data: []
      }
     ]
  }

}
