import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { PageEvent } from '@angular/material/paginator';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { lastValueFrom } from 'rxjs';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { CustomHttpResponse, DepotTNT } from '../../../_models';
import { DialogService } from '../../../_services/dailog.service';
import { DepotTntService } from '../../../_services/depot-tnt.service';

@Component({
  selector: 'app-depot-tnt-list',
  templateUrl: './depot-tnt-list.component.html',
  styleUrls: ['./depot-tnt-list.component.scss']
})
export class DepotTntListComponent implements OnInit {

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  DepotTntList: DepotTNT[] | undefined;
  displayedColumnDepotTnt: string[] = ['DEPOT_TNT_LABEL','DEPOT_TNT_ADDRESS', 'DEPOT_TNT_STATUS', 'ACTIONS'];
  searchForm: FormGroup = this.formBuilder.group({
    keyword: [''],
  });
  rolePermission:any;

  constructor(private formBuilder: FormBuilder,
              private depotTntService: DepotTntService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService,
              private dataSharingService: DataSharingService) {}

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.checkForListCache();
    this.loadDependencies();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        keyword: formCache.keyword
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST);
      if(dataCache !== undefined) {

        this.DepotTntList = dataCache.data as DepotTNT[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.loadingService.hide();
  }

  get fr() {
    return this.searchForm.controls;
  }

  documentStatus(item: DepotTNT) {
    const statusList = APP_CONSTANTS.DEPOT_TNT_INFO_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.DEPOT_TNT_STATUS_ACTIVE:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.DEPOT_TNT_STATUS_IN_ACTIVE:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST_FORM, JSON.stringify(formData));
    const payload = {
      keyword: formData.keyword
    };
    this.currentPage = 1;
    this._searchDepotTntInfo(payload, this.currentPage, this.pageSize);
  }

  _searchDepotTntInfo(data: { keyword: string}, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<DepotTNT[]>) => {
        if(res.status === "success") {
          this.DepotTntList = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST);
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.depotTntService.searchDepotTNTInfo(data, page, pageSize).subscribe( observer);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIGNE_LIST);
    if(listCache !== undefined) {
      let listLigne: any[] = listCache.data;
      listLigne = (listLigne as DepotTNT[]).filter(r => r.id != id);

      listCache.data = listLigne;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listLigne.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      keyword: formData.keyword
    };
    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._searchDepotTntInfo(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveUser(depot_tnt_label: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: depot_tnt_label })}</h3>`;
    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeServiceExpedition(depot_tnt_id: number, depot_tnt_label:string) {
    if(!this.rolePermission['9_54']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveUser(depot_tnt_label);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteServiceExpeditionInfo(depot_tnt_id);
    }
  }

  async _deleteServiceExpeditionInfo(depot_tnt_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(depot_tnt_id);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.searchFormSubmitter();
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.depotTntService.deleteDepotTnt(depot_tnt_id).subscribe(observer);
  }


}
