import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators, FormArray } from '@angular/forms';
import {DatePipe, Location} from '@angular/common';
import { AuthService } from '../../../_services/auth.service';
import { TranslateService } from '@ngx-translate/core';
import {ActivatedRoute, Params, Router} from '@angular/router';
import {NumeroSerieComponent} from "../../inventory/numero-serie/numero-serie.component";
import { MatDialog } from '@angular/material/dialog';
import {TournantService} from "../../../_services/tournant.service";
import {DialogService} from "../../../_services/dailog.service";
import {CommonService} from "../../../_services/common.service";
import {
  BonTournant,
  BonTournantItem,
  CustomHttpResponse,
  LigneInfoList,
  StockLocationList,
  StoreInfoList
} from "../../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {
  CUSTOM_DATE_FORMAT,
  CUSTOM_DATE_TIME_API_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import * as moment from "moment";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import {MatSelect} from "@angular/material/select";
import {BonInventory, BonInventoryItem} from "../../../_models/inventory";
import {Article} from "../../../_models/article";
import {firstValueFrom} from "rxjs";
import {DataSharingService} from "../../../_services/data-sharing.service";
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {HistoriqueComponent} from "../../../_components/historique/historique.component";

@Component({
  selector: 'app-create-inventory-tournant',
  templateUrl: './create-inventory-tournant.component.html',
  styleUrls: ['./create-inventory-tournant.component.scss']
})
export class CreateInventoryTournantComponent implements OnInit {

  @ViewChild('stockLocationInput') stockLocationInputEl: MatSelect | undefined;

  createTournantForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],
    STOCK_SOURCE_ID: ["", Validators.required],

    LIGNE_INFO: [''], // Ligne ID
    LIGNE_CUSTOM_LABEL: [''], // Ligne Label

    ARTICLES_COUNT: [''],
    SUPPLIER:[''],
    MINIMUM_AMOUNT: [0, [Validators.minLength(APP_CONSTANTS.INVENTORY_TOURNANT_MIN_AMOUNT_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.INVENTORY_TOURNANT_MIN_AMOUNT_MAX_LENGTH)]],
    LINE_ITEMS: new FormArray([])
  });

  bonDocumentNum: string | undefined = "*********";
  bonTournant: BonTournant | undefined;
  bonTournantItems: BonTournantItem[] = [];
  bonTournantItemArticles: Article[] = [];

  ligneList: LigneInfoList[] = [];
  stockLocationList: StockLocationList[] = [];
  storeSupplierList:StoreInfoList[] = [];
  articleList: Article[] = [];
  totalRows: number = 0;
  currentPage: number = 1;
  pageSize: number = 25;
  pageSizeOptions = [25, 50, 100];

  customLigneInput: boolean = false;

  articleCount =[...APP_CONSTANTS.INVENTORY_TOURNANT_ARTICLE_COUNT];
  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  rolePermission:any;

  constructor(private location: Location,
              private formBuilder: FormBuilder,
              private authService: AuthService,
              private translateService: TranslateService,
              private router: Router,
              private route: ActivatedRoute,
              private datePipe: DatePipe,
              private dialog: MatDialog,
              private tournantService: TournantService,
              private dialogService: DialogService,
              private commonService: CommonService,
              private loadingService: NgxSpinnerService,
              private dataSharingService: DataSharingService) {
    }

  ngOnInit(): void {
    this.loadDependencies();

    this.createTournantForm.get("LIGNE_INFO")?.valueChanges.subscribe(
      (res) => {
        if(res === 'custom-ligne') {
          this.createTournantForm.get("LIGNE_CUSTOM_LABEL")?.setValue("");
          this.customLigneInput = true;
        } else {
          this.createTournantForm.get("LIGNE_CUSTOM_LABEL")?.setValue(res.LGILBL);
          this.customLigneInput = false;
        }
      }
    )
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.createTournantForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)
    });

    this._getRolePermission();
    await this._getLigneList();
    await this._getStockLocationList();
    await this._getStoreSupplierList();

    const routeParams = await firstValueFrom(this.route.queryParams);

    if(routeParams.hasOwnProperty('document_no') && routeParams["document_no"] !== null && routeParams["document_no"] !== "") {
      this.bonDocumentNum = routeParams["document_no"];

      if(this.bonDocumentNum !== undefined && this.bonDocumentNum !== null) {
        await this._getBonTournantDetail(this.bonDocumentNum);
      }
    }

    await this.loadingService.hide();
  }

  _getRolePermission(){
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
  }

  get fr() {
    return this.createTournantForm.controls;
  }

  get lineItems(): FormArray {
    return this.createTournantForm.get('LINE_ITEMS') as FormArray;
  }

  onCloseDocument() {
    if(this.isBonInventoryModifiable && (!this.lineItems || this.lineItems.controls.length === 0)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ARTICLE_ADDED_DOCUMENT'
      })

      return;
    }

    this.returnBack();
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST_FORM);
  }

  onSubmitDocument(type: 'save_continue' | 'save_exit' | 'validate' = 'save_continue') {

    if(type === 'validate'){
      if(!this.rolePermission['3_35']) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KACCESS_DENIED'
        })
        return;
      }
    }

    if(!this.lineItems || this.lineItems.controls.length === 0) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ARTICLE_ADDED_DOCUMENT'
      })

      return;
    }

    if(this.createTournantForm.invalid) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });

      return;
    }

    this.loadingService.show();

    const documentFormData = this.createTournantForm.getRawValue();
    let bonArticles: BonTournantItem[] = [];
    if(this.isBonInventoryModifiable) {
      bonArticles = [...this.bonTournantItems];
    }

    const lineItems = documentFormData.LINE_ITEMS;

    for(let i=0; i < lineItems.length; i++) {
      const articleExists = this.bonTournantItems.find((item) => item.article_id == lineItems[i].ARTICLE_ID);
      if(articleExists) {
        bonArticles.map((row) => {

          if(row.article_id == lineItems[i].ARTICLE_ID) {
            row.qte = lineItems[i].QTE;
            row.qte_stock = lineItems[i].AVAIL_STOCK;
            row.unit_price = lineItems[i].PRIX_UNIT;
            row.total_price = lineItems[i].PRIX_TOTAL;
            row.article_serials = lineItems[i].SERIALS_LIST ? lineItems[i].SERIALS_LIST.split(",") : [];
            row.removed_serials = lineItems[i].REMOVED_SERIALS_LIST ? lineItems[i].REMOVED_SERIALS_LIST.split(",") : [];
          }

          return row;
        });
      } else {
        bonArticles.push({
          article_id: lineItems[i].ARTICLE_ID,
          article_reference: lineItems[i].REFERENCE,
          article_desc: lineItems[i].LABEL,
          qte: lineItems[i].QTE,
          qte_stock: lineItems[i].AVAIL_STOCK,
          unit_price: lineItems[i].PRIX_UNIT,
          total_price: lineItems[i].PRIX_TOTAL,
          article_serials: lineItems[i].SERIALS_LIST ? lineItems[i].SERIALS_LIST.split(",") : [],
          removed_serials: lineItems[i].REMOVED_SERIALS_LIST ? lineItems[i].REMOVED_SERIALS_LIST.split(",") : [],
        });
      }
    }

    const bonStatus = type === 'validate' ? APP_CONSTANTS.INVENTORY_TOURNANT_VALIDATION_STAT_YES : APP_CONSTANTS.INVENTORY_TOURNANT_VALIDATION_STAT_NO;
    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonTournant = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      stock_location_id: +documentFormData.STOCK_SOURCE_ID,
      supplier: documentFormData.SUPPLIER,
      minimum_amount: documentFormData.MINIMUM_AMOUNT,
      articles: bonArticles,
      status: bonStatus,
    };

    if(this.isBonInventoryModifiable) {
      this.__resetListCache();
      this._modifyInventoryTournant(this.bonDocumentNum, bonDocument).then(
        () => {
          if(type === 'validate' || type === 'save_exit') {
            this.returnBack();
          }
        }
      )

      return;
    }

    this._addInventoryTournant(bonDocument).then(
      () => {
        this.createTournantForm.get("STOCK_SOURCE_ID")?.disable({ onlySelf: true });

        // After adding a new Line item.
        this.__updateCurrentRouteDocumentNum(this.bonDocumentNum);
        this.__resetListCache();
        if(type === 'validate' || type === 'save_exit') {
          this.returnBack();
        }
      }
    )
  }

  generateInventoryTournantPdf(print?: boolean) {
    if(!this.isBonInventoryModifiable || (this.bonDocumentNum === undefined)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'No Content'
      })
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.tournantService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      }
    });
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_INVENTORY_TOURNANT_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  returnBack() {
    this.router.navigate(['/inventory-tournant/list']);
  }

  _getStockLocationList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.stockLocationList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getStockLocationList().subscribe( observer);
    })
  }

  _getStoreSupplierList(): Promise<any> {
    const payload = {
      type: 'supplier'
    };
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<StoreInfoList[]>) => {
          if(res.status === 'success') {
            this.storeSupplierList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListStores(payload).subscribe( observer);
    })
  }

  _getLigneList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getAllLigneList().subscribe( observer);
    })
  }

  checkStockSource(){
    const location_id = this.createTournantForm.get('STOCK_SOURCE_ID')?.value;
    if(location_id === undefined || location_id === null || location_id === "") {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KTOURNANT_CHOOSE_STOCK_SOURCE_BEFORE'
      });
      this.createTournantForm.get('ARTICLES_COUNT')?.setValue("");
      this.createTournantForm.get('MINIMUM_AMOUNT')?.setValue("0");
      this.createTournantForm.get('SUPPLIER')?.setValue("");
      return true;
    }
    return false;
  }

  getRandomArticles(rowCount: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const payload = new FormData();
      const tournantForm = this.createTournantForm.getRawValue();

      payload.append('article_count', rowCount);
      payload.append('sourcelocation_id', tournantForm.STOCK_SOURCE_ID);
      payload.append('supplier', tournantForm.SUPPLIER);
      payload.append('minimum_amount', tournantForm.MINIMUM_AMOUNT);

      const observer = {
        next: (res: CustomHttpResponse<Article[]>) => {
          if(res.status === 'success') {
            this.articleList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.tournantService.generateTournantRandomArticles(payload).subscribe(observer);
    });
  }

  async generateRandomArticleRows() {
    const rowCount = this.createTournantForm.get('ARTICLES_COUNT')?.value;

    this.lineItems.clear();
    if(this.checkStockSource()){
      return;
    }
    if(rowCount === undefined || rowCount === null || rowCount === "") {
      return;
    }

    await this.getRandomArticles(rowCount);

    this.__generateLineItems(this.articleList);
  }

  __generateLineItems(articles: Article[] | BonTournantItem[], existingItem: boolean = false) {
    if(articles !== undefined && articles !== null && articles.length > 0) {
      const formArr = this.createTournantForm.get('LINE_ITEMS') as FormArray;
      formArr.clear();

      articles.forEach((row) => {
        let articleRow: Article = row as Article;
        let tournantItem: BonTournantItem = row as BonTournantItem;

        const lineItem = this.formBuilder.group({
          ARTICLE_ID: ["", Validators.required],
          REFERENCE: [{ value: "", disabled: true }, Validators.required],
          LABEL: [{ value: "", disabled: true }, Validators.required],

          QTE: [{ value: "0", disabled: true }],

          PRIX_UNIT: ['', [Validators.required]],
          PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required]],
          AVAIL_STOCK: [{ value: "0", disabled: true }],

          SERIALS_LIST: [''],
          REMOVED_SERIALS_LIST: [''],
          OLD_SERIALS_LIST: [''],
        });

        let unitPrix = 0;
        let totalPrix = 0;
        let quantityActual = 0;
        let quantityStock = 0;

        let serialList = "",
          removedSerials = "",
          oldSerialsList = "";

        if(existingItem) {
          articleRow = (this.bonTournantItemArticles.find(row => row.id == tournantItem.article_id) || {}) as Article;
          // Generate Articles after saving the document with existing saved items (BonTournantItem[]).
          if(tournantItem?.unit_price !== undefined && tournantItem?.unit_price !== null) {
            unitPrix = +tournantItem?.unit_price;
          }

          if(tournantItem?.qte_stock !== undefined && tournantItem?.qte_stock !== null) {
            quantityStock = +tournantItem?.qte_stock;
          }

          if(articleRow.serial_list !== undefined && articleRow.serial_list !== null) {
            oldSerialsList = articleRow.serial_list || "";
          }

          quantityActual = tournantItem.qte;
          unitPrix = tournantItem.unit_price;

          if(tournantItem.article_serials !== undefined && tournantItem.article_serials !== null && tournantItem.article_serials.length > 0) {
            serialList = tournantItem.article_serials.join(',');
          }

          if(tournantItem.removed_serials !== undefined && tournantItem.removed_serials !== null && tournantItem.removed_serials.length > 0) {
            removedSerials = tournantItem.removed_serials.join(',');
          }

          totalPrix = +unitPrix * +quantityActual;

          lineItem.patchValue({
            ARTICLE_ID: tournantItem.article_id,
            REFERENCE: tournantItem.article_reference,
            LABEL: tournantItem.article_desc,

            QTE: quantityActual.toString(),

            PRIX_UNIT: unitPrix.toString(),
            PRIX_TOTAL: totalPrix.toString(),
            AVAIL_STOCK: quantityStock.toString(),

            SERIALS_LIST: serialList,
            REMOVED_SERIALS_LIST: removedSerials,
            OLD_SERIALS_LIST: oldSerialsList,
          });
        } else {
          // Generate Articles before saving the document
          if(articleRow?.public_prix !== undefined && articleRow?.public_prix !== null) {
            unitPrix = +articleRow?.public_prix;
          }

          if(articleRow?.avail_stock !== undefined && articleRow?.avail_stock !== null) {
            quantityStock = +articleRow?.avail_stock;
          }

          if(articleRow.serial_list !== undefined && articleRow.serial_list !== null) {
            oldSerialsList = articleRow.serial_list;
          }

          totalPrix = +unitPrix * +quantityActual;

          lineItem.patchValue({
            ARTICLE_ID: articleRow.id,
            REFERENCE: articleRow.reference,
            LABEL: articleRow.label,

            QTE: quantityActual.toString(),

            PRIX_UNIT: unitPrix.toString(),
            PRIX_TOTAL: totalPrix.toString(),
            AVAIL_STOCK: quantityStock.toString(),

            SERIALS_LIST: serialList,
            REMOVED_SERIALS_LIST: removedSerials,
            OLD_SERIALS_LIST: oldSerialsList,
          });
        }

        formArr.push(lineItem);
      });
    }

    console.log(this.createTournantForm.get('LINE_ITEMS'));
  }

  _getBonTournantDetail(document_no: string): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<BonTournant>) => {
          if(res.status === 'success') {
            this.bonTournant = res.data;

            this.__refreshBonDetails(this.bonTournant);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(() => {
              this.router.navigate(['/inventory-tournant/list']);
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.tournantService.getTournantDetail(document_no).subscribe( observer);
    })
  }

  __refreshBonDetails(bonTournant: BonTournant) {
    if(bonTournant !== undefined && bonTournant !== null) {
      this.bonDocumentNum = bonTournant.document_no;
      this.bonTournantItems = bonTournant.articles;
      this.bonTournantItemArticles = bonTournant.custom_articles ? bonTournant.custom_articles : [];

      this.createTournantForm.get("NUMBON")?.setValue(bonTournant.document_no);
      this.createTournantForm.get("DATE_DU_BON")?.setValue(moment(bonTournant.dated).format(CUSTOM_DATE_FORMAT));
      this.createTournantForm.get("STOCK_SOURCE_ID")?.setValue(bonTournant.stock_location_id);

      // Disabling the Stock location dropdown after one article is added to the document as a document should be assigned to one Stock location only
      this.createTournantForm.get("STOCK_SOURCE_ID")?.disable({ onlySelf: true });
      this.createTournantForm.get("SUPPLIER")?.disable({ onlySelf: true });
      this.createTournantForm.get("MINIMUM_AMOUNT")?.disable({ onlySelf: true });
      const article_count = {
        label: this.bonTournantItems.length,
        value: this.bonTournantItems.length
      }
      this.articleCount.push(article_count);
      this.createTournantForm.get("ARTICLES_COUNT")?.setValue(this.bonTournantItems.length || "");
      this.createTournantForm.get("ARTICLES_COUNT")?.disable({ onlySelf: true });

      this.__generateLineItems(this.bonTournantItems, true);

      if(bonTournant.ligne_lbl !== undefined && bonTournant.ligne_lbl !== null && bonTournant.ligne_lbl !== "") {
        if(bonTournant.ligne_id === undefined || bonTournant.ligne_id === null || bonTournant.ligne_id === "") {
          // Custom Ligne Input
          this.customLigneInput = true;
          this.createTournantForm.get("LIGNE_INFO")?.setValue('custom-ligne');
          this.createTournantForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonTournant.ligne_lbl);
        } else {
          // Chosen Ligne Input from the dropdown
          const currentLigneItem = this.ligneList.find(r => r.LGIID == bonTournant.ligne_id)
          this.createTournantForm.get("LIGNE_INFO")?.setValue(currentLigneItem);
          this.createTournantForm.get("LIGNE_CUSTOM_LABEL")?.setValue(currentLigneItem?.LGILBL);
        }
      }
    }
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  get isBonInventoryModifiable() {
    return this.bonTournant !== undefined;
  }

  _addInventoryTournant(bonDocument: BonTournant): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.tournantService.createTournant(bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonTournant>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonTournantData = res.data;
            this.bonTournant = bonTournantData;

            this.__refreshBonDetails(bonTournantData);
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject();
          }
        },
        error: err => {
          this.loadingService.hide();

          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  _modifyInventoryTournant(document_no: string | undefined, bonDocument: BonTournant): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined) {
        reject("INVALID_DOCUMENT_NO");
        return;
      }

      this.tournantService.modifyTournant(document_no, bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonTournant>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonTournantData = res.data;
            this.bonTournant = bonTournantData;

            this.__refreshBonDetails(bonTournantData);
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: err => {
          this.loadingService.hide();

          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  __updateCurrentRouteDocumentNum(document_no: string | undefined) {
    if(document_no === undefined) {
      return;
    }

    const queryParams: Params = { document_no };

    this.router.navigate(
      [],
      {
        relativeTo: this.route,
        queryParams: queryParams,
        queryParamsHandling: 'merge', // remove to replace all query params by provided
      });
  }

  changeQuantity(i: any, prixCaisse: any) {
    let listSerials = [],
      oldListSerials = [],
      removedListSerials = [];

    const article_id = this.lineItems.at(i).get("ARTICLE_ID")?.value;
    const articleDetail = this.articleList?.find(r => r.id == article_id);

    const itemSerials = this.lineItems.at(i).get('SERIALS_LIST')?.value;
    if(itemSerials !== undefined && itemSerials !== null && itemSerials !== "") {
      listSerials = itemSerials.split(",");
    }

    listSerials = this.commonService.formatSerialNumbers(listSerials);

    const itemOldSerials = this.lineItems.at(i).get('OLD_SERIALS_LIST')?.value;
    if(itemOldSerials !== undefined && itemOldSerials !== null && itemOldSerials !== "") {
      oldListSerials = itemOldSerials.split(",");
    }

    oldListSerials = this.commonService.formatSerialNumbers(oldListSerials);

    const itemRemovedSerials = this.lineItems.at(i).get('REMOVED_SERIALS_LIST')?.value;
    if(itemRemovedSerials !== undefined && itemRemovedSerials !== null && itemRemovedSerials !== "") {
      removedListSerials = itemRemovedSerials.split(",");
    }

    removedListSerials = this.commonService.formatSerialNumbers(removedListSerials);

    const blockedInSerials = articleDetail?.blocked_in_serials;
    const blockedOutSerials = articleDetail?.blocked_out_serials;

    this.openUpSerialNumPopup(i, {
      newListSerials: listSerials,
      removeListSerials: removedListSerials,
      oldListSerials: oldListSerials,
      block_in_serials: blockedInSerials,
      block_out_serials: blockedOutSerials
    }, prixCaisse);
  }

  openUpSerialNumPopup(row_id: number, data: {}, prixCaisse: any) {
    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: {
          numeroType:'inventory-tournant-numero-series',
          ...data
        }
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        console.log("validation",result.data);
        if(result.data && result.data !== '') {
          const { newSerials, removedSerials } = result.data;

          this.lineItems.at(row_id).get('QTE')?.setValue(newSerials.length.toString());
          this.lineItems.at(row_id).get('SERIALS_LIST')?.setValue(newSerials.join(','));
          this.lineItems.at(row_id).get('REMOVED_SERIALS_LIST')?.setValue(removedSerials.join(','));

          this.calculatePrixTotal(row_id);

          prixCaisse.select();
          prixCaisse.focus();
        }
      }
    });
  }

  calculatePrixTotal(i: number) {
    const formData = (this.lineItems.at(i) as FormGroup).getRawValue();

    let totalPrix = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      this.lineItems.at(i).get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      this.lineItems.at(i).get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    this.lineItems.at(i).get("PRIX_TOTAL")?.setValue(totalPrix.toFixed(2));
  }

}
