import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import {PageEvent} from '@angular/material/paginator';
import {NgxSpinnerService} from "ngx-spinner";
import {TournantService} from '../../../_services/tournant.service'
import {CustomHttpResponse, ListInventoryTournant} from "../../../_models";
import * as moment from "moment";
import {CUSTOM_DATE_API_FORMAT} from "../../../_helpers/custom-date-formats";
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {TranslateService} from "@ngx-translate/core";
import {lastValueFrom} from "rxjs";
import {InventoryList} from "../../../_models/inventory";
import {DialogService} from "../../../_services/dailog.service";

@Component({
  selector: 'app-inventory-tournant-list',
  templateUrl: './inventory-tournant-list.component.html',
  styleUrls: ['./inventory-tournant-list.component.scss']
})
export class InventoryTournantListComponent implements OnInit {

  searchForm:FormGroup = this.formBuilder.group({
    document_no: [''],
    start_date: [{ value: '', disabled: true }],
    end_date: [{ value: '', disabled: true }],
  });

  listTournant: ListInventoryTournant[] | undefined;

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  displayedColumnsInventory: string[] = ['TNTDOCNO', 'TNTDOCDATE', 'STLLBL', 'LGILBL', 'STATUS', 'ACTIONS'];

  rolePermission:any;

  constructor(private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private tournantService: TournantService,
              private dataSharingService: DataSharingService,
              private translateService: TranslateService,
              private dialogService: DialogService) {
  }

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }

    this.checkForListCache();
  }

  documentStatus(item: ListInventoryTournant) {
    const statusList = APP_CONSTANTS.INVENTORY_TOURNANT_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.INVENTORY_TOURNANT_VALIDATION_STAT_YES:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.INVENTORY_TOURNANT_VALIDATION_STAT_NO:
        result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        document_no: formCache.document_no,
        start_date: formCache.start_date,
        end_date: formCache.end_date
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST);
      if(dataCache !== undefined) {

        this.listTournant = dataCache.data as ListInventoryTournant[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  get startDate() {
    return this.searchForm.get('start_date');
  }

  get endDate() {
    return this.searchForm.get('end_date');
  }

  isDocumentValidated(item: ListInventoryTournant): boolean {
    return item.status === APP_CONSTANTS.INVENTORY_TOURNANT_VALIDATION_STAT_YES
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST_FORM, JSON.stringify(formData));
    const start_date = formData.start_date ? moment(formData.start_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";
    const end_date = formData.end_date ? moment(formData.end_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";

    const payload = {
      document_no: formData.document_no,
      start_date: start_date,
      end_date: end_date
    };

    this.currentPage = 1;

    this._getTournantList(payload, this.currentPage, this.pageSize);
  }

  _getTournantList(payload: { document_no: string, start_date: string, end_date: string }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<ListInventoryTournant[]>) => {
        if(res.status === "success") {
          this.listTournant = res.data;

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST);
        }


        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.tournantService.getTournantList(payload, page, pageSize).subscribe(observer);
  }

  __updateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST);
    if(listCache !== undefined) {
      let listTournant: any[] = listCache.data;
      listTournant = (listTournant as ListInventoryTournant[]).filter(r => r.document_no != id);

      listCache.data = listTournant;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listTournant.length === 0) {
        this.searchFormSubmitter();
      }
    }
  }

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();

    const start_date = formData.start_date ? moment(formData.start_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";
    const end_date = formData.end_date ? moment(formData.end_date).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";

    const payload = {
      document_no: formData.document_no,
      start_date: start_date,
      end_date: end_date
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._getTournantList(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveTournant(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeTournant(document_no: string, item: ListInventoryTournant) {
    if((this.isDocumentValidated(item) && !this.rolePermission['3_37']) || (!this.isDocumentValidated(item) && !this.rolePermission['3_34'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveTournant(document_no);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteTournant(document_no);
    }
  }

  async _deleteTournant(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(document_no);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();

        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.tournantService.deleteTournant(document_no).subscribe(observer);
  }

}
