import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import * as moment  from 'moment';
import { animate, state, style, transition, trigger } from '@angular/animations';
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {MatDialog} from "@angular/material/dialog";
import {NgxSpinnerService} from "ngx-spinner";
import {TournantService} from "../../../_services/tournant.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {NumeroSerieComponent} from '../../inventory/numero-serie/numero-serie.component';
import {
  CustomHttpResponse,
  BonTournant,
  BonTournantItem,
  LigneInfoList,
  StockLocationList,
  StoreInfoList
} from "../../../_models";
import {DialogService} from "../../../_services/dailog.service";
import {CUSTOM_DATE_FORMAT} from "../../../_helpers/custom-date-formats";
import {CommonService} from "../../../_services/common.service";
import {Article} from "../../../_models/article";
import { HistoriqueComponent } from 'src/app/_components/historique/historique.component';

@Component({
  selector: 'app-view-inventory-tournant',
  templateUrl: './view-inventory-tournant.component.html',
  styleUrls: ['./view-inventory-tournant.component.scss'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({ height: '0px', minHeight: '0', visibility: 'hidden' })),
      state('expanded', style({ height: '*', visibility: 'visible' })),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class ViewInventoryTournantComponent implements OnInit {

  viewInventoryTournantForm: FormGroup = this.formBuilder.group({
    dated: new FormControl({ value: '', disabled: true }),
    status: new FormControl({ value: '', disabled: true }, [Validators.required]),
    stock_location_id: new FormControl({ value: '', disabled: true }, [Validators.required]),
    ligne_id: new FormControl({ value: '', disabled: true }),
    ligne_lbl:new FormControl({ value: '', disabled: true }),
    articles_count:new FormControl({ value: '', disabled: true }),
    supplier: new FormControl({ value: '', disabled: true }),
    minimum_amount:new FormControl({ value: '', disabled: true }),
  });

  storageList: StockLocationList[] = [];
  ligneList: LigneInfoList[] = [];

  bonDocumentNum: string | undefined;
  tournantDetail: BonTournant | undefined;
  tournantItems: BonTournantItem[] | undefined;
  storeSupplierList:StoreInfoList[] = [];
  bonInventoryTournantItemArticles: Article[] = [];
  displayedColumnsArticle: string[] = ['REF', 'DESC', 'QTE','PU','PT','QATTN','ACTIONS'];

  customLigneInput: boolean = false;

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  constructor(private formBuilder: FormBuilder,
              private router: Router,
              private route: ActivatedRoute,
              private dialog: MatDialog,
              private commonService: CommonService,
              private tournantService: TournantService,
              private translateService: TranslateService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService) {
  }

  ngOnInit(): void {
    this.route.params.subscribe(params => {
      if(params.hasOwnProperty('document_no') && params["document_no"] !== null && params["document_no"] !== "") {
        this.bonDocumentNum = params["document_no"];
        this.loadDependencies();
      }
    });
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getStorageList();
    await this.getLigneList();
    await this._getStoreSupplierList();
    if(this.bonDocumentNum !== undefined) {
      await this.getTournantDetail(this.bonDocumentNum);
    }

    await this.loadingService.hide();
  }

  getStorageList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.storageList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getStockLocationList().subscribe( observer);
    })
  }

  _getStoreSupplierList(): Promise<any> {
    const payload = {
      type: 'supplier'
    };
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<StoreInfoList[]>) => {
          if(res.status === 'success') {
            this.storeSupplierList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListStores(payload).subscribe( observer);
    })
  }

  getLigneList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getAllLigneList().subscribe( observer);
    })
  }

  getTournantDetail(document_no: string): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<BonTournant>) => {
          if(res.status === 'success') {
            this.tournantDetail = res.data;
            this._refreshBonDetails(this.tournantDetail);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(() => {
              this.router.navigate(['/inventory-tournant/list']);
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.tournantService.getTournantDetail(document_no).subscribe( observer);
    })
  }

  _refreshBonDetails(tournantDetail: BonTournant) {
    this.bonDocumentNum = tournantDetail.document_no;

    this.tournantItems = tournantDetail.articles;

    if(tournantDetail.custom_articles) {
      this.bonInventoryTournantItemArticles = tournantDetail.custom_articles;
    }

    this.viewInventoryTournantForm.get('dated')?.setValue(moment(tournantDetail.dated).format(CUSTOM_DATE_FORMAT));
    this.viewInventoryTournantForm.get('status')?.setValue(tournantDetail.status);
    this.viewInventoryTournantForm.get('stock_location_id')?.setValue(tournantDetail.stock_location_id);
    this.viewInventoryTournantForm.get('articles_count')?.setValue(tournantDetail.articles.length);
    this.viewInventoryTournantForm.get('supplier')?.setValue(tournantDetail.supplier);
    this.viewInventoryTournantForm.get('minimum_amount')?.setValue(tournantDetail.minimum_amount);

    if(tournantDetail.ligne_lbl !== undefined && tournantDetail.ligne_lbl !== null && tournantDetail.ligne_lbl !== "") {
      if(tournantDetail.ligne_id === undefined || tournantDetail.ligne_id === null || tournantDetail.ligne_id === "") {
        // Custom Ligne Input
        this.customLigneInput = true;
        this.viewInventoryTournantForm.get("ligne_id")?.setValue('custom-ligne');
        this.viewInventoryTournantForm.get("ligne_lbl")?.setValue(tournantDetail.ligne_lbl);
      } else {
        // Chosen Ligne Input from the dropdown
        const currentLigneItem = this.ligneList.find(r => r.LGIID == tournantDetail.ligne_id)
        this.viewInventoryTournantForm.get("ligne_id")?.setValue(currentLigneItem);
        this.viewInventoryTournantForm.get("ligne_lbl")?.setValue(tournantDetail.ligne_lbl);
      }
    }
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  viewItemSerials(item:any,index:number) {
    const itemSelectedSerials = item.article_serials;
    const selectedListSerials = this.commonService.formatSerialNumbers(itemSelectedSerials ? itemSelectedSerials : []);

    const itemRemovedSerials = item.removed_serials;
    const removedListSerials = this.commonService.formatSerialNumbers(itemRemovedSerials ? itemRemovedSerials : []);

    const itemOldSerials = this.bonInventoryTournantItemArticles[index].serial_list?.split(",");
    const oldListSerials = this.commonService.formatSerialNumbers(itemOldSerials ? itemOldSerials : []);

    this.openUpSerialNumPopup({
      newListSerials: selectedListSerials,
      oldListSerials: oldListSerials,
      removeListSerials: removedListSerials,
      readonly: true
    });
  }

  openUpSerialNumPopup(data: {}) {
    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: {
          numeroType:'inventory-tournant-numero-series',
          ...data
        }
      }
    );
  }

  generateInventoryTournantPdf(print?: boolean) {
    if(this.bonDocumentNum === undefined || this.bonDocumentNum === null) {
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.tournantService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      }
    });
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_INVENTORY_TOURNANT_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  openHistorique(){
    const data = {
      document_no: this.bonDocumentNum,
      document_type: APP_CONSTANTS.DOCUMENT_TYPE_TOURNANT
    };
    this.dialog.open(HistoriqueComponent,
      {
        minWidth: '80%',
        maxHeight: '90vh',
        data: data || {}
      }
    );
  }

}
