import {Component, OnInit, ViewChild} from '@angular/core';
import {FormArray, FormBuilder, FormGroup, Validators} from '@angular/forms';
import {ActivatedRoute, Params, Router} from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { MatDialog } from '@angular/material/dialog';
import {NumeroSerieComponent} from "../numero-serie/numero-serie.component";
import { trigger, state, style, transition, animate } from '@angular/animations';
import {InventoryService} from "../../../_services/inventory.service";
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import {CommonService} from "../../../_services/common.service";
import * as moment from 'moment';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {MatSelect} from "@angular/material/select";
import {CustomHttpResponse, LigneInfoList, NumeroSerialList, StockLocationList} from "../../../_models";
import {BonInventory, BonInventoryItem, InventoryList} from "../../../_models/inventory";
import {Article} from "../../../_models/article";
import {
  CUSTOM_DATE_FORMAT,
  CUSTOM_DATE_TIME_API_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import {firstValueFrom} from "rxjs";
import {PageEvent} from "@angular/material/paginator";
import { HistoriqueComponent } from 'src/app/_components/historique/historique.component';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-edit-inventory',
  templateUrl: './edit-inventory.component.html',
  styleUrls: ['./edit-inventory.component.scss'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({ height: '0px', minHeight: '0', visibility: 'hidden' })),
      state('expanded', style({ height: '*', visibility: 'visible' })),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class EditInventoryComponent implements OnInit {

  @ViewChild('stockLocationInput') stockLocationInputEl: MatSelect | undefined;

  bonDocumentForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],
    STOCK_SOURCE_ID: ["", Validators.required],

    LIGNE_INFO: [''], // Ligne ID
    LIGNE_CUSTOM_LABEL: [''], // Ligne Label

    LINE_ITEMS: new FormArray([])
  })

  bonDocumentNum: string | undefined = "*********";
  bonInventory: BonInventory | undefined;
  bonInventoryItems: BonInventoryItem[] = [];
  bonInventoryItemArticles: Article[] = [];

  ligneList: LigneInfoList[] = [];
  stockLocationList: StockLocationList[] = [];

  articlesList: Article[] | undefined;
  totalRows: number = 0;
  currentPage: number = 1;
  pageSize: number = 25;
  pageSizeOptions = [25, 50, 100];

  customLigneInput: boolean = false;

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  rolePermission:any;
  constructor(public dialog: MatDialog,
              private commonService: CommonService,
              private inventoryService: InventoryService,
              private router: Router,
              private route: ActivatedRoute,
              private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    this.loadDependencies();

    this.bonDocumentForm.get("LIGNE_INFO")?.valueChanges.subscribe(
      (res) => {
        if(res === 'custom-ligne') {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue("");
          this.customLigneInput = true;
        } else {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(res.LGILBL);
          this.customLigneInput = false;
        }
      }
    );
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.bonDocumentForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)
    });
    this._getRolePermission();
    await this._getLigneList();
    await this._getStockLocationList();

    const routeParams = await firstValueFrom(this.route.params);

    if(routeParams.hasOwnProperty('document_no') && routeParams["document_no"] !== null && routeParams["document_no"] !== "") {
      this.bonDocumentNum = routeParams["document_no"];

      await this._getBonInventoryDetail(this.bonDocumentNum);

      if (this.bonInventory !== undefined && this.bonInventory !== null) {
        this.currentPage = 1;
        await this._generateArticles({ location_id: this.bonInventory.stock_location_id }, this.currentPage, this.pageSize);
      }
    } else {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KINVENTORY_INVALID_DOCUMENT'
      }).then(() => {
        this.returnBack();
      })

      return;
    }

    await this.loadingService.hide();
  }

  _getRolePermission(){
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
  }

  get lineItems(): FormArray {
    return this.bonDocumentForm.get('LINE_ITEMS') as FormArray;
  }

  get isBonInventoryModifiable() {
    return this.bonInventory !== undefined;
  }

  get __isDocumentValidated() {
    return this.bonInventory && this.bonInventory.status === APP_CONSTANTS.INVENTORY_VALIDATION_STAT_YES;
  }

  onCloseDocument() {
    if(!this.lineItems || this.lineItems.controls.length === 0) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ARTICLE_ADDED_DOCUMENT'
      })

      return;
    }

    this.returnBack();
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST);
    console.log(payload)
    if(listCache !== undefined) {
      let listInventory = listCache.data as InventoryList[];
      listInventory = listInventory.map((row) => {
        if(row.document_no == payload.document_no) {
          console.log(row)
          row.document_date = payload.document_date;
          row.document_no = payload.document_no;
          row.inventory_id = payload.inventory_id;
          row.ligne_id = payload.ligne_id;
          row.ligne_lbl = payload.ligne_lbl;
          row.status = payload.status;
          row.stock_location_id = payload.stock_location_id;
          row.stock_location_lbl = payload.stock_location_lbl;
        }
        return row;
      });

      listCache.data = listInventory;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST, JSON.stringify(listCache));
    }
  }

  onSubmitDocument(type: 'save_continue' | 'save_exit' | 'validate' = 'save_continue') {

    if(type === 'validate'){
      if(!this.rolePermission['3_15']) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KACCESS_DENIED'
        })
        return;
      }
    }

    if(!this.lineItems || this.lineItems.controls.length === 0) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ARTICLE_ADDED_DOCUMENT'
      })

      return;
    }

    if(this.bonDocumentForm.invalid) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });

      return;
    }

    this.loadingService.show();

    const documentFormData = this.bonDocumentForm.getRawValue();

    let bonArticles: BonInventoryItem[] = [...this.bonInventoryItems];

    const lineItems = documentFormData.LINE_ITEMS;

    for(let i=0; i < lineItems.length; i++) {
      const articleExists = this.bonInventoryItems.find((item) => item.article_id == lineItems[i].ARTICLE_ID);
      if(articleExists) {
        bonArticles.map((row) => {

          if(row.article_id == lineItems[i].ARTICLE_ID) {
            row.qte = lineItems[i].QTE;
            row.qte_stock = lineItems[i].AVAIL_STOCK;
            row.unit_price = lineItems[i].PRIX_UNIT;
            row.total_price = lineItems[i].PRIX_TOTAL;
            row.article_serials = lineItems[i].SERIALS_LIST ? lineItems[i].SERIALS_LIST.split(",") : [];
            row.removed_serials = lineItems[i].REMOVED_SERIALS_LIST ? lineItems[i].REMOVED_SERIALS_LIST.split(",") : [];
          }

          return row;
        });
      } else {
        bonArticles.push({
          article_id: lineItems[i].ARTICLE_ID,
          article_reference: lineItems[i].REFERENCE,
          article_desc: lineItems[i].LABEL,
          qte: lineItems[i].QTE,
          qte_stock: lineItems[i].AVAIL_STOCK,
          unit_price: lineItems[i].PRIX_UNIT,
          total_price: lineItems[i].PRIX_TOTAL,
          article_serials: lineItems[i].SERIALS_LIST ? lineItems[i].SERIALS_LIST.split(",") : [],
          removed_serials: lineItems[i].REMOVED_SERIALS_LIST ? lineItems[i].REMOVED_SERIALS_LIST.split(",") : [],
        });
      }
    }

    let bonStatus = type === 'validate' ? APP_CONSTANTS.INVENTORY_VALIDATION_STAT_YES : APP_CONSTANTS.INVENTORY_VALIDATION_STAT_NO;
    const ligneInfo = documentFormData.LIGNE_INFO;

    if(this.bonInventory && this.__isDocumentValidated) {
      bonStatus = this.bonInventory.status;
    }

    const bonDocument: BonInventory = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      stock_location_id: +documentFormData.STOCK_SOURCE_ID,

      articles: bonArticles,

      status: bonStatus,
    };

    this._modifyInventory(this.bonDocumentNum, bonDocument).then(
      () => {
        if(type === 'validate' || type === 'save_exit') {
          this.returnBack();
        }
      }
    )
  }

  generateReceptionPdf(print?: boolean) {
    if(this.bonDocumentNum === undefined) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KINVENTORY_NO_CONTENT'
      })
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.inventoryService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      }
    });
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_INVENTORY_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  returnBack() {
    this.router.navigate(['/inventory/inventory-list']);
  }

  _getStockLocationList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.commonService.getAllStockList().subscribe({
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.stockLocationList = res.data;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  _getLigneList(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.commonService.getAllLigneList().subscribe({
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  _generateArticles(data: { location_id: number }, page = 1, pageSize = 50) {
    return new Promise((resolve, reject) => {
      this.inventoryService.generateInventoryArticles(data, page, pageSize).subscribe({
        next: (res: CustomHttpResponse<Article[]>) => {
          this.loadingService.hide();
          if(res.status === 'success') {
            this.articlesList = res.data;

            this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
            this.totalRows = res.totalRows ? res.totalRows : this.totalRows;

            this.__generateLineItems(this.articlesList);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    });
  }

  __generateLineItems(articles: Article[]) {
    if(articles !== undefined && articles !== null && articles.length > 0) {
      const formArr = this.bonDocumentForm.get('LINE_ITEMS') as FormArray;
      formArr.clear();

      articles.forEach((row) => {
        const lineItem = this.formBuilder.group({
          ARTICLE_ID: ["", Validators.required],
          REFERENCE: [{ value: "", disabled: true }, Validators.required],
          LABEL: [{ value: "", disabled: true }, Validators.required],

          QTE: [{ value: "0", disabled: true }],

          PRIX_UNIT: ['', [Validators.required]],
          PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required]],
          AVAIL_STOCK: [{ value: "0", disabled: true }],

          SERIALS_LIST: [''],
          REMOVED_SERIALS_LIST: [''],
          OLD_SERIALS_LIST: [''],
        });

        let unitPrix = 0;
        let totalPrix = 0;
        let quantityActual = 0;
        let quantityStock = 0;

        let serialList = "",
          removedSerials = "",
          oldSerialsList = "";

        if(row?.public_prix !== undefined && row?.public_prix !== null) {
          unitPrix = +row?.public_prix;
        }

        if(row?.avail_stock !== undefined && row?.avail_stock !== null) {
          quantityStock = +row?.avail_stock;
        }

        if(row.serial_list !== undefined && row.serial_list !== null) {
          oldSerialsList = row.serial_list;
        }

        const bonItem = this.bonInventoryItems.find((item) => item.article_id == row.id);

        if(bonItem !== undefined && bonItem !== null) {
          quantityActual = bonItem.qte;
          unitPrix = bonItem.unit_price;

          if(bonItem.article_serials !== undefined && bonItem.article_serials !== null && bonItem.article_serials.length > 0) {
            serialList = bonItem.article_serials.join(',');
          }

          if(bonItem.removed_serials !== undefined && bonItem.removed_serials !== null && bonItem.removed_serials.length > 0) {
            removedSerials = bonItem.removed_serials.join(',');
          }
        }

        totalPrix = +unitPrix * +quantityActual;

        lineItem.patchValue({
          ARTICLE_ID: row.id,
          REFERENCE: row.reference,
          LABEL: row.label,

          QTE: quantityActual.toString(),

          PRIX_UNIT: unitPrix.toString(),
          PRIX_TOTAL: totalPrix.toString(),
          AVAIL_STOCK: quantityStock.toString(),

          SERIALS_LIST: serialList,
          REMOVED_SERIALS_LIST: removedSerials,
          OLD_SERIALS_LIST: oldSerialsList,
        });

        formArr.push(lineItem);
      });
    }

    console.log(this.bonDocumentForm.get('LINE_ITEMS'));
  }

  _getBonInventoryDetail(document_no: string | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined || document_no === null || document_no === '') {
        reject();
        return;
      }

      this.bonDocumentForm.get("STOCK_SOURCE_ID")?.disable({ onlySelf: true });

      this.inventoryService.getInventoryDetail(document_no).subscribe( {
        next: (res: CustomHttpResponse<BonInventory>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonInventoryData = res.data;
            this.bonInventory = bonInventoryData;
            if(!this.rolePermission['3_16'] && this.bonInventory.status === APP_CONSTANTS.INVENTORY_VALIDATION_STAT_YES) {
              this.dialogService.displayAlert({
                title: this.translateService.instant('KERROR'),
                message: this.translateService.instant('KACCESS_DENIED')
              }).then(() => {
                this.returnBack();
              });

              return;
            }
            this.__refreshBonDetails(bonInventoryData);
            resolve(res);
          } else {
            // If the bon reception not available
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/inventory/create']);
              }
            );
            reject(res);
          }
        },
        error: err => {
          this.loadingService.hide();

          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  __refreshBonDetails(bonInventory: BonInventory) {
    if(bonInventory !== undefined && bonInventory !== null) {
      this.bonDocumentNum = bonInventory.document_no;
      this.bonInventoryItems = bonInventory.articles;
      this.bonInventoryItemArticles = bonInventory.custom_articles ? bonInventory.custom_articles : [];

      this.bonDocumentForm.get("NUMBON")?.setValue(bonInventory.document_no);
      this.bonDocumentForm.get("DATE_DU_BON")?.setValue(moment(bonInventory.dated).format(CUSTOM_DATE_FORMAT));
      this.bonDocumentForm.get("STOCK_SOURCE_ID")?.setValue(bonInventory.stock_location_id);

      // Disabling the Stock location dropdown after one article is added to the document as a document should be assigned to one Stock location only
      this.bonDocumentForm.get("STOCK_SOURCE_ID")?.disable({ onlySelf: true });

      if(bonInventory.ligne_lbl !== undefined && bonInventory.ligne_lbl !== null && bonInventory.ligne_lbl !== "") {
        if(bonInventory.ligne_id === undefined || bonInventory.ligne_id === null || bonInventory.ligne_id === "") {
          // Custom Ligne Input
          this.customLigneInput = true;
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue('custom-ligne');
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonInventory.ligne_lbl);
        } else {
          // Chosen Ligne Input from the dropdown
          const currentLigneItem = this.ligneList.find(r => r.LGIID == bonInventory.ligne_id)
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue(currentLigneItem);
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(currentLigneItem?.LGILBL);
        }
      }
    }
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  _modifyInventory(document_no: string | undefined, bonDocument: BonInventory): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined) {
        reject("INVALID_DOCUMENT_NO");
        return;
      }

      this.inventoryService.modifyInventory(document_no, bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonInventory>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonInventoryData = res.data;
            this.bonInventory = bonInventoryData;
            this.__updateListCache(bonInventoryData);
            this.__refreshBonDetails(bonInventoryData);

            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });

            reject(res);
          }
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  pageChanged(event: PageEvent) {
    const location_id = this.bonDocumentForm.get('STOCK_SOURCE_ID')?.value;
    const data = {
      location_id
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this.loadingService.show();
    this._generateArticles(data, this.currentPage, this.pageSize);
  }

  changeQuantity(i: number, prixUnit: any) {
    let listSerials: any[] = [],
      oldListSerials: any[] = [],
      removedListSerials: any[] = [];

    const article_id = this.lineItems.at(i).get("ARTICLE_ID")?.value;
    const articleDetail = this.articlesList?.find(r => r.id == article_id);

    const itemSerials = this.lineItems.at(i).get("SERIALS_LIST")?.value;
    if(itemSerials !== undefined && itemSerials !== null && itemSerials !== "") {
      listSerials = itemSerials.split(",");
    }

    listSerials = this.commonService.formatSerialNumbers(listSerials);

    const itemOldSerials = this.lineItems.at(i).get("OLD_SERIALS_LIST")?.value;
    if(itemOldSerials !== undefined && itemOldSerials !== null && itemOldSerials !== "") {
      oldListSerials = itemOldSerials.split(",");
    }

    oldListSerials = this.commonService.formatSerialNumbers(oldListSerials);

    const itemRemovedSerials = this.lineItems.at(i).get("REMOVED_SERIALS_LIST")?.value;
    if(itemRemovedSerials !== undefined && itemRemovedSerials !== null && itemRemovedSerials !== "") {
      removedListSerials = itemRemovedSerials.split(",");
    }

    removedListSerials = this.commonService.formatSerialNumbers(removedListSerials);

    const blockedInSerials = articleDetail?.blocked_in_serials;
    const blockedOutSerials = articleDetail?.blocked_out_serials;

    if(this.bonInventory && this.__isDocumentValidated) {
      oldListSerials = oldListSerials.filter((row: any) => {
        const exists: NumeroSerialList = listSerials.find((_row: NumeroSerialList) => _row.serial_no == row.serial_no);
        if(exists !== undefined && exists !== null) {
          return false;
        }

        return true;
      })

      removedListSerials = removedListSerials.map((row) => {
        const exists = oldListSerials.find((_row: NumeroSerialList) => _row.serial_no == row.serial_no);
        if(exists !== undefined && exists !== null) {
          row.disabled = true;
        }

        return row;
      });
    }

    this.openUpSerialNumPopup(i, {
      newListSerials: listSerials,
      removeListSerials: removedListSerials,
      oldListSerials: oldListSerials,
      block_in_serials: blockedInSerials,
      block_out_serials: blockedOutSerials
    }, prixUnit);
  }

  openUpSerialNumPopup(i: number, data: {}, prixUnit: any) {
    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: {
          numeroType:'inventory-numero-series',
          ...data
        }
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        console.log(result.data);
        if(result.data && result.data !== '') {
          const { newSerials, removedSerials } = result.data;

          this.lineItems.at(i).get('QTE')?.setValue(newSerials.length.toString());
          this.lineItems.at(i).get('SERIALS_LIST')?.setValue(newSerials.join(','));
          this.lineItems.at(i).get('REMOVED_SERIALS_LIST')?.setValue(removedSerials.join(','));

          prixUnit.select();
          prixUnit.focus();

          this.calculatePrixTotal(i);
        }
      }
    });
  }

  calculatePrixTotal(i: number) {
    const formData = (this.lineItems.at(i) as FormGroup).getRawValue();

    let totalPrix = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      this.lineItems.at(i).get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      this.lineItems.at(i).get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    this.lineItems.at(i).get("PRIX_TOTAL")?.setValue(totalPrix.toFixed(2));
  }

  openHistorique(){
    const formData = this.bonDocumentForm.getRawValue();
    const data = {
      document_no: formData.NUMBON,
      document_type: APP_CONSTANTS.DOCUMENT_TYPE_INVENTORY
    };
    this.dialog.open(HistoriqueComponent,
      {
        minWidth: '80%',
        maxHeight: '90vh',
        data: data || {}
      }
    );
  }

}
