import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { PageEvent } from '@angular/material/paginator';
import { DataSharingService} from '../../../_services/data-sharing.service';
import { InventoryService} from '../../../_services/inventory.service';
import {CustomHttpResponse} from "../../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {InventoryList} from "../../../_models/inventory";
import {lastValueFrom} from "rxjs";
import {TranslateService} from "@ngx-translate/core";
import {DialogService} from "../../../_services/dailog.service";
import { DashboardService } from 'src/app/_services/dashboard.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';

@Component({
  selector: 'app-inventory-list',
  templateUrl: './inventory-list.component.html',
  styleUrls: ['./inventory-list.component.scss']
})
export class InventoryListComponent implements OnInit {

  searchForm: FormGroup = this.formbuilder.group({
    document_no: [''],
    start_date: [{ value: '', disabled: true }],
    end_date: [{ value: '', disabled: true }],
    status:['']
  });

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  inventoryList: InventoryList[] | undefined;

  statusList = APP_CONSTANTS.INVENTORY_STATUS;

  displayedColumnsInventory: string[] = ['ITDOCNO', 'ITDOCDATE', 'STLLBL', 'LGILBL', 'VALIDATED', 'actions'];
  rolePermission:any;
  constructor(private formbuilder: FormBuilder,
              private dataSharingService: DataSharingService,
              private inventoryService:InventoryService,
              private translateService: TranslateService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private dashboardService: DashboardService) {
    }

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    const statusValue = this.dashboardService.dashboardStatusValue;
    this.dashboardService.dashboardStatusValue = '';
    if(statusValue){
      this.searchForm.get('status')?.setValue(parseInt(statusValue));
      this.searchFormSubmitter()
      return;
    }
    this.checkForListCache();
  }

  documentStatus(item: InventoryList) {
    const statusList = APP_CONSTANTS.INVENTORY_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.INVENTORY_VALIDATION_STAT_YES:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.INVENTORY_VALIDATION_STAT_NO:
        result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        document_no:  formCache.document_no,
        start_date:  formCache.start_date,
        end_date:  formCache.end_date,
        status: formCache.status
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST);
      if(dataCache !== undefined) {

        this.inventoryList = dataCache.data as InventoryList[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  get startDate() {
    return this.searchForm.get('start_date')?.value;
  }

  isDocumentValidated(item: InventoryList): boolean {
    return item.status === APP_CONSTANTS.INVENTORY_VALIDATION_STAT_YES;
  }

  searchFormSubmitter() {
    this.currentPage = 1;
    this._searchInventory(this.currentPage, this.pageSize);
  }

  _searchInventory(page: number, pageSize: number = 10) {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST_FORM, JSON.stringify(formData));
    const payload = {
      document_no:formData.document_no,
      start_date:formData.start_date,
      end_date:formData.end_date,
      status: formData.status
    };

    const observer = {
      next: (res: CustomHttpResponse<InventoryList[]>) => {
        if(res.status === "success") {
          this.inventoryList = res.data;

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.inventoryService.searchInventory(payload, page, pageSize).subscribe(observer);
  }

  __updateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST);
    if(listCache !== undefined) {
      let listInventory: any[] = listCache.data;
      listInventory = (listInventory as InventoryList[]).filter(r => r.document_no != id);

      listCache.data = listInventory;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listInventory.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._searchInventory(this.currentPage, this.pageSize);
  }

  confirmRemoveInventory(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeInventory(document_no: string, item:InventoryList) {
    if((this.isDocumentValidated(item) && !this.rolePermission['3_17']) || (!this.isDocumentValidated(item) && !this.rolePermission['3_14'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveInventory(document_no);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteInventory(document_no);
    }
  }

  async _deleteInventory(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(document_no);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();

        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.inventoryService.deleteInventory(document_no).subscribe(observer);
  }

}
