import {Component, OnInit} from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import {ActivatedRoute, Router} from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import * as moment  from 'moment';
import { animate, state, style, transition, trigger } from '@angular/animations';
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {InventoryService} from "../../../_services/inventory.service";
import {MatDialog} from "@angular/material/dialog";
import {NgxSpinnerService} from "ngx-spinner";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {CustomHttpResponse} from "../../../_models";
import {BonInventory, BonInventoryItem} from "../../../_models/inventory";
import {CommonService} from "../../../_services/common.service";
import {DialogService} from "../../../_services/dailog.service";
import {
  CUSTOM_DATE_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import {firstValueFrom} from "rxjs";
import {NumeroSerieComponent} from '../numero-serie/numero-serie.component';
import {Article} from "../../../_models/article";

@Component({
  selector: 'app-view-inventory',
  templateUrl: './view-inventory.component.html',
  styleUrls: ['./view-inventory.component.scss'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({ height: '0px', minHeight: '0', visibility: 'hidden' })),
      state('expanded', style({ height: '*', visibility: 'visible' })),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class ViewInventoryComponent implements OnInit {

  bonDocumentForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],
    STOCK_SOURCE_ID: [{ value: "", disabled: true }],

    LIGNE_INFO: [{ value: "", disabled: true }], // Ligne ID
    LIGNE_CUSTOM_LABEL: [{ value: "", disabled: true }], // Ligne Label
  })

  bonDocumentNum: string | undefined = "*********";
  bonInventory: BonInventory | undefined;
  bonInventoryItems: BonInventoryItem[] = [];
  bonInventoryItemArticles: Article[] = [];

  totalArticles: number = 0;
  totalQte: number = 0;
  totalPT: number = 0;

  displayedColumns: string[] = [ 'REFERENCE', 'DESC', 'QTE', 'QTE_SUPP', 'QTE_EST', 'PRIX_UNIT', 'PRIX_TOTAL','ACTIONS'];

  customLigneInput: boolean = false;

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  constructor(public dialog: MatDialog,
              private commonService: CommonService,
              private inventoryService: InventoryService,
              private router: Router,
              private route: ActivatedRoute,
              private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService) {
  }

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.bonDocumentForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)
    });

    const params = await firstValueFrom(this.route.queryParams);
    if(params.hasOwnProperty('document_no') && params["document_no"] !== null && params["document_no"] !== "") {
      this.bonDocumentNum = params["document_no"];

      await this._getBonInventoryDetail(this.bonDocumentNum);
    }

    await this.loadingService.hide();
  }

  viewItemSerials(item: BonInventoryItem) {
    const itemSelectedSerials = item.article_serials;
    const selectedListSerials = this.commonService.formatSerialNumbers(itemSelectedSerials ? itemSelectedSerials : []);

    const itemRemovedSerials = item.removed_serials;
    const removedListSerials = this.commonService.formatSerialNumbers(itemRemovedSerials ? itemRemovedSerials : []);

    const articleData = this.bonInventoryItemArticles.find((row) => row.id == item.article_id);
    let oldListSerials: any = [];
    if(articleData) {
      const itemOldSerials = articleData.serial_list?.split(",");
      oldListSerials = itemOldSerials ? itemOldSerials : [];
    }

    oldListSerials = this.commonService.formatSerialNumbers(oldListSerials);

    this.openUpSerialNumPopup({
      newListSerials: selectedListSerials,
      oldListSerials: oldListSerials,
      removeListSerials: removedListSerials,
      readonly: true
    });
  }

  openUpSerialNumPopup(data?: any) {
    data.numeroType = "inventory-numero-series";

    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: data || {}
      }
    );
  }

  generateReceptionPdf(print?: boolean) {
    if(this.bonDocumentNum === undefined) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KINVENTORY_NO_CONTENT'
      })
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.inventoryService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      }
    });
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_INVENTORY_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  _getBonInventoryDetail(document_no: string | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined || document_no === null || document_no === '') {
        reject();
        return;
      }

      this.inventoryService.getInventoryDetail(document_no).subscribe( {
        next: (res: CustomHttpResponse<BonInventory>) => {
          if(res.status === "success") {
            const bonInventoryData = res.data;
            this.bonInventory = bonInventoryData;

            this.__refreshBonDetails(bonInventoryData);
          } else {
            // If the bon reception not available
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/inventory/inventory-list']);
              }
            );
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  __refreshBonDetails(bonInventory: BonInventory) {
    if(bonInventory !== undefined && bonInventory !== null) {
      this.bonDocumentNum = bonInventory.document_no;
      this.bonInventoryItems = bonInventory.articles;
      this.bonInventoryItemArticles = bonInventory.custom_articles ? bonInventory.custom_articles : [];

      this.totalArticles = 0;
      this.totalQte = 0;
      this.totalPT = 0;

      this.bonInventoryItems.forEach((row) => {
        this.totalArticles += 1;
        this.totalQte += +row.qte;
        this.totalPT += +row.total_price;
      })

      this.bonDocumentForm.get("NUMBON")?.setValue(bonInventory.document_no);
      this.bonDocumentForm.get("DATE_DU_BON")?.setValue(moment(bonInventory.dated).format(CUSTOM_DATE_FORMAT));
      this.bonDocumentForm.get("STOCK_SOURCE_ID")?.setValue(bonInventory.stock_location_lbl);

      if(bonInventory.ligne_lbl !== undefined && bonInventory.ligne_lbl !== null && bonInventory.ligne_lbl !== "") {
        this.bonDocumentForm.get("LIGNE_INFO")?.setValue(bonInventory.ligne_lbl);
      }
    }
  }

  __calculateItemEstQte(item: BonInventoryItem): number {
    let total = 0;

    total = item.qte_stock;
    total -= item.removed_serials ? item.removed_serials.length : 0;
    total += item.qte;

    return total;
  }

}
