import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {LigneInfo} from "../../../_models/ligne";
import {firstValueFrom} from "rxjs";
import {CustomHttpResponse} from "../../../_models";
import {LigneService} from "../../../_services/ligne.service";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-edit-ligne',
  templateUrl: './edit-ligne.component.html',
  styleUrls: ['./edit-ligne.component.scss']
})
export class EditLigneComponent implements OnInit {

  submitted: boolean = false;
  modifyLigneForm: FormGroup = this.formBuilder.group({
    LGISTAT: new FormControl('', [Validators.required]),
    LGILBL: new FormControl('', [Validators.required, Validators.maxLength(APP_CONSTANTS.LIGNE_LABEL_MAX_LENGTH), Validators.minLength(APP_CONSTANTS.LIGNE_LABEL_MIN_LENGTH)]),
  });

  ligne_id: number | undefined;
  ligneDetail: LigneInfo | undefined;

  statusList = APP_CONSTANTS.LIGNE_INFO_STATUS;

  readOnly: boolean = false;
  pageTitle: string = 'KMODIFY_LIGNE';

  constructor(private ligneService: LigneService,
              private formBuilder: FormBuilder,
              private router: Router,
              private route: ActivatedRoute,
              private translateService: TranslateService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private dataSharingService:DataSharingService) {
  }

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.ligne_id = +params['ligne_id'];
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];

        if(this.readOnly) {
          this._changeToDetailMode();
        }
      }
    })
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this._getLigneInfoDetail(this.ligne_id);
    this._prefillFormData();

    await this.loadingService.hide();
  }

  _changeToDetailMode() {
    this.pageTitle = 'KDETAIL_LIGNE';

    this.modifyLigneForm.get('LGISTAT')?.disable({ onlySelf: true });
    this.modifyLigneForm.get('LGILBL')?.disable({ onlySelf: true });
  }

  _ligneIsActive(status: number | undefined): boolean {
    if(status === APP_CONSTANTS.LIGNE_STATUS_ACTIVE) {
      return true;
    }

    return false;
  }

  get fr() {
    return this.modifyLigneForm.controls;
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIGNE_LIST);
    if(listCache !== undefined) {
      let listLigneInfo = listCache.data as LigneInfo[];
      listLigneInfo = listLigneInfo.map((row) => {
        if(row.LGIID == this.ligne_id) {
          row.LGIID = this.ligne_id;
          row.LGILBL = payload.LGILBL;
          row.LGISTAT = payload.LGISTAT;
        }
        return row;
      });

      listCache.data = listLigneInfo;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST, JSON.stringify(listCache));
    }
  }

  _getLigneInfoDetail(ligne_id: number | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(ligne_id === undefined || ligne_id === null) {
        reject();
        return;
      }

      const observer = {
        next: async (res: CustomHttpResponse<LigneInfo>) => {
          if(res.status === "success") {
            this.ligneDetail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            // If the Ligne Info not available
            await this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });

            this.router.navigate(['/options/ligne/list']);

            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          reject(err);
        }
      };

      this.ligneService.getLigneDetail(ligne_id).subscribe(observer);
    })
  }

  _prefillFormData() {
    this.modifyLigneForm.patchValue({
      LGISTAT: this.ligneDetail?.LGISTAT,
      LGILBL: this.ligneDetail?.LGILBL,
    })
  }

  modifyLigneFormSubmitter() {
    if(this.readOnly) {
      return;
    }

    this.submitted = true;
    if(this.modifyLigneForm.invalid) {
      return;
    }

    const formData = this.modifyLigneForm.getRawValue();
    const payload = {
      LGILBL: formData.LGILBL,
      LGISTAT: formData.LGISTAT
    };

    this._modifyLigneInfo(payload);
  }

  async _modifyLigneInfo(payload: { LGILBL: string, LGISTAT: number }) {
    if(this.ligne_id === undefined || this.ligne_id === null) {
      return;
    }

    const observer = {
      next: async (res: CustomHttpResponse<LigneInfo>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/options/ligne/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.ligneService.modifyLigne(this.ligne_id, payload).subscribe(observer);
  }

}
