import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import {PageEvent} from '@angular/material/paginator';
import {LigneService} from "../../../_services/ligne.service";
import {CustomHttpResponse} from "../../../_models";
import {LigneInfo} from "../../../_models/ligne";
import {NgxSpinnerService} from "ngx-spinner";
import {lastValueFrom} from "rxjs";
import {DialogService} from "../../../_services/dailog.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";

import { TranslateService } from '@ngx-translate/core';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-ligne-list',
  templateUrl: './ligne-list.component.html',
  styleUrls: ['./ligne-list.component.scss']
})
export class LigneListComponent implements OnInit {

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  ligneList: LigneInfo[] | undefined;
  displayedColumnsLigne: string[] = ['LGILBL', 'LGISTAT', 'ACTIONS'];

  searchForm: FormGroup = this.formBuilder.group({
    keyword: [''],
  });
  rolePermission:any;
  constructor(private formBuilder: FormBuilder,
              private router: Router,
              private ligneService: LigneService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService:TranslateService,
              private dataSharingService:DataSharingService) {
  }

  ngOnInit(): void {
    this.checkForListCache();
    this.loadDependencies();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_LIGNE_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        keyword: formCache.keyword
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIGNE_LIST);
      if(dataCache !== undefined) {

        this.ligneList = dataCache.data as LigneInfo[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    await this.loadingService.hide();
  }

  get fr() {
    return this.searchForm.controls;
  }

  documentStatus(item: LigneInfo) {
    const statusList = APP_CONSTANTS.LIGNE_INFO_STATUS;

    const statusData = statusList.find((r) => r.value == item.LGISTAT);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.LGISTAT) {
      case APP_CONSTANTS.LIGNE_STATUS_ACTIVE:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.LIGNE_STATUS_IN_ACTIVE:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST_FORM, JSON.stringify(formData));
    const payload = {
      keyword: formData.keyword
    };

    this.currentPage = 1;

    this._searchLigneInfo(payload, this.currentPage, this.pageSize);
  }

  _searchLigneInfo(data: { keyword: string }, page: number = 1, pageSize: number = 10) {
    const observer = {
      next: (res: CustomHttpResponse<LigneInfo[]>) => {
        if(res.status === "success") {
          this.ligneList = res.data;

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.ligneService.searchLigneInfo(data, page, pageSize).subscribe( observer);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIGNE_LIST);
    if(listCache !== undefined) {
      let listLigne: any[] = listCache.data;
      listLigne = (listLigne as LigneInfo[]).filter(r => r.LGIID != id);

      listCache.data = listLigne;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listLigne.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      keyword: formData.keyword
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._searchLigneInfo(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveLigne(ligne_label: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: ligne_label })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeLigne(ligne_id: number, ligne_label: string) {
    if(!this.rolePermission['9_34']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveLigne(ligne_label);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteLigneInfo(ligne_id);
    }
  }

  async _deleteLigneInfo(ligne_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(ligne_id);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.ligneService.deleteLigne(ligne_id).subscribe(observer);
  }

}
