import { Component, OnInit } from '@angular/core';
import {FormGroup, Validators, FormBuilder} from '@angular/forms';
import { AuthService } from '../../_services/auth.service';
import { TokenStorageService } from '../../_services/token-storage.service';
import { Router } from '@angular/router';
import { DataSharingService } from '../../_services/data-sharing.service';
import { TranslateService } from '@ngx-translate/core';
import {NgxSpinnerService} from "ngx-spinner";
import {CustomHttpResponse} from "../../_models";
import {DialogService} from "../../_services/dailog.service";
import {APP_CONSTANTS} from "../../_helpers/constants";

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['../../sass/_login.scss']
})
export class LoginComponent implements OnInit {

  submitted: boolean = false;
  loginForm: FormGroup = this.formBuilder.group({
    username: ['', [Validators.required]],
    password: ['', [Validators.required]]
  });

  constructor(private authService:AuthService,
              private formBuilder: FormBuilder,
              private tokenStorage: TokenStorageService,
              private router: Router,
              private dataSharingService: DataSharingService,
              private translateService:TranslateService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService) {
  }

  ngOnInit(): void {
  }

  get fr() {
    return this.loginForm.controls;
  }

  loginFormSubmitter() {
      this.submitted = true;
      if(this.loginForm.invalid) {
        return;
      }

      this.loadingService.show();
      const formData = this.loginForm.getRawValue();
      const payload = {
        username: formData.username,
        password: formData.password
      };

      this._userLogIn(payload);
  }

  _userLogIn(payload: { username: string, password: string }) {
    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === "success") {
          if(res?.token && res?.refresh_token) {
            this.tokenStorage.saveToken(res?.token);
            this.tokenStorage.saveRefreshToken(res?.refresh_token);

            this.tokenStorage.saveUser(res.data);

            this._getUserRolePermission();
            this.dataSharingService.isUserLoggedIn.next(true);

            this.router.navigate([APP_CONSTANTS.HOME_ROUTE]);
          }
        } else {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });

          return;
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        console.log(err.status)
        if(err.status == 401){
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'INCORRECT_LOGIN_DETAIL'
          });
        }else{
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
      }
    };

    this.authService.login(payload).subscribe(observer);
  }

  _getUserRolePermission() {
    const userRolePermissions = this.tokenStorage.getUser().user_role_permission;

    let processed: any = {};
    for (let role of userRolePermissions) {
      processed[`${role.id}_${role.value}`] = role.status;
      let child = role.child;
      if (child != null) {
        for (let child_role of child) {
          processed[`${child_role.id}_${child_role.value}`] = child_role.status;
          let sub_child = child_role.child;
          if (sub_child != null) {
            for (let sub_child_role of sub_child) {
              processed[`${sub_child_role.id}_${sub_child_role.value}`] = sub_child_role.status;
            }
          }
        }
      }
    }

    localStorage.removeItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    localStorage.setItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY, JSON.stringify(processed));
  }

}
