import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {Router} from "@angular/router";
import {MatDialog} from "@angular/material/dialog";
import {NgxSpinnerService} from "ngx-spinner";
import {TranslateService} from "@ngx-translate/core";
import {DialogService} from "../../_services/dailog.service";
import {PageEvent} from "@angular/material/paginator";
import {
  CustomHttpResponse,
  QuickSearchArticle,
  QuickSearchContact, QuickSearchDocument,
  StockLocationList
} from "../../_models";
import {CommonService} from "../../_services/common.service";
import {QuickSearchService} from "../../_services/quick-search.service";
import {APP_CONSTANTS} from "../../_helpers/constants";
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-quick-search',
  templateUrl: './quick-search.component.html',
  styleUrls: ['./quick-search.component.scss']
})
export class QuickSearchComponent implements OnInit {

  @Output() quickEvent = new EventEmitter();
  @Input() flowType: string | undefined ='quick_search';

  quickSearchType: 'contact' | 'article' | 'document' = 'article';
  quickSearchForm: FormGroup = this.formBuilder.group({
    SEARCH_TYPE: [this.quickSearchType, Validators.required],
    KEYWORD: [''],
    LOCATION_ID: [''],
  });

  quickSearchTypeList: { label: string, value: string }[] = [
    {
      label: "KQUICK_SEARCH_CONTACT",
      value: "contact"
    },
    {
      label: "KQUICK_SEARCH_ARTICLE",
      value: "article"
    },
    {
      label: "KQUICK_SEARCH_DOCUMENT",
      value: "document"
    },
  ];

  listStockLocation: StockLocationList[] | undefined;

  listData: any | undefined;

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  displayedColumns: string[] | undefined;
  displayedColumnsArticle: string[] = ['REFERENCE', 'LABEL', 'STOCK', 'ACTIONS'];
  displayedColumnsContact: string[] = ['NOM', 'ADDRESS', 'TELEPHONE', 'ACTIONS'];
  displayedColumnsDocument: string[] = ['DOCUMENT_DATE', 'DOCUMENT_NO', 'STORE_NOM', 'TOTAL_PRICE', 'STATUS', 'ACTIONS'];

  constructor(private formBuilder: FormBuilder,
              private commonService: CommonService,
              private quickSearchService: QuickSearchService,
              private router:Router,
              public dialog: MatDialog,
              private loadingService: NgxSpinnerService,
              private translateService:TranslateService,
              private dialogService: DialogService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    this.checkForListCache();
    this.loadDependencies();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST_FORM);
    if(formCache !== undefined) {
      this.quickSearchForm.patchValue({
        SEARCH_TYPE:formCache.SEARCH_TYPE,
        KEYWORD: formCache.KEYWORD,
        LOCATION_ID: formCache.LOCATION_ID,
      });
      this._refreshDisplayColumns(formCache.SEARCH_TYPE);
      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST);
      if(dataCache !== undefined) {
        if(formCache.SEARCH_TYPE === 'contact') {
          this.listData = dataCache.data as QuickSearchContact[];
        }

        if(formCache.SEARCH_TYPE === 'article') {
          this.listData = dataCache.data as QuickSearchArticle[];
        }

        if(formCache.SEARCH_TYPE === 'document') {
          this.listData = dataCache.data as QuickSearchDocument[];
        }

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.onQuickSearchFormSubmitter();
    }
  }

  _refreshDisplayColumns(type: 'contact' | 'article' | 'document') {
    if(type === 'contact') {
      this.displayedColumns = this.displayedColumnsContact;
      return;
    }

    if(type === 'article') {
      this.displayedColumns = this.displayedColumnsArticle;
      return;
    }

    if(type === 'document') {
      this.displayedColumns = this.displayedColumnsDocument;
      return;
    }
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getStockLocationList();
    this.getDashboardData();

    await this.loadingService.hide();
  }

  get fr() {
    return this.quickSearchForm.controls;
  }

  get quickSearchFormType(): string {
    return this.quickSearchForm.get('SEARCH_TYPE')?.value;
  }

  getDashboardData(){
    if(this.quickSearchService.dashboardFlag == true){
      const data = this.quickSearchService.quickSearchFormData;
      this.quickSearchForm.patchValue({
        SEARCH_TYPE: data.SEARCH_TYPE,
        KEYWORD: data.KEYWORD,
        LOCATION_ID: data.LOCATION_ID,
      });
      this.onQuickSearchFormSubmitter();
    }
  }

  documentStatus(row: QuickSearchDocument) {
    const result = {
      statusClass: "",
      statusText: ""
    };

    switch (row.status) {
      case APP_CONSTANTS.DOCUMENT_QUICK_SEARCH_VALIDATION_YES:
        result.statusClass = "bg-success";
        if(row.document_type === 'reception') {
          result.statusText = "KRECEPTION_STATUS_VALIDATED";
        } else {
          result.statusText = "KLIVRAISON_STATUS_VALIDATED";
        }
        break;
      case APP_CONSTANTS.DOCUMENT_QUICK_SEARCH_VALIDATION_NO:
        result.statusClass = "bg-warning";
        if(row.document_type === 'reception') {
          result.statusText = "KRECEPTION_STATUS_IN_PROGRESS";
        } else {
          result.statusText = "KLIVRAISON_STATUS_IN_PROGRESS";
        }
        break;
    }
    return result;
  }

  getStockLocationList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const observer = {
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.listStockLocation = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getStockLocationList().subscribe( observer);
    })
  }

  moveQuickSearch(){
    this.quickSearchService.dashboardFlag = true;
    this.quickSearchService.quickSearchFormData = this.quickSearchForm.value;
    this.router.navigate(['/quick-search']);
  }

  onQuickSearchFormSubmitter() {
    if(this.quickSearchForm.invalid) {
      return;
    }
    const formData = this.quickSearchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST_FORM, JSON.stringify(formData));
    this.currentPage = 1;

    const payload: any = {
      keyword: formData.KEYWORD,
    };

    this.quickSearchType = formData.SEARCH_TYPE;

    if(this.quickSearchType === 'article') {
      payload.location_id = formData.LOCATION_ID;
      this.displayedColumns = this.displayedColumnsArticle;

      this._searchArticles(payload, this.currentPage, this.pageSize);
      return;
    }

    if(this.quickSearchType === 'contact') {
      this.displayedColumns = this.displayedColumnsContact;

      this._searchContacts(payload, this.currentPage, this.pageSize);
      return;
    }

    if(this.quickSearchType === 'document') {
      this.displayedColumns = this.displayedColumnsDocument;

      this._searchDocuments(payload, this.currentPage, this.pageSize);
      return;
    }
  }

  _searchContacts(payload: { keyword: string }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<QuickSearchContact[]>) => {
        if(res.status === "success") {
          this.listData = res.data as QuickSearchContact[];

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.quickSearchService.searchContactsList(payload, page, pageSize).subscribe(observer);
  }

  _searchArticles(payload: { keyword: string, location_id: number }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<QuickSearchArticle[]>) => {
        if(res.status === "success") {
          this.listData = res.data as QuickSearchArticle[];

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.quickSearchService.searchArticlesList(payload, page, pageSize).subscribe(observer);
  }

  _searchDocuments(payload: { keyword: string }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<QuickSearchDocument[]>) => {
        if(res.status === "success") {
          this.listData = res.data as QuickSearchDocument[];

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.quickSearchService.searchDocumentsList(payload, page, pageSize).subscribe(observer);
  }

  pageChanged(event: PageEvent) {
    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex+1;

    const formData = this.quickSearchForm.getRawValue();

    const payload: any = {
      keyword: formData.KEYWORD,
    };

    if(formData.SEARCH_TYPE === 'article') {
      payload.location_id = formData.LOCATION_ID;
      this.displayedColumns = this.displayedColumnsArticle;

      this._searchArticles(payload, this.currentPage, this.pageSize);
      return;
    }

    if(formData.SEARCH_TYPE === 'contact') {
      this.displayedColumns = this.displayedColumnsContact;

      this._searchContacts(payload, this.currentPage, this.pageSize);
      return;
    }

    if(formData.SEARCH_TYPE === 'document') {
      this.displayedColumns = this.displayedColumnsDocument;

      this._searchDocuments(payload, this.currentPage, this.pageSize);
      return;
    }
  }

  onViewItemClicked(row: any) {
    if(this.quickSearchType === 'document') {
      const item = row as QuickSearchDocument;

      if(item.document_type === 'reception') {
        this.router.navigate(['/bon-de-reception/detail', item.document_no]);
      } else {
        this.router.navigate(['/bon-de-livraison/detail', item.document_no]);
      }

      return;
    }

    if(this.quickSearchType === 'contact') {
      const item = row as QuickSearchContact;

      this.router.navigate(['/contact/detail', item.contact_id]);
      return;
    }

    if(this.quickSearchType === 'article') {
      const item = row as QuickSearchArticle;

      this.router.navigate(['/article/view-article', item.id]);
      return;
    }
  }

}
