import { Component, OnInit, ViewChild } from '@angular/core';
import { AbstractControl, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatTable } from '@angular/material/table';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { ReportingPlanificationService } from 'src/app/_services/reporting-planification.service';

@Component({
  selector: 'app-reporting-planification-add',
  templateUrl: './reporting-planification-add.component.html',
  styleUrls: ['./reporting-planification-add.component.scss']
})
export class ReportingPlanificationAddComponent implements OnInit {

  planificationEmail:any = [];

  displayedColumnsStorage:string[] = ['EMAIL']

  semaines = APP_CONSTANTS.SEMAINES_DAYS;
  month_names =  APP_CONSTANTS.MOIS;
  month_days = APP_CONSTANTS.MOIS_DAY;
  statusList = APP_CONSTANTS.REPORTING_PLANIFICATION_STATUS;

  reportingPlanForm: FormGroup = this.formBuilder.group({
    title: ['', [Validators.required]],
    period: ['', [Validators.required]],
    hour: ['', [Validators.required]],
    status: ['', [Validators.required]]
  });

  emailForm: FormGroup = this.formBuilder.group({
    email: ['', [Validators.required,Validators.pattern(APP_CONSTANTS.EMAIL_PATTERN)]],
  });

  @ViewChild(MatTable) table!: MatTable<any>;

  submitted = false;

  constructor(private formBuilder: FormBuilder,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private translateService: TranslateService,
              private router: Router,
              private reportingPlanificationService:ReportingPlanificationService,
              private dataSharingService: DataSharingService) {}

  ngOnInit(): void {
    this.loadingService.show();

    this.clearAllCheckBox()

    this.loadingService.hide();
  }

  clearAllCheckBox(){
    this.semaines.map((week)=>{
      if(week.status == true){
        week.status = false
      }
    })
    this.month_names.map((month_name)=>{
      if(month_name.status == true){
        month_name.status = false
      }
    })
    this.month_days.map((month_day)=>{
      if(month_day.status == true){
        month_day.status = false
      }
    })
  }

  get periodData(){
    return this.reportingPlanForm.get('period');
  }

  get f(): { [key: string]: AbstractControl } {
    return this.emailForm.controls;
  }

  addEmail(){
    this.submitted = true;
    console.log(this.emailForm.value.email)
    if(this.emailForm.invalid || this.emailForm.value.email == null || this.emailForm.value.email == ''){
      return
    }
    this.planificationEmail.push(this.emailForm.value.email);
    this.table.renderRows();
    this.submitted = false;
    this.emailForm.reset();
  }

  reportingFormSubmit(){
    if(this.reportingPlanForm.invalid || this.reportingPlanificationService.reportingFormData.invalid || this.planificationEmail.length == 0){
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }
    this.processReportPlanData();
  }

  processReportPlanData(){
    const reporting = this.reportingPlanificationService.reportingFormData.getRawValue();
    const formdata = this.reportingPlanForm.getRawValue();

    const payload = {
    title: formdata.title,
    period: formdata.period,
    hour: formdata.hour,
    variants: [],
    _r_brand_id: reporting.BRAND_ID,
    _r_store_id: reporting.STORE_ID,
    _r_store_type: reporting.STORE_TYPE,
    _r_report_type: reporting.REPORT_TYPE,
    _r_movement_type: reporting.STOCK_MOVEMENT_TYPE,
    _r_start_date: reporting.START_DATE,
    _r_end_date: reporting.END_DATE,
    _r_articles: reporting.REF_ARTICLES,
    _r_columns: reporting.REPORT_COLUMNS,
    emails: this.planificationEmail,
    status: formdata.status
    }

    if(this.periodData?.value === 'week'){
      const semaine:any =[];
      this.semaines.map((data:any)=>{
        if(data.status == true){
          semaine.push(data.label);
        }
      })
      payload.variants = semaine;
    }else if(this.periodData?.value === 'month'){
      const variants:any = {
        month_names: [],
        month_days: []
      }
      this.month_names.map((data:any)=>{
        if(data.status == true){
          variants.month_names.push(data.label);
        }
      })
      this.month_days.map((data:any)=>{
        if(data.status == true){
          variants.month_days.push(data.label);
        }
      })
      payload.variants = variants;
    }
    this.createReportPlanification(payload);

  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST_FORM);
  }

  async createReportPlanification(payload:any){
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/reporting-planification/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }
    this.reportingPlanificationService.createReportingPlanification(payload).subscribe(observer);
  }

  removeEmail(index:number){
    this.planificationEmail.splice(index, 1);
    this.table.renderRows();
  }

}
