import { Component, OnInit, ViewChild } from '@angular/core';
import { AbstractControl, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatTable } from '@angular/material/table';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse } from 'src/app/_models';
import { ReportingPlanification } from 'src/app/_models/reporting-planification';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { ReportingPlanificationService } from 'src/app/_services/reporting-planification.service';

@Component({
  selector: 'app-reporting-planification-edit',
  templateUrl: './reporting-planification-edit.component.html',
  styleUrls: ['./reporting-planification-edit.component.scss']
})
export class ReportingPlanificationEditComponent implements OnInit {

  planificationEmail:any = [];
  repotingPlanData:any=[];

  displayedColumnsStorage:string[] = ['EMAIL']
  pageTitle = 'KMODIFYREPPLAN';

  semaines = APP_CONSTANTS.SEMAINES_DAYS;
  month_names =  APP_CONSTANTS.MOIS;
  month_days = APP_CONSTANTS.MOIS_DAY;
  statusList = APP_CONSTANTS.REPORTING_PLANIFICATION_STATUS;

  disabledCheck:boolean = false;
  submitted :boolean = false;
  readOnly :boolean = false;
  plan_id : any;


  editReportingPlanForm: FormGroup = this.formBuilder.group({
    title: ['', [Validators.required]],
    period: ['', [Validators.required]],
    hour: ['', [Validators.required]],
    status: ['', [Validators.required]]
  });

  emailForm: FormGroup = this.formBuilder.group({
    email: ['', [Validators.required,Validators.pattern(APP_CONSTANTS.EMAIL_PATTERN)]],
  });

  @ViewChild(MatTable) table!: MatTable<any>;

  constructor(private formBuilder: FormBuilder,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private translateService: TranslateService,
              private router: Router,
              private reportingPlanificationService:ReportingPlanificationService,
              private route: ActivatedRoute,
              private dataSharingService: DataSharingService) {}

  ngOnInit(): void {
    this.route.data.subscribe(data => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];
        if(this.readOnly) {
          this.changeToDetailMode();
        }
      }
    });

    this.route.params.subscribe(params => {
      if(params !== undefined && params !== null && params.hasOwnProperty('plan_id')) {
        this.plan_id = params["plan_id"];
        this.loadDependencies();
      }
    });
  }

  async loadDependencies() {
    await this.loadingService.show();
    this.clearAllCheckBox();
    await this._getRepotingPlanificationDetail(this.plan_id);
    this.prefillFormDetail(this.repotingPlanData);
    await this.loadingService.hide();
  }

  clearAllCheckBox(){
    this.semaines.map((week)=>{
      if(week.status == true){
        week.status = false
      }
    })
    this.month_names.map((month_name)=>{
      if(month_name.status == true){
        month_name.status = false
      }
    })
    this.month_days.map((month_day)=>{
      if(month_day.status == true){
        month_day.status = false
      }
    })
  }

  prefillFormDetail(stockLocationData: ReportingPlanification | undefined) {
    if(stockLocationData === undefined || stockLocationData === null) {
      return;
    }
    this.editReportingPlanForm.patchValue({
      title: this.repotingPlanData.title,
      period: this.repotingPlanData.period,
      hour: this.repotingPlanData.hour,
      status: this.repotingPlanData.status
    });
    this.planificationEmail = JSON.parse(this.repotingPlanData.emails);
    const checkBox = JSON.parse(this.repotingPlanData.variants);
    if(checkBox != null){
      if(this.repotingPlanData.period == 'week'){
        const defaultweek = APP_CONSTANTS.SEMAINES_DAYS;
        checkBox.map((selected_week_day:string) =>{
          defaultweek.map((dw)=>{
            if(selected_week_day == dw.label){
              dw.status = true
            }
          })
        })
      }
      if(this.repotingPlanData.period == 'month'){
        const defaultmonth_name = APP_CONSTANTS.MOIS;
        checkBox.month_names.map((month_name:string)=>{
          defaultmonth_name.map((dmn)=>{
            if(month_name == dmn.label){
              dmn.status = true
            }
          })
        })
        const defaultmonth_days = APP_CONSTANTS.MOIS_DAY;
        checkBox.month_days.map((month_day:string)=>{
          defaultmonth_days.map((dmd)=>{
            if(month_day == dmd.label){
              dmd.status = true
            }
          })
        })
      }
    }
  }

  changeToDetailMode() {
    this.pageTitle = 'KDETAILREPPLAN'
    this.disabledCheck = true;
    this.editReportingPlanForm.get('title')?.disable({ onlySelf: true });
    this.editReportingPlanForm.get('period')?.disable({ onlySelf: true });
    this.editReportingPlanForm.get('hour')?.disable({ onlySelf: true });
    this.editReportingPlanForm.get('status')?.disable({ onlySelf: true });
  }

  get periodData(){
    return this.editReportingPlanForm.get('period');
  }

  get f(): { [key: string]: AbstractControl } {
    return this.emailForm.controls;
  }

  _getRepotingPlanificationDetail(plan_id: number | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(plan_id === undefined) {
        resolve('INVALID_LOCATION');
        return;
      }

      const observer = {
        next: (res: CustomHttpResponse<ReportingPlanification>) => {
          this.loadingService.hide();
          if(res.status === 'success') {
            this.repotingPlanData = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(() => {
              this.router.navigate(['/reporting-planification/list']);
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.reportingPlanificationService.getStockLocationDetail(plan_id).subscribe(observer);
    });
  }

  addEmail(){
    this.submitted = true;
    if(this.emailForm.invalid || this.emailForm.value.email == null || this.emailForm.value.email == ''){
      return
    }
    this.planificationEmail.push(this.emailForm.value.email);
    this.table.renderRows();
    this.submitted = false;
    this.emailForm.reset();
  }

  reportingFormSubmit(){
    if(this.editReportingPlanForm.invalid || this.reportingPlanificationService.reportingFormData.invalid || this.planificationEmail.length == 0){
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }

    this._processreportingData();
  }

  _processreportingData(){
    const reporting = this.reportingPlanificationService.reportingFormData.getRawValue();
    const formdata = this.editReportingPlanForm.getRawValue();
    const payload = {
    title: formdata.title,
    period: formdata.period,
    hour: formdata.hour,
    variants: [],
    _r_brand_id: reporting.BRAND_ID,
    _r_store_id: reporting.STORE_ID,
    _r_store_type: reporting.STORE_TYPE,
    _r_report_type: reporting.REPORT_TYPE,
    _r_movement_type: reporting.STOCK_MOVEMENT_TYPE,
    _r_start_date: reporting.START_DATE,
    _r_end_date: reporting.END_DATE,
    _r_articles: reporting.REF_ARTICLES,
    _r_columns: reporting.REPORT_COLUMNS,
    emails: this.planificationEmail,
    status: formdata.status
    }

    if(this.periodData?.value === 'week'){
      const semaine:any =[]
      this.semaines.map((data:any)=>{
        if(data.status == true){
          semaine.push(data.label);
        }
      })
      payload.variants = semaine;
    }else if(this.periodData?.value === 'month'){
      const variants:any = {
        month_names: [],
        month_days: []
      }
      this.month_names.map((data:any)=>{
        if(data.status == true){
          variants.month_names.push(data.label);
        }
      })
      this.month_days.map((data:any)=>{
        if(data.status == true){
          variants.month_days.push(data.label);
        }
      })
      payload.variants = variants;
    }
    this.modifyReportPlanification(payload);
  }


  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST);
    if(listCache !== undefined) {
      let listReportingPlanification = listCache.data as ReportingPlanification[];
      listReportingPlanification = listReportingPlanification.map((row) => {
        if(row.plan_id == this.plan_id) {
          row.plan_id = this.plan_id;
          row.title = payload.title;
          row.period = payload.period;
          row.hour = payload.hour;
          row.status = payload.status;
        }
        return row;
      });

      listCache.data = listReportingPlanification;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST, JSON.stringify(listCache));
    }
  }

  async modifyReportPlanification(payload: any){
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/reporting-planification/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    }
    this.reportingPlanificationService.modifyReportingPlanification(this.plan_id, payload).subscribe(observer);
  }

  removeEmail(index: number){
    this.planificationEmail.splice(index, 1);
    this.table.renderRows();
  }

}
