import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { PageEvent } from '@angular/material/paginator';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { lastValueFrom } from 'rxjs';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse } from 'src/app/_models';
import { ReportingPlanification } from 'src/app/_models/reporting-planification';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { ReportingPlanificationService } from 'src/app/_services/reporting-planification.service';

@Component({
  selector: 'app-reporting-planification-list',
  templateUrl: './reporting-planification-list.component.html',
  styleUrls: ['./reporting-planification-list.component.scss']
})
export class ReportingPlanificationListComponent implements OnInit {

  reportingPlanificationList:ReportingPlanification[]=[];
  displayedColumnsReportPlan:string[] = ['TITLE', 'PERIOD', 'HOUR', 'STATUS', 'ACTION'];

  isLoading = false;
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  searchForm:FormGroup = this.formBuilder.group({
    title: new FormControl('', ),
    period: new FormControl('', )
  });

  rolePermission:any;

  constructor(public dialog: MatDialog,
              public dataSharingService:DataSharingService,
              private dialogService: DialogService,
              private reportingPlanificationService:ReportingPlanificationService,
              private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private translateService:TranslateService) {}

  ngOnInit(): void {

    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.checkForListCache();
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();
    this.searchFormSubmitter();
    await this.loadingService.hide();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        title: formCache.title,
        period: formCache.period
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST);
      if(dataCache !== undefined) {

        this.reportingPlanificationList = dataCache.data as ReportingPlanification[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  documentStatus(item: ReportingPlanification) {
    const statusList = APP_CONSTANTS.REPORTING_PLANIFICATION_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.REPORTING_PLANIFICATION_ACTIVE:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.REPORTING_PLANIFICATION_IN_ACTIVE:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  searchFormSubmitter(){
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST_FORM, JSON.stringify(formData));
    const payload = {
      title: formData.title,
      period: formData.period
    };
    this.currentPage = 1;
    this._searchTypeDeMaterialInfo(payload, this.currentPage, this.pageSize);
  }

  _searchTypeDeMaterialInfo(data: { title: string, period: string}, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<ReportingPlanification[]>) => {
        if(res.status === "success") {
          this.reportingPlanificationList = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST);
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.reportingPlanificationService.searchReportingPlanification(data, page, pageSize).subscribe( observer);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST);
    if(listCache !== undefined) {
      let listReportingPlanification: any[] = listCache.data;
      listReportingPlanification = (listReportingPlanification as ReportingPlanification[]).filter(r => r.plan_id != id);

      listCache.data = listReportingPlanification;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listReportingPlanification.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      title: formData.title,
      period: formData.period
    };
    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._searchTypeDeMaterialInfo(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveReportingPlanification(title: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: title })}</h3>`;
    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeReportingPlanification(title: string, plan_id: number) {
    if(!this.rolePermission['10_4']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }

    const removeConfirmation = await this.confirmRemoveReportingPlanification(title);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteReportingPlanificationById(plan_id);
    }
  }

  async _deleteReportingPlanificationById(plan_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(plan_id);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.reportingPlanificationService.deleteReportingPlanificationById(plan_id).subscribe(observer);
  }

}
