import { Component, Input, OnInit, Output, SimpleChanges } from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {
  ArticlesList,
  BrandInfoList,
  CustomHttpResponse,
  ReportColumn,
  ReportColumnType, ReportDownloadData,
  ReportingType, ReportListData, ReportRequest,
  StockMovementType
} from "../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../_services/dailog.service";
import {CommonService} from "../../_services/common.service";
import {TranslateService} from "@ngx-translate/core";
import {ContactService} from "../../_services/contact.service";
import {ReportingService} from "../../_services/reporting.service";
import {APP_CONSTANTS} from "../../_helpers/constants";
import * as moment from "moment";
import {CUSTOM_DATE_API_FORMAT} from "../../_helpers/custom-date-formats";
import {PageEvent} from "@angular/material/paginator";
import {NumeroSerieComponent} from "../inventory/numero-serie/numero-serie.component";
import {MatDialog} from "@angular/material/dialog";
import { ActivatedRoute } from '@angular/router';
import { EventEmitter } from '@angular/core';
import { ReportingPlanificationService } from 'src/app/_services/reporting-planification.service';

@Component({
  selector: 'app-reporting',
  templateUrl: './reporting.component.html',
  styleUrls: ['./reporting.component.scss']
})
export class ReportingComponent implements OnInit {

  @Input() ParentData:any;
  @Input() flowType: string | undefined ='reporting';

  reportingForm: FormGroup = this.formBuilder.group({
    BRAND_ID: [''],
    STORE_ID: [''],
    STORE_TYPE: [''],

    REPORT_TYPE: ['', Validators.required],
    STOCK_MOVEMENT_TYPE: [],

    START_DATE: [],
    END_DATE: [],

    REF_ARTICLES: [null],
    REPORT_COLUMNS: [null, Validators.required],
    BRAND_RESERVE_STATUS:[{ value: false, disabled: true }],
    EOL_ONLY: [{ value: false, disabled: true }]
  });

  reportingTypeList: ReportingType[] = [];
  stockMovementTypeList: StockMovementType[] = [];

  reportingColumnTypes: ReportColumnType[] | undefined;
  reportingColumnList: ReportColumn[] = [];
  totalReportColumnList: ReportColumn[] = [];

  brandList: BrandInfoList[] = [];
  storeList: any = [];

  articleList: ArticlesList[] = [];
  _filteredArticles: ArticlesList[] = [];
  _filterArticleKey: any[] = [];

  dateDisabled: boolean = true;

  reportURL: string | undefined;
  generatedReportFile: string | undefined;

  listReportData: ReportListData[] | undefined;
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  displayedColumns: { columns: string[], columnDetail: ReportColumn[] } | undefined;

  rolePermission: any;
  readOnly:any;
  constructor(private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private dialog: MatDialog,
              private dialogService: DialogService,
              private reportingSerivce: ReportingService,
              private commonService: CommonService,
              private translateService: TranslateService,
              private contactService: ContactService,
              private route: ActivatedRoute,
              private reportingPlanificationService:ReportingPlanificationService) { }

  ngOnInit(): void {
    this.reportingForm.valueChanges.subscribe(_newVal =>{
      this.reportingPlanificationService.reportingFormData = this.reportingForm;
    });

    this.loadDependencies();
  }

  async loadDependencies() {
    this.loadingService.show();

    const userPemissions = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(userPemissions != null) {
      this.rolePermission = JSON.parse(userPemissions);
    }
    this.disabledReportingFormFields();
    await this.getBrandList();
    await this.getReportingTypeList();
    await this.getReportingColumnList();
    await this.getArticleList();

    this.route.data.subscribe(data => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];
        if(this.readOnly) {
          this.changeToDetailMode();
        }
      }
    });

    if(this.flowType === 'reporting-planification' && this.ParentData){
      this.prefillFormDetail(this.ParentData);
    }

    this.loadingService.hide();
  }

  changeToDetailMode(){
    this.reportingForm.get('BRAND_ID')?.disable({ onlySelf: true });
    this.reportingForm.get('STORE_ID')?.disable({ onlySelf: true });
    this.reportingForm.get('STORE_TYPE')?.disable({ onlySelf: true });
    this.reportingForm.get('REPORT_TYPE')?.disable({ onlySelf: true });
    this.reportingForm.get('STOCK_MOVEMENT_TYPE')?.disable({ onlySelf: true });
    this.reportingForm.get('START_DATE')?.disable({ onlySelf: true });
    this.reportingForm.get('END_DATE')?.disable({ onlySelf: true });
    this.reportingForm.get('REF_ARTICLES')?.disable({ onlySelf: true });
    this.reportingForm.get('REPORT_COLUMNS')?.disable({ onlySelf: true });
  }

  prefillFormDetail(ParentData:any){

    this.reportingForm.patchValue({
      BRAND_ID: ParentData._r_brand_id != null? ParentData._r_brand_id : '',
      STORE_ID: ParentData._r_store_id != null ? ParentData._r_store_id : '',
      STORE_TYPE: ParentData._r_store_type,
      REPORT_TYPE: ParentData._r_report_type,
      STOCK_MOVEMENT_TYPE: JSON.parse(ParentData._r_movement_type),
      START_DATE: ParentData._r_start_date,
      END_DATE: ParentData._r_end_date,
      REF_ARTICLES: JSON.parse(ParentData._r_articles),
      REPORT_COLUMNS: JSON.parse(ParentData._r_columns)
    });

    this.onReportingTypeChange();
    this.onBrandChange()

  }


  get startDate() {
    return this.reportingForm.get('START_DATE');
  }

  get endDate() {
    return this.reportingForm.get('END_DATE');
  }

  get reportingType() {
    return this.reportingForm.get('REPORT_TYPE');
  }

  get isReportingTypeStockHistory() {
    return this.reportingType?.value === APP_CONSTANTS.REPORTING_TYPE_STOCK_MOVEMENT;
  }

  get isReportingTypeStockStatus() {
    return this.reportingType?.value === APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS;
  }

  get isReportingTypeStockValuation() {
    return this.reportingType?.value === APP_CONSTANTS.REPORTING_TYPE_STOCK_VALUATION;
  }

  get reservation_status(){
    return this.reportingForm.get('BRAND_RESERVE_STATUS');
  }

  get isEolSelected() {
    const selectedColumns = this.reportingForm.get('REPORT_COLUMNS')?.value;
    if(selectedColumns === undefined || selectedColumns === null || selectedColumns.length === 0) {
      return false;
    }

    if(selectedColumns.indexOf('eol_tag') > 0) {
      return true;
    }

    return false;
  }

  get eol_tag(){
    return this.reportingForm.get('EOL_ONLY');
  }

  get storeId(){
    return this.reportingForm.get('STORE_ID');
  }

  get brandId(){
    return this.reportingForm.get('BRAND_ID');
  }

  disabledReportingFormFields(){
    this.reportingForm.controls['STOCK_MOVEMENT_TYPE'].disable();
    this.reportingForm.controls['START_DATE'].disable();
    this.reportingForm.controls['END_DATE'].disable();
  }

  reportingFormSubmitter(event: SubmitEvent) {
    const displayType: string = event.submitter?.getAttribute('value') || APP_CONSTANTS.REPORTING_DISPLAY_TYPE_DOWNLOAD;

    if(this.rolePermission && !this.rolePermission['7_1'] && displayType === APP_CONSTANTS.REPORTING_DISPLAY_TYPE_LIST) {
      this.dialogService.displayAlert({
        title: 'KACCESS_DENIED',
        message: 'KREPORTING_NO_ACCESS_CONSULTATION'
      });

      return;
    }

    if(this.rolePermission && !this.rolePermission['7_2'] && displayType === APP_CONSTANTS.REPORTING_DISPLAY_TYPE_DOWNLOAD) {
      this.dialogService.displayAlert({
        title: 'KACCESS_DENIED',
        message: 'KREPORTING_NO_ACCESS_GENERATION'
      });

      return;
    }

    if(this.reportingForm.invalid) {
      return;
    }

    this.loadingService.show();

    const formData = this.reportingForm.getRawValue();

    const start_date = formData.START_DATE ? moment(formData.START_DATE).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";
    const end_date = formData.END_DATE ? moment(formData.END_DATE).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";

    const payload = {
      brand_id : formData.BRAND_ID ? formData.BRAND_ID : "",
      customer : {
        store_id : formData.STORE_ID ? formData.STORE_ID : "",
        store_type : formData.STORE_TYPE ? formData.STORE_TYPE : ""
      },
      report_type : formData.REPORT_TYPE,
      stock_history_type : formData.STOCK_MOVEMENT_TYPE ? formData.STOCK_MOVEMENT_TYPE : [],
      document_start_date : start_date,
      document_end_date : end_date,
      reference_article : formData.REF_ARTICLES ? formData.REF_ARTICLES: [],
      selected_column: formData.REPORT_COLUMNS ? formData.REPORT_COLUMNS : [],
      display_type: displayType,
      brand_reserve_status: formData.BRAND_RESERVE_STATUS,
      only_eol: formData.EOL_ONLY
    };

    this.currentPage = 1;

    this.generateReporting(displayType, payload);
  }

  pageChanged(event: PageEvent) {
    if(this.rolePermission && !this.rolePermission['7_1']) {
      this.dialogService.displayAlert({
        title: 'KACCESS_DENIED',
        message: 'KREPORTING_NO_ACCESS_CONSULTATION'
      });

      return;
    }

    this.loadingService.show();
    const formData = this.reportingForm.getRawValue();

    const displayType: string = APP_CONSTANTS.REPORTING_DISPLAY_TYPE_LIST;

    const start_date = formData.START_DATE ? moment(formData.START_DATE).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";
    const end_date = formData.END_DATE ? moment(formData.END_DATE).startOf('hour').format(CUSTOM_DATE_API_FORMAT) : "";

    const payload = {
      brand_id : formData.BRAND_ID ? formData.BRAND_ID : "",
      customer : {
        store_id : formData.STORE_ID ? formData.STORE_ID : "",
        store_type : formData.STORE_TYPE ? formData.STORE_TYPE : ""
      },
      report_type : formData.REPORT_TYPE,
      stock_history_type : formData.STOCK_MOVEMENT_TYPE ? formData.STOCK_MOVEMENT_TYPE : [],
      document_start_date : start_date,
      document_end_date : end_date,
      reference_article : formData.REF_ARTICLES ? formData.REF_ARTICLES: [],
      selected_column: formData.REPORT_COLUMNS ? formData.REPORT_COLUMNS : [],
      display_type: displayType
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this.generateReporting(displayType, payload, this.currentPage, this.pageSize);
  }

  _getColumnTitle(property: string): string {
    const columnData = this.displayedColumns?.columnDetail.find(row => row.property === property);
    if(columnData) {
      return columnData.label;
    }

    return '';
  }

  viewItemSerials(item: ReportListData) {
    const selectedListSerials = this.commonService.formatSerialNumbers((item.article_serials && item.article_serials !== '') ? item.article_serials.split(',') : []);
    const removedListSerials = this.commonService.formatSerialNumbers((item.removed_serials && item.removed_serials !== '') ? item.removed_serials.split(',') : []);

    const documentType = item.document_value;

    this.openUpSerialNumPopup({
      reportingDocumentType: documentType || 'default',
      oldListSerials: [],
      newListSerials: selectedListSerials,
      removeListSerials: removedListSerials,
      readonly: true
    });
  }

  openUpSerialNumPopup(data?: any) {
    data.numeroType = "reporting-numero-series";

    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: data || {}
      }
    );
  }

  generateReporting(displayType: string, payload: ReportRequest, page: number = 1, pageSize: number = 10): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ReportListData[] | ReportDownloadData>) => {
          this.loadingService.hide();
          if(res.status === 'success') {
            if(displayType === APP_CONSTANTS.REPORTING_DISPLAY_TYPE_DOWNLOAD) {
              const response = res.data as ReportDownloadData;
              this.generatedReportFile = response.file;
              this.reportURL = res.imageurl;

              window.open(`${this.reportURL}/${this.generatedReportFile}`, '_blank');
            } else {
              const response = res.data as ReportListData[];
              const columnLabels = this.reportingColumnList;
              this.displayedColumns = {
                columns: [
                  ...payload.selected_column,
                ].filter(r => r !== 'article_serials'),
                columnDetail: columnLabels
              };
              const viewActions = this.actionBasedOnCondition();
              if(!viewActions){
                this.displayedColumns.columns.push('ACTIONS')
              }

              // Removing the EOL tag from the columns list.
              // as we're going to display it in the Reference column itself.
              this.displayedColumns.columns = this.displayedColumns.columns.filter((columnName) => columnName !== 'eol_tag');

              this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
              this.totalRows = res.totalRows ? res.totalRows : this.totalRows;

              this.listReportData = response;
              if(this.reportingType?.value === APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS){
                this.listReportData = this.listReportData.map(report =>{
                  if(!report.available_stock && report.reception_count && report.livraison_count){
                    report.available_stock = report.reception_count - report.livraison_count;
                  }
                  return report;
                })
              }
            }

          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.reportingSerivce.generateReport(payload, page, pageSize).subscribe(observer);
    });
  }

  getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  getReportingTypeList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ReportingType[]>) => {
          if(res.status === 'success') {
            this.reportingTypeList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.reportingSerivce.getReportingTypesList().subscribe(observer);
    });
  }

  getReportingColumnList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ReportColumnType[]>) => {
          if(res.status === 'success') {
            this.reportingColumnTypes = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.reportingSerivce.getReportColumnList().subscribe(observer);
    });
  }

  getReportingStockMovementTypeList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<StockMovementType[]>) => {
          if(res.status === 'success') {
            this.stockMovementTypeList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.reportingSerivce.getStockMovementTypesList().subscribe(observer);
    });
  }

  getArticleList(data?: { brand_id: number, store_id: number, store_type: string }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticlesList[]>) => {
          if(res.status === 'success') {
            this.articleList = res.data;
            this._filteredArticles = this.articleList;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListArticles(data).subscribe(observer);
    });
  }

  _onArticleFilterChange(search: string) {
    this._filteredArticles = this.articleList.filter((row: ArticlesList) => row.article_label.toLowerCase().indexOf(search.toLowerCase()) > -1);
  }

  getReservationValue(){
    const typedereporting = this.reportingType?.value;
    if(typedereporting === APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS){
      this.reportingColumnList = this.filterDisplayColumn(this.totalReportColumnList);
      const defaultColumns = [...this.reportingColumnList].map((row) => row.property);
      this.reportingForm.get('REPORT_COLUMNS')?.setValue(defaultColumns);

      this.storeId?.setValue('');
      if(this.reservation_status?.value){
        this.storeId?.disable()
      }else{
        this.storeId?.enable()
      }
    }
  }

  async onReportingTypeChange() {
    const reportingType = this.reportingType?.value;
    if(reportingType !== APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS){
      this.reservation_status?.setValue(false);
      this.storeId?.enable();
      this.reservation_status?.disable();
    }else{
      if(this.reservation_status?.value){
        this.storeId?.setValue('');
        this.storeId?.disable();
      }else{
        this.storeId?.enable();
      }
      this.reservation_status?.enable();
    }

    this.eol_tag?.setValue(false);
    if (reportingType === APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS || reportingType === APP_CONSTANTS.REPORTING_TYPE_STOCK_VALUATION) {
      this.eol_tag?.enable({ onlySelf: true });
    } else {
      this.eol_tag?.disable({ onlySelf: true });
    }

    this.stockMovementTypeList = [];
    this.reportingColumnList = [];
    const columnTypeData = this.reportingColumnTypes?.find(row => row.column_type === reportingType);
    if(columnTypeData) {
      this.reportingColumnList = [...columnTypeData.column_list];
      this.totalReportColumnList = this.reportingColumnList;
      this.reportingColumnList = this.filterDisplayColumn(this.totalReportColumnList);
      this.reportingColumnList = this.reportingColumnList.sort((a, b) => {
        return a.order - b.order; // Sorting columns in the Ascending order.
      });

      const defaultColumns = [...this.reportingColumnList].map((row) => row.property);
      if(this.ParentData || this.flowType === 'repoting-planification'){
        this.reportingForm.get('REPORT_COLUMNS')?.setValue(JSON.parse(this.ParentData._r_columns));
      }else{
        this.reportingForm.get('REPORT_COLUMNS')?.setValue(defaultColumns);
      }
    }

    if(reportingType !== APP_CONSTANTS.REPORTING_TYPE_STOCK_VALUATION) {
      const selectedColumns = this.reportingForm.get('REPORT_COLUMNS')?.value;
      const updatedColumns = selectedColumns.filter((r: string) =>  r !== "eol_tag");
      this.reportingForm.get('REPORT_COLUMNS')?.setValue(updatedColumns);
    }

    this.reportingForm.get('STOCK_MOVEMENT_TYPE')?.disable({ onlySelf: true });

    if(reportingType && reportingType === APP_CONSTANTS.REPORTING_TYPE_STOCK_MOVEMENT) {
      this.loadingService.show();
      if(!this.readOnly){
      this.reportingForm.get('STOCK_MOVEMENT_TYPE')?.enable({ onlySelf: true });
      }
      this.dateDisabled = false;

      await this.getReportingStockMovementTypeList();

      this.loadingService.hide();
    }
  }

  async onBrandChange() {
    const brand_id = this.reportingForm.get('BRAND_ID')?.value;
    this.storeId?.setValue("");
    this.storeList = [];

    this.reportingColumnList = this.filterDisplayColumn(this.totalReportColumnList);
    const defaultColumns = [...this.reportingColumnList].map((row) => row.property);
    this.reportingForm.get('REPORT_COLUMNS')?.setValue(defaultColumns);

    if(brand_id) {
      this.loadingService.show();
      const brandDetail = this.brandList.find((row) => row.BRID == brand_id);

      this.reportingForm.get('STORE_TYPE')?.setValue(brandDetail?.BRTYPE);

      await this.getStoreListByBrand(brand_id);
      await this.getArticleList();

      this.loadingService.hide();
    }
  }

  async onStoreChange() {
    const brand_id = this.reportingForm.get('BRAND_ID')?.value;
    const store_id = this.reportingForm.get('STORE_ID')?.value;
    const store_type = this.reportingForm.get('STORE_TYPE')?.value;

    await this.loadingService.show();
    this.reportingColumnList = this.filterDisplayColumn(this.totalReportColumnList);
    const defaultColumns = [...this.reportingColumnList].map((row) => row.property);
    this.reportingForm.get('REPORT_COLUMNS')?.setValue(defaultColumns);

    let payload: any = {
      brand_id: brand_id || null,
      store_id: store_id || null,
      store_type: store_type || null
    };

    if((brand_id === undefined || brand_id == null || brand_id == '') ||
      (store_id === undefined || store_id == null || store_id == '')) {
      payload = undefined;
    }

    await this.getArticleList(payload);

    await this.loadingService.hide();
  }

  getStoreListByBrand(brand_id: number): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const payload = {
        brand_id
      };

      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.storeList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.contactService.getStoreListByBrand(payload).subscribe(observer);
    });
  }

  filterDisplayColumn(reportingColumnList: ReportColumn[]){
    if(this.brandId?.value === "" && this.reportingType?.value == APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS && !this.reservation_status?.value){
      reportingColumnList = reportingColumnList.filter((item) => !(item.property === "brand_name" || item.property === "store_name"));
    }else if((this.brandId?.value !== "" || this.brandId?.value === "") && this.reportingType?.value == APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS && this.reservation_status?.value){
      reportingColumnList = reportingColumnList.filter((item) => !(item.property === "store_name"));
    }else if(this.brandId?.value !== "" && this.reportingType?.value == APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS && !this.reservation_status?.value){
      reportingColumnList = reportingColumnList.filter((item) => !(item.property === "article_serials"));
    }
    return reportingColumnList;
  }

  actionBasedOnCondition(){
    return this.brandId?.value !== "" && this.reportingType?.value == APP_CONSTANTS.REPORTING_TYPE_STOCK_STATUS && !this.reservation_status?.value;
  }

}
