import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse, Settings } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { SettingsService } from 'src/app/_services/settings.service';

@Component({
  selector: 'app-settings',
  templateUrl: './settings.component.html',
  styleUrls: ['./settings.component.scss']
})
export class SettingsComponent implements OnInit {

  statusList = APP_CONSTANTS.SETTINGS_TLS_STATUS;
  upload_method_type = APP_CONSTANTS.UPLOAD_METHOD_TYPE;
  ftp_ssl = APP_CONSTANTS.FTP_SSL_VALUE;
  integerMask = APP_CONSTANTS.INTEGER_MASK;

  createSettingsForm: FormGroup = this.formBuilder.group({
    smtp_host: ['', [Validators.required]],
    smtp_port: ['', [Validators.required]],
    smtp_username: ['', [Validators.required]],
    smtp_password: [''],
    smtp_from_email: ['', [Validators.required, Validators.email]],
    smtp_from_name: ['', [Validators.required]],
    smtp_tls_status: ['', [Validators.required]],
    upload_type: ['', [Validators.required]],
    ftp_address: [''],
    ftp_port: [''],
    ftp_ssl: [''],
    ftp_user: [''],
    ftp_password: [''],
    ftp_folder_path: [''],
    shared_folder_path: [''],
    account_number: ['', [Validators.required]]
  });

  submitted: boolean = false;

  constructor(private formBuilder: FormBuilder,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private translateService: TranslateService,
              private settingsService: SettingsService) { }

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.viewSettingsData();

    await this.loadingService.hide();
  }

  get fr() {
    return this.createSettingsForm.controls;
  }

  get getupload_type() {
    return this.createSettingsForm.get('upload_type')?.value;
  }

  createSettingsFormSubmitter(){
    if(this.createSettingsForm.invalid){
      return;
    }
    const payload = this.createSettingsForm.getRawValue();
    console.log(this.createSettingsForm.value)
    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.submitted = false;
          this.fillSettingsForm(res.data);
          this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message:res.message
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }
    this.settingsService.modifySettingsData(payload).subscribe(observer);

  }

  onChangeUploadType(){
    const returnStatus = this.createSettingsForm.get('upload_type')?.value;
    if(returnStatus === 'ftp') {
      this.createSettingsForm.get('ftp_address')?.setValidators([Validators.required])
      this.createSettingsForm.get('ftp_user')?.setValidators([Validators.required])
      this.createSettingsForm.get('ftp_password')?.setValidators([Validators.required])
      this.createSettingsForm.get('ftp_folder_path')?.setValidators([Validators.required])
      this.createSettingsForm.get('ftp_address')?.updateValueAndValidity();
      this.createSettingsForm.get('ftp_user')?.updateValueAndValidity();
      this.createSettingsForm.get('ftp_password')?.updateValueAndValidity();
      this.createSettingsForm.get('ftp_folder_path')?.updateValueAndValidity();
      this.createSettingsForm.get('shared_folder_path')?.clearValidators();
      this.createSettingsForm.get('shared_folder_path')?.updateValueAndValidity();
    }else{
      this.createSettingsForm.get('ftp_address')?.clearValidators();
      this.createSettingsForm.get('ftp_user')?.clearValidators();
      this.createSettingsForm.get('ftp_password')?.clearValidators();
      this.createSettingsForm.get('ftp_folder_path')?.clearValidators();
      this.createSettingsForm.get('ftp_address')?.updateValueAndValidity();
      this.createSettingsForm.get('ftp_user')?.updateValueAndValidity();
      this.createSettingsForm.get('ftp_password')?.updateValueAndValidity();
      this.createSettingsForm.get('ftp_folder_path')?.updateValueAndValidity();
      this.createSettingsForm.get('shared_folder_path')?.setValidators([Validators.required])
      this.createSettingsForm.get('shared_folder_path')?.updateValueAndValidity();
    }
  }

  viewSettingsData(){
    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.fillSettingsForm(res.data);
          this.onChangeUploadType();
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }
    this.settingsService.viewSettingsData().subscribe(observer);
  }

  fillSettingsForm(data:Settings){
    this.createSettingsForm.setValue({
      smtp_host: data.smtp_host,
      smtp_port: data.smtp_port.toString(),
      smtp_username: data.smtp_username,
      smtp_password: "",
      smtp_from_email: data.smtp_from_email,
      smtp_from_name: data.smtp_from_name,
      smtp_tls_status: data.smtp_tls_status,
      upload_type: data.upload_type,
      ftp_address: data.ftp_address,
      ftp_port: data.ftp_port.toString(),
      ftp_ssl: data.ftp_ssl,
      ftp_user: data.ftp_user,
      ftp_password: data.ftp_password,
      ftp_folder_path: data.ftp_folder_path,
      shared_folder_path: data.shared_folder_path,
      account_number: data.account_number
    });
  }

}
