import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { TranslateService } from '@ngx-translate/core';
import { Router, ActivatedRoute } from '@angular/router';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {BrandInfoList, CustomHttpResponse, StockLocation} from "../../../_models";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {CommonService} from "../../../_services/common.service";
import {StockLocationService} from "../../../_services/stock-location.service";
import { DataSharingService } from 'src/app/_services/data-sharing.service';


@Component({
  selector: 'app-edit-storage-location',
  templateUrl: './edit-storage-location.component.html',
  styleUrls: ['./edit-storage-location.component.scss']
})
export class EditStorageLocationComponent implements OnInit {

  editStorageForm: FormGroup = this.formBuilder.group({
    STLLBL: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STOCK_LOCATION_LABEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STOCK_LOCATION_LABEL_MAX_LENGTH)]],
    STLBRID: [''],
    STLSTAT: ['', Validators.required],
  });

  submitted: boolean = false;
  brandList: BrandInfoList[] = [];

  stockLocationId: number | undefined;
  stockLocationData: StockLocation | undefined;

  statusList = APP_CONSTANTS.STOCK_LOCATION_STATUS;
  readOnly: boolean = false;

  constructor(private formBuilder: FormBuilder,
              public stockLocationService: StockLocationService,
              private commonService: CommonService,
              private translateService: TranslateService,
              private router: Router,
              private route: ActivatedRoute,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private dataSharingService:DataSharingService) {
   }

  ngOnInit(): void {
    this.route.data.subscribe(data => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];
        if(this.readOnly) {
          this.changeToDetailMode();
        }
      }
    });

    this.route.params.subscribe(params => {
      if(params !== undefined && params !== null && params.hasOwnProperty('location_id')) {
        this.stockLocationId = params["location_id"];

        this.loadDependencies();
      }
    });
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this._getBrandList();
    await this._getStockLocationDetail(this.stockLocationId);
    this.prefillFormDetail(this.stockLocationData);

    await this.loadingService.hide();
  }

  get fr() {
    return this.editStorageForm.controls;
  }

  changeToDetailMode() {
    this.editStorageForm.get('STLLBL')?.disable({ onlySelf: true });
    this.editStorageForm.get('STLBRID')?.disable({ onlySelf: true });
    this.editStorageForm.get('STLSTAT')?.disable({ onlySelf: true });
  }

  _locationIsActive(status: number | undefined): boolean {
    if(status === APP_CONSTANTS.STOCK_LOCATION_STATUS_ACTIVE) {
      return true;
    }

    return false;
  }

  _getStockLocationDetail(location_id: number | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(location_id === undefined) {
        resolve('INVALID_LOCATION');
        return;
      }

      const observer = {
        next: (res: CustomHttpResponse<StockLocation>) => {
          this.loadingService.hide();
          if(res.status === 'success') {
            this.stockLocationData = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(() => {
              this.router.navigate(['/options/storage-location/list']);
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.stockLocationService.getStockLocationDetail(location_id).subscribe(observer);
    });
  }

  prefillFormDetail(stockLocationData: StockLocation | undefined) {
    if(stockLocationData === undefined || stockLocationData === null) {
      return;
    }

    this.editStorageForm.patchValue({
      STLLBL: stockLocationData.stock_location_lbl,
      STLBRID: stockLocationData.brand_id || '',
      STLSTAT: stockLocationData.status,
    });
  }

  _getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  modifyFormSubmitter() {
    if(this.readOnly) {
      return;
    }

    this.submitted = true;
    if (this.editStorageForm.invalid) {
      return;
    }

    if(this.stockLocationId === undefined || this.stockLocationId === null) {
      return;
    }

    const formData = this.editStorageForm.getRawValue();
    const payload = {
      brand_id: formData.STLBRID,
      stock_location_lbl: formData.STLLBL,
      stock_location_status: formData.STLSTAT
    }

    this._modifyStockLocation(this.stockLocationId, payload);
  }

  async _modifyStockLocation(location_id: number, payload: { brand_id: number, stock_location_lbl: string, stock_location_status: number }) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        this.__updateListCache(res.data);
        if(res.status === 'success') {
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.editStorageForm.reset();
          this.submitted = false;
          this.router.navigate(['/options/storage-location/list']);
        } else {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }

    this.stockLocationService.modifyStockLocation(location_id, payload).subscribe(observer);
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST);
    if(listCache !== undefined) {
      let listStockLocation = listCache.data as StockLocation[];
      listStockLocation = listStockLocation.map((row) => {
        if(row.stock_location_id == payload.stock_location_id) {0
          row.brand_id = payload.brand_id
          row.brand_lbl = payload.brand_lbl
          row.created_at = payload.created_at
          row.created_userid = payload.created_userid
          row.created_username = payload.created_username
          row.modified_at = payload.modified_at
          row.modified_userid = payload.modified_userid
          row.modified_username = payload.modified_username
          row.status = payload.status
          row.stock_location_id = payload.stock_location_id
          row.stock_location_lbl = payload.stock_location_lbl
        }
        return row;
      });

      listCache.data = listStockLocation;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST, JSON.stringify(listCache));
    }
  }


}
