import { Component, OnInit } from '@angular/core';
import {PageEvent} from '@angular/material/paginator';
import { FormBuilder, FormGroup } from '@angular/forms';
import {StockLocationService} from '../../../_services/stock-location.service';
import {NgxSpinnerService} from "ngx-spinner";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import { TranslateService } from '@ngx-translate/core';
import {CustomHttpResponse, StockLocation} from "../../../_models";
import {lastValueFrom} from "rxjs";
import {DialogService} from "../../../_services/dailog.service";
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-storage-location-list',
  templateUrl: './storage-location-list.component.html',
  styleUrls: ['./storage-location-list.component.scss']
})
export class StorageLocationListComponent implements OnInit {

  searchForm: FormGroup = this.formBuilder.group({
    label: [''],
  });

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [5, 10, 25, 100];

  stockLocationList: StockLocation[] | undefined;
  displayedColumnsStorage: string[] = ['LOCATION_LABEL', 'LOCATION_BRAND', 'LOCATION_STATUS', 'ACTIONS'];
  rolePermission:any;
  constructor(private formBuilder: FormBuilder,
              public stockLocationService: StockLocationService,
              private loadingService: NgxSpinnerService,
              private translateService: TranslateService,
              private dialogService: DialogService,
              private dataSharingService: DataSharingService) {
   }

  ngOnInit(): void {
    this.checkForListCache();
    this.loadDependencies();
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_STOCK_LOCATIONT_LIST_FORM);
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        label: formCache.label
      });
      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST);
      if(dataCache !== undefined) {
        this.stockLocationList = dataCache.data  as StockLocation[];
        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.loadingService.hide();
  }

  documentStatus(item: StockLocation) {
    const statusList = APP_CONSTANTS.STOCK_LOCATION_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.STOCK_LOCATION_STATUS_ACTIVE:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.STOCK_LOCATION_STATUS_IN_ACTIVE:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();

    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATIONT_LIST_FORM, JSON.stringify(formData))

    const payload = {
      stock_location_lbl: formData.label
    }

    this.currentPage = 1;

    this._searchStockLocation(payload);
  }

  _searchStockLocation(payload: { stock_location_lbl: string }, page: number = 1, pageSize = 10) {
    this.loadingService.show();

    const observer = {
      next: (res: CustomHttpResponse<StockLocation[]>) => {
        if(res.status === "success") {
          this.stockLocationList = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST, JSON.stringify(res));

        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST);
        this.loadingService.hide();
      }
    };

    this.stockLocationService.searchStockLocationList(payload, page, pageSize).subscribe(observer);
  }

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      stock_location_lbl: formData.label
    }

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._searchStockLocation(payload, this.currentPage, this.pageSize);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST);
    if(listCache !== undefined) {
      let listStackLocation: any[] = listCache.data;

      listStackLocation = (listStackLocation as StockLocation[]).filter(r => r.stock_location_id != id);

      listCache.data = listStackLocation;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listStackLocation.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  confirmRemoveStockLocation(location_label: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: location_label })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeStockLocation(location_id: number, location_label: string) {

    if(!this.rolePermission['9_91114']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveStockLocation(location_label);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteStockLocation(location_id);
    }

  }

  async _deleteStockLocation(location_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(location_id);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.stockLocationService.deleteStockLocation(location_id).subscribe(observer);
  }

}
