import {Component, ElementRef, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {DatePipe} from '@angular/common';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import {ActivatedRoute, Params, Router} from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {MatDialog} from "@angular/material/dialog";
import { trigger, state, style, transition, animate } from '@angular/animations';
import {CommonService} from "../../../_services/common.service";
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {NgxSpinnerService} from "ngx-spinner";
import {TransferDeStockService} from "../../../_services/transfer-de-stock.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {DialogService} from "../../../_services/dailog.service";
import {MatSelect} from "@angular/material/select";
import {
  BonReceptionItem,
  BonTransferDeStock,
  BonTransferDeStockItem,
  BrandInfoList,
  CustomHttpResponse,
  LigneInfoList,
  StockLocationList
} from "../../../_models";
import {Article, SearchArticle} from "../../../_models/article";
import * as moment from "moment";
import {
  CUSTOM_DATE_API_FORMAT,
  CUSTOM_DATE_FORMAT,
  CUSTOM_DATE_TIME_API_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import {firstValueFrom, lastValueFrom, Observable, Subscription} from "rxjs";
import {ModifyItemComponent} from "../../bon-de-reception/modify-item/modify-item.component";
import {NumeroSerieComponent} from "../../inventory/numero-serie/numero-serie.component";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import {DataSharingService} from "../../../_services/data-sharing.service";
import { HistoriqueComponent } from 'src/app/_components/historique/historique.component';
import {SearchArticleComponent} from "../../article/search-article/search-article.component";

@Component({
  selector: 'app-create-transfer-de-stock',
  templateUrl: './create-transfer-de-stock.component.html',
  styleUrls: ['./create-transfer-de-stock.component.scss'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({ height: '0px', minHeight: '0', visibility: 'hidden' })),
      state('expanded', style({ height: '*', visibility: 'visible' })),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class CreateTransferDeStockComponent implements OnInit, OnDestroy {

  @ViewChild('prixUnitInput') unitPrixInputEl: ElementRef | undefined;
  @ViewChild('quantiteInput') quantiteInputEl: ElementRef | undefined;
  @ViewChild('stockLocationInput') stockLocationInputEl: MatSelect | undefined;
  @ViewChild('rechercheRefInput') rechercheRefInputEl: ElementRef | undefined;

  bonDocumentForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],
    DOCUMENT_ASSOC: [''],

    STOCK_LOCATION_SOURCE: ['', Validators.required], // Stock Location Source
    STOCK_LOCATION_DESTINATION: [''], // Stock Location Destination

    LIGNE_INFO: [''], // Ligne ID
    LIGNE_CUSTOM_LABEL: [''], // Ligne Label

    SEARCH_REF: [""],
    ITEM_BRAND:[""]
  })

  bonTransferDeStockItemForm: FormGroup = this.formBuilder.group({
    ARTICLE_ID: ["", Validators.required],
    REFERENCE: [{ value: "", disabled: true }, Validators.required],
    LABEL: [{ value: "", disabled: true }, Validators.required],
    QTE: [{ value: "0", disabled: true }, [Validators.required, Validators.min(1)]],
    PRIX_UNIT: ['', [Validators.required, ValidateGreaterThan]],
    PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
    AVAIL_STOCK: [{ value: "0", disabled: true }],

    SERIALS_LIST: ['', Validators.required],
    SELECTED_SERIALS_LIST:[''],
  });

  newArticleSelection: Article | undefined;

  bonDocumentNum: string | undefined = "*********";
  bonTransferDeStock: BonTransferDeStock | undefined;
  bonTransferDeStockItems: BonTransferDeStockItem[] = [];
  BonTransferDeStockArticles: Article[] = [];

  totalArticles: number = 0;
  totalQte: number = 0;
  totalPT: number = 0;

  brandList: BrandInfoList[] = [];
  ligneList: LigneInfoList[] = [];
  stockLocationList: StockLocationList[] = [];

  fullKeywordMatch: boolean = false;
  isKeywordSerial: boolean = false;
  serialStockLocation: string = '';
  searchRefArticlesList: Article[] | undefined = undefined;

  displayedColumns: string[] = [ 'REFERENCE', 'LABEL', 'QTE', 'PRIX_UNIT', 'PRIX_TOTAL', 'ACTIONS'];

  customLigneInput: boolean = false;

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  subscriptions: Subscription[] | undefined = [];

  rolePermission:any;
  constructor(public dialog: MatDialog,
              private commonService: CommonService,
              private transferDeStockService: TransferDeStockService,
              private datePipe: DatePipe,
              private router: Router,
              private route: ActivatedRoute,
              private formBuilder: FormBuilder,
              private dataSharingService: DataSharingService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService) {
  }

  ngOnInit(): void {

    this.loadDependencies();

    const ligneInfoSubscription = this.bonDocumentForm.get("LIGNE_INFO")?.valueChanges.subscribe(
      (res) => {
        if(res !== undefined && res !== null) {
          if(res === 'custom-ligne') {
            this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue("");
            this.customLigneInput = true;
          } else {
            this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(res.LGILBL);
            this.customLigneInput = false;
          }
        }
      }
    )

    if(ligneInfoSubscription) {
      this.subscriptions?.push(ligneInfoSubscription);
    }
  }

  ngOnDestroy() {
    if(this.subscriptions && this.subscriptions.length > 0) {
      this.subscriptions.forEach((_s) => {
        if(_s) {
          _s.unsubscribe();
        }
      });
    }
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.bonDocumentForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)

    });
    this._getRolePermission();
    await this._getLigneList();
    await this._getBrandList();
    await this._getStockLocationList();

    firstValueFrom(this.route.queryParams).then(
      (params) => {
        console.log(params);
        if(params.hasOwnProperty('document_no') && params["document_no"] !== null && params["document_no"] !== "") {
          this.bonDocumentNum = params["document_no"];

          this._getBonTransferDeStockDetail(this.bonDocumentNum);
        }
      }
    )

    const selectionSubscription = this.dataSharingService.selectionItems.subscribe((res) => {
      if(res !== undefined && res !== null) {
        const { type, items, extras } = res;
        const { document_type, document_no, documentDetail } = extras;

        if(type === APP_CONSTANTS.SELECTION_BR_TO_TR) {
          let receptionItems: any[] = [...items];

          // Extracting the Stock Source location from first item which is copied.
          // Items will contain same stock location otherwise only one item will be received.
          const stockSourceLocation = items[0].stock_source;
          this.bonDocumentForm.get('STOCK_LOCATION_SOURCE')?.setValue(stockSourceLocation);

          receptionItems = receptionItems.map((row: BonReceptionItem) => {
            return {
              article_id: +row.article_id,
              article_reference: row.article_reference,
              article_desc: row.article_desc,
              qte: +row.qte,
              qte_stock: +row.qte_stock,
              unit_price: +row.unit_price,
              total_price: +row.total_price,
              article_serials: row.article_serials
            }
          });

          this._addCopiedBonTransferDeStockItems(receptionItems as BonTransferDeStockItem[], document_no, document_type);
        }
      }
    })

    this.subscriptions?.push(selectionSubscription);

    await this.loadingService.hide();
  }

  _getRolePermission(){
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
  }

  async onCloseDocument() {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    this.returnBack();
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST_FORM);
  }

  async onSubmitDocument(type: 'validate' | 'ok' = 'ok') {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    if(type === 'validate'){
      if(!this.rolePermission['3_25']) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KACCESS_DENIED'
        })
        return;
      }
    }

    if(!this.isBonTransferStockModifiable) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ARTICLE_ADDED'
      })

      return;
    }

    const documentFormData = this.bonDocumentForm.getRawValue();

    const bonArticles: BonTransferDeStockItem[] = [...this.bonTransferDeStockItems];
    const bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    const bonStatus = type === 'validate' ? APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_YES : APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_NO;

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonTransferDeStock = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),

      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),

      stock_source_location_id: documentFormData.STOCK_LOCATION_SOURCE,
      stock_dest_location_id: (documentFormData.STOCK_LOCATION_DESTINATION ? documentFormData.STOCK_LOCATION_DESTINATION : null),

      docs_assoc: documentFormData.DOCUMENT_ASSOC,
      articles: bonArticles,
      brand_id: documentFormData.ITEM_BRAND,
      total_price: bonTotalPrix,
      status: bonStatus,
    };
    this.__resetListCache();
    this._modifyTransferDeStock(this.bonDocumentNum, bonDocument).then(
      () => {
        this.returnBack(type === 'validate' ? 'consultation' : 'list');
      }
    )
  }

  generateTransferDeStockPdf(print?: boolean) {
    if(!this.isBonTransferStockModifiable || (this.bonDocumentNum === undefined)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KTRANSFER_STOCK_NO_CONTENT'
      })
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.subscriptions?.push(this.transferDeStockService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      } else {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      }
    }));
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_TRANSFER_DU_STOCK_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  returnBack(type: 'list' | 'consultation' = 'list') {
    if(type === 'list') {
      this.router.navigate(['/transfer-de-stock/list']);
    } else if (type === 'consultation') {
      this.router.navigate(['/transfer-de-stock/detail', this.bonDocumentNum]);
    }
  }

  _getLigneList(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.subscriptions?.push(this.commonService.getAllLigneList().subscribe({
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    });
  }

  _getBrandList(): Promise<any> { // get both Client and Supplier Brands
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  _getStockLocationList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.subscriptions?.push(this.commonService.getAllStockList().subscribe({
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.stockLocationList = res.data;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    })
  }

  _searchArticles(data: { keyword: string, location_id: string }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      let searchObs: Observable<CustomHttpResponse<SearchArticle>> | undefined;

      searchObs = this.commonService.searchArticlesListByRef(data);

      if(searchObs === undefined) {
        return;
      }

      this.subscriptions?.push(searchObs.subscribe({
        next: (res: CustomHttpResponse<SearchArticle>) => {
          if(res.status === 'success') {
            this.fullKeywordMatch = res.data.fullMatch || false;
            this.isKeywordSerial = res.data.isSerial || false;
            this.serialStockLocation = res.data.serial_stock_location_id || '';
            this.searchRefArticlesList = res.data.rows;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    })
  }

  _getBonTransferDeStockDetail(document_no: string | undefined): Promise<any> {
    console.log(document_no)
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined || document_no === null || document_no === '') {
        reject();
        return;
      }

      this.bonDocumentForm.get("STOCK_LOCATION_SOURCE")?.disable({ onlySelf: true });

      this.subscriptions?.push(this.transferDeStockService.getTransferDeStockDetail(document_no).subscribe( {
        next: (res: CustomHttpResponse<BonTransferDeStock>) => {
          if(res.status === "success") {
            const bonTransferDeStockData = res.data;
            this.bonTransferDeStock = bonTransferDeStockData;

            this.__refreshBonDetails(bonTransferDeStockData);
          } else {
            // If the bon reception not available
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/transfer-de-stock/create']);
              }
            );
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    })
  }

  __refreshBonDetails(bonTransferDeStock: BonTransferDeStock) {
    if(bonTransferDeStock !== undefined && bonTransferDeStock !== null) {
      this.bonDocumentNum = bonTransferDeStock.document_no;
      this.bonTransferDeStockItems = bonTransferDeStock.articles;
      this.BonTransferDeStockArticles = bonTransferDeStock?.custom_articles ? bonTransferDeStock?.custom_articles : [];

      this.totalArticles = 0;
      this.totalQte = 0;
      this.totalPT = 0;

      this.bonTransferDeStockItems.forEach((row) => {
        this.totalArticles += 1;
        this.totalQte += +row.qte;
        this.totalPT += +row.total_price;
      })

      this.bonDocumentForm.get("NUMBON")?.setValue(bonTransferDeStock.document_no);
      this.bonDocumentForm.get("DATE_DU_BON")?.setValue(moment(bonTransferDeStock.dated).format(CUSTOM_DATE_FORMAT));

      this.bonDocumentForm.get("STOCK_LOCATION_SOURCE")?.setValue(bonTransferDeStock.stock_source_location_id);
      this.bonDocumentForm.get("STOCK_LOCATION_DESTINATION")?.setValue(bonTransferDeStock.stock_dest_location_id);

      this.bonDocumentForm.get("DOCUMENT_ASSOC")?.setValue(bonTransferDeStock.docs_assoc);

      if(bonTransferDeStock.ligne_lbl !== undefined && bonTransferDeStock.ligne_lbl !== null && bonTransferDeStock.ligne_lbl !== "") {
        if(bonTransferDeStock.ligne_id === undefined || bonTransferDeStock.ligne_id === null || bonTransferDeStock.ligne_id === "") {
          // Custom Ligne Input
          this.customLigneInput = true;
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue('custom-ligne');
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonTransferDeStock.ligne_lbl);
        } else {
          // Chosen Ligne Input from the dropdown
          const currentLigneItem = this.ligneList.find(r => r.LGIID == bonTransferDeStock.ligne_id)
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue(currentLigneItem);
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonTransferDeStock.ligne_lbl);
        }
      }
    }
  }

  __serialsListDisplay(serialsArr: string[]) {
    return serialsArr.join(", ");
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  get documentAssocie() {
    return this.bonDocumentForm.get("DOCUMENT_ASSOC");
  }

  get isBonTransferStockModifiable() {
    return this.bonTransferDeStock !== undefined;
  }

  get __isDocumentValidated() {
    return this.bonTransferDeStock && this.bonTransferDeStock.status === APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_YES;
  }

  async addBonTransferDeStockItemFormSubmitter() {
    console.log(this.bonTransferDeStockItemForm);
    if(this.bonTransferDeStockItemForm.invalid) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }

    this.loadingService.show();
    const documentFormData = this.bonDocumentForm.getRawValue();
    const itemFormData = this.bonTransferDeStockItemForm.getRawValue();

    let bonArticles: BonTransferDeStockItem[] = [];
    let duplicationArticleFound: BonTransferDeStockItem | undefined = undefined;

    if(this.isBonTransferStockModifiable) {
      bonArticles = [...this.bonTransferDeStockItems];

      duplicationArticleFound = this.bonTransferDeStockItems.find((item) => {
        if (+item.article_id === +itemFormData.ARTICLE_ID) {
          return true;
        }

        return false;
      });
    }

    const itemSerials = itemFormData.SELECTED_SERIALS_LIST.split(',');
    if(this.isBonTransferStockModifiable && duplicationArticleFound !== undefined && duplicationArticleFound !== null) {
      bonArticles = bonArticles.map((rowItem) => {
        if(rowItem.item_id === duplicationArticleFound?.item_id) {
          const quantity = rowItem.qte + (+itemFormData.QTE);
          const unitPrix = +itemFormData.PRIX_UNIT;
          const totalPrix = (quantity * unitPrix).toFixed(2);

          rowItem.article_reference = itemFormData.REFERENCE;
          rowItem.article_desc = itemFormData.LABEL;
          rowItem.qte = quantity;
          rowItem.qte_stock = +itemFormData.AVAIL_STOCK;
          rowItem.unit_price = +unitPrix;
          rowItem.total_price = +totalPrix;
          rowItem.article_serials = [...rowItem.article_serials, ...itemSerials];
        }

        return rowItem;
      })
    } else {
      bonArticles.push({
        article_id: +itemFormData.ARTICLE_ID,
        article_reference: itemFormData.REFERENCE,
        article_desc: itemFormData.LABEL,
        qte: +itemFormData.QTE,
        qte_stock: +itemFormData.AVAIL_STOCK,
        unit_price: +itemFormData.PRIX_UNIT,
        total_price: +itemFormData.PRIX_TOTAL,
        article_serials: itemSerials
      })
    }

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    let bonStatus = APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_NO;

    if(this.isBonTransferStockModifiable) {
      bonStatus = this.bonTransferDeStock?.status ? this.bonTransferDeStock?.status : APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_NO;
    }

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonTransferDeStock = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),

      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),

      stock_source_location_id: documentFormData.STOCK_LOCATION_SOURCE,
      stock_dest_location_id: (documentFormData.STOCK_LOCATION_DESTINATION ? documentFormData.STOCK_LOCATION_DESTINATION : null),

      docs_assoc: documentFormData.DOCUMENT_ASSOC,
      articles: bonArticles,

      total_price: bonTotalPrix,
      status: bonStatus,
      brand_id: documentFormData.ITEM_BRAND
    };

    console.log('payload: ', bonDocument);
    if(this.isBonTransferStockModifiable) {
      this._modifyTransferDeStock(this.bonDocumentNum, bonDocument).then(
        () => {
          this.__resetNewLineItemForm();
        }
      )

      return;
    }

    try {
      await this._addNewLineItem(bonDocument);

      this.bonDocumentForm.get("STOCK_LOCATION_SOURCE")?.disable({ onlySelf: true });
      this.__resetNewLineItemForm();
      // After adding a new Line item.
      this.__updateCurrentRouteDocumentNum(this.bonDocumentNum);
    } catch (e: any) {
      // Catch Block
    }
  }

  async _addCopiedBonTransferDeStockItems(items: BonTransferDeStockItem[], link_document_no: string, link_document_type: string) {
    if(items === undefined || items === null || items.length === 0) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_SELECTION_ITEM_AVAILABLE'
      });

      return;
    }

    this.loadingService.show();
    const documentFormData = this.bonDocumentForm.getRawValue();

    let bonArticles: BonTransferDeStockItem[] = [...items];

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    let bonStatus = APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_NO;

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonTransferDeStock = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),

      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),

      stock_source_location_id: documentFormData.STOCK_LOCATION_SOURCE,
      stock_dest_location_id: null,

      docs_assoc: documentFormData.DOCUMENT_ASSOC,
      articles: bonArticles,

      total_price: bonTotalPrix,
      status: bonStatus,

      linked_document_no: link_document_no,
      linked_document_type: link_document_type
    };

    console.log('payload: ', bonDocument);
    try {
      await this._addNewLineItem(bonDocument);

      this.bonDocumentForm.get("STOCK_LOCATION_SOURCE")?.disable({ onlySelf: true });
      this.__resetNewLineItemForm();
      // After adding a new Line item.
      this.__updateCurrentRouteDocumentNum(this.bonDocumentNum);
    } catch (e: any) {
      // Catch Block
    }
  }

  removeLineItem(currentItem: BonTransferDeStockItem) {
    this.loadingService.show();

    let bonArticles: BonTransferDeStockItem[] = [...this.bonTransferDeStockItems];
    bonArticles = bonArticles.filter((row) => row.item_id !== currentItem.item_id);

    const documentFormData = this.bonDocumentForm.getRawValue();

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    const bonStatus = this.bonTransferDeStock?.status ? this.bonTransferDeStock?.status : APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_NO;

    const ligneInfo = documentFormData.LIGNE_INFO;

    let bonDocument: BonTransferDeStock = {
      ...this.bonTransferDeStock,

      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),

      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),

      stock_source_location_id: documentFormData.STOCK_LOCATION_SOURCE,
      stock_dest_location_id: (documentFormData.STOCK_LOCATION_DESTINATION ? documentFormData.STOCK_LOCATION_DESTINATION : null),

      docs_assoc: documentFormData.DOCUMENT_ASSOC,

      articles: bonArticles,
      total_price: bonTotalPrix,
      status: bonStatus
    };

    delete bonDocument.tds_id;

    this._modifyTransferDeStock(this.bonDocumentNum, bonDocument);
  }

  _addNewLineItem(bonDocument: BonTransferDeStock): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.subscriptions?.push(this.transferDeStockService.createTransferDeStock(bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonTransferDeStock>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonTransferDeStockData = res.data;
            this.bonTransferDeStock = bonTransferDeStockData;

            // Emptying the Selection items once the items are added successfully.
            this.dataSharingService.selectionItems.next(undefined);

            this.__refreshBonDetails(bonTransferDeStockData);
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });

            // Emptying the Selection items if we got any errors
            this.dataSharingService.selectionItems.next(undefined);

            reject(res);
            return;
          }
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });

          // Emptying the Selection items if we got any errors
          this.dataSharingService.selectionItems.next(undefined);

          reject(err);
        }
      }));
    })
  }

  _modifyTransferDeStock(document_no: string | undefined, bonDocument: BonTransferDeStock): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined) {
        reject("INVALID_DOCUMENT_NO");
        return;
      }

      this.subscriptions?.push(this.transferDeStockService.modifyTransferDeStock(document_no, bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonTransferDeStock>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonTransferDeStockData = res.data;
            this.bonTransferDeStock = bonTransferDeStockData;

            this.__refreshBonDetails(bonTransferDeStockData);
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });

            reject(res);
            return;
          }
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }));
    })
  }

  __updateCurrentRouteDocumentNum(document_no: string | undefined) {
    if(document_no === undefined) {
      return;
    }

    const queryParams: Params = { document_no };

    this.router.navigate(
      [],
      {
        relativeTo: this.route,
        queryParams: queryParams,
        queryParamsHandling: 'merge', // remove to replace all query params by provided
      });
  }

  __resetNewLineItemForm() {
    this.newArticleSelection = undefined;
    this.bonTransferDeStockItemForm.patchValue({
      ARTICLE_ID: "",
      REFERENCE: "",
      LABEL: "",
      QTE: "0",
      PRIX_UNIT: "",
      PRIX_TOTAL: "0",
      AVAIL_STOCK: "",

      SERIALS_LIST: "",
      SELECTED_SERIALS_LIST:"",
    })
    this.bonTransferDeStockItemForm.updateValueAndValidity();

    this.bonDocumentForm.patchValue({
      SEARCH_REF: "",
    });

    this.bonDocumentForm.updateValueAndValidity();
    this.rechercheRefInputEl?.nativeElement.focus();
  }

  onStockLocationChange(event: any) {
    this.stockLocationInputEl?.close();

    if(this.newArticleSelection) {
      // Changing the Stock location when already an article selected
      const referenceValue = this.bonDocumentForm.get("SEARCH_REF")?.value;

      if(referenceValue !== '') {
        this.searchArticles(undefined).then(
          (res) => {
            const currentArticleMatch = this.searchRefArticlesList?.find((r) => r.id === this.newArticleSelection?.id);
            if(currentArticleMatch) {
              this.newArticleSelection = currentArticleMatch;

              this.__fillNewLineItemForm('existing');
            }
          }
        );
      }
    } else {
      // Changing the Stock location at first
      this.rechercheRefInputEl?.nativeElement.focus();

      this.searchRefArticlesList = undefined;

      // Changing the Stock location when already an article selected
      const referenceValue = this.bonDocumentForm.get("SEARCH_REF")?.value;

      if(referenceValue !== '') {
        this.searchArticles(undefined).then(
          (res) => {
            const currentArticleMatch = this.searchRefArticlesList?.find((r) => r.id === this.newArticleSelection?.id);
            if(currentArticleMatch) {
              this.newArticleSelection = currentArticleMatch;

              this.__fillNewLineItemForm('existing');
            }
          }
        );
      }
    }
  }

  async searchArticles(event: any | undefined): Promise<any> {
    return new Promise(async (resolve, reject) => {
      if(event !== undefined && (event.keyCode == 38 || event.keyCode == 40 || event.keyCode == 13)) {
        return;
      }

      const searchStockLocation = this.bonDocumentForm.get("STOCK_LOCATION_SOURCE")?.value;

      if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
        this.searchRefArticlesList = undefined;

        this.stockLocationInputEl?.focus();
        this.stockLocationInputEl?.open();
        return;
      }

      let payload = {
        location_id: '',
        keyword: ''
      };

      if(searchStockLocation !== undefined && searchStockLocation !== null && searchStockLocation !== '') {
        payload.location_id = searchStockLocation;
      }

      const searchRefKeyword = this.bonDocumentForm.get("SEARCH_REF")?.value;
      payload = {
        ...payload,
        keyword: searchRefKeyword
      };

      const searchResult = await this._searchArticles(payload);

      resolve(searchResult);
    })
  }

  async onScannerSearchArticle() {
    const searchRef = this.bonDocumentForm.get('SEARCH_REF')?.value;

    if(searchRef == undefined || searchRef == "") {
      return;
    }

    await this.searchArticles(searchRef);

    if(this.searchRefArticlesList === undefined || this.searchRefArticlesList === null) {
      return;
    }

    if(this.searchRefArticlesList.length > 0 && this.fullKeywordMatch) {
      if(this.isKeywordSerial) {
        const serialNo = this.bonDocumentForm.get("SEARCH_REF")?.value;

        const firstArticle = this.searchRefArticlesList[0];

        const blockedOutSerials = firstArticle.blocked_out_serials || "";
        const formattedBlockedSerials = this.commonService.formatBlockedSerialNumbers(blockedOutSerials);
        const blockedSerialExists = formattedBlockedSerials.find((row) => row.serial_no === serialNo.toUpperCase());

        // Check whether the entered S/N is already blocked or not
        if (blockedSerialExists !== undefined && blockedSerialExists !== null) {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KSN_ALREADY_BLOCKED',
            messageTranslateParam: { DOCUMENTS: `${blockedSerialExists.document_no} (${blockedSerialExists.serial_no})` }
          });

          return;
        }

        this.bonTransferDeStockItemForm.get('SELECTED_SERIALS_LIST')?.setValue(serialNo.toUpperCase());
        this.quantiteInputEl?.nativeElement.dispatchEvent(new Event('change'));

        this.searchSelected(firstArticle, true, true);
      } else {
        const searchStockLocation = this.bonDocumentForm.get("STOCK_LOCATION_SOURCE")?.value;
        if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
          this.searchRefArticlesList = undefined;

          this.stockLocationInputEl?.focus();
          this.stockLocationInputEl?.open();
          return;
        }

        const firstArticle = this.searchRefArticlesList[0];
        this.searchSelected(firstArticle, true);
      }
    } else {
      const searchStockLocation = this.bonDocumentForm.get("STOCK_LOCATION_SOURCE")?.value;
      if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
        this.searchRefArticlesList = undefined;

        this.stockLocationInputEl?.focus();
        this.stockLocationInputEl?.open();
        return;
      }

      this.openSearchArticlePopup();
    }
  }

  async openSearchArticlePopup() {
    const searchKeyword = this.bonDocumentForm.get('SEARCH_REF')?.value;
    const stockLocation = this.bonDocumentForm.get('STOCK_LOCATION_SOURCE')?.value;

    const data = {
      currentArticle: this.newArticleSelection,
      stockLocation: stockLocation,
      searchKeyword: searchKeyword
    };

    const dialog = await this.dialog.open(SearchArticleComponent, {
      data,
      maxHeight: "90vh"
    });

    dialog.afterClosed().subscribe((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        this.newArticleSelection = res.data;

        this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);
        this.bonTransferDeStockItemForm.get("SERIALS_LIST")?.setValue(this.newArticleSelection?.serial_list);
        console.log(this.bonTransferDeStockItemForm.getRawValue());

        this.__fillNewLineItemForm('new');
        console.log('SEARCH RES: ', res);

        this.changeQuantity('creation');
      }
    });
  }

  searchSelected(event: any, directCall: boolean = false, directInsert: boolean = false) {
    if(directCall) {
      this.newArticleSelection = event;
    } else {
      this.newArticleSelection = event.option.value;
    }

    this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);

    this.bonTransferDeStockItemForm.get("SERIALS_LIST")?.setValue(this.newArticleSelection?.serial_list);
    console.log(this.bonTransferDeStockItemForm.getRawValue());
    this.__fillNewLineItemForm('new');

    if(directInsert) {
      this.addBonTransferDeStockItemFormSubmitter();
      return;
    }

    this.changeQuantity('creation');
  }

  __fillNewLineItemForm(type: 'new' | 'existing') {
    let unitPrix = 0;
    if(this.newArticleSelection?.public_prix !== undefined && this.newArticleSelection?.public_prix !== null) {
      unitPrix = this.newArticleSelection?.public_prix;
    }

    this.bonTransferDeStockItemForm.patchValue({
      ARTICLE_ID: this.newArticleSelection?.id,
      REFERENCE: this.newArticleSelection?.reference,
      LABEL: this.newArticleSelection?.label,
      AVAIL_STOCK: this.newArticleSelection?.avail_stock === null ? 0 : this.newArticleSelection?.avail_stock,
    });

    if(type === "new") {
      const itemSelectedSerials = this.bonTransferDeStockItemForm.get('SELECTED_SERIALS_LIST')?.value;
      const selectedListSerials = itemSelectedSerials ? (typeof itemSelectedSerials === 'string' ? itemSelectedSerials.split(",") : itemSelectedSerials):[];

      this.bonTransferDeStockItemForm.patchValue({
        QTE: selectedListSerials.length.toString(),
        PRIX_UNIT: unitPrix.toString(),
        PRIX_TOTAL: "0",
        SERIALS_LIST: this.bonTransferDeStockItemForm.get("SERIALS_LIST")?.value
      });
    }
  }

  changeQuantity(type: 'creation' | 'modification') {
    const itemSerials = this.bonTransferDeStockItemForm.get("SERIALS_LIST")?.value;
    const oldListSerials = this.commonService.formatSerialNumbers(itemSerials ? (typeof itemSerials === 'string'? itemSerials.split(","): itemSerials) :[]);

    const itemSelectedSerials = this.bonTransferDeStockItemForm.get('SELECTED_SERIALS_LIST')?.value;
    const selectedListSerials = itemSelectedSerials ? (typeof itemSelectedSerials === 'string' ? itemSelectedSerials.split(",") : itemSelectedSerials):[];

    const blockedInSerials = this.newArticleSelection?.blocked_in_serials;
    const blockedOutSerials = this.newArticleSelection?.blocked_out_serials;

    this.openUpSerialNumPopup({
      oldListSerials,
      selectedSerials: selectedListSerials,
      block_in_serials: blockedInSerials,
      block_out_serials: blockedOutSerials
    });
  }

  viewItemSerials(item: BonTransferDeStockItem) {
    const itemSelectedSerials = item.article_serials;
    const selectedListSerials = itemSelectedSerials ? itemSelectedSerials : [];

    // Sending Selected Serials and Old List Serials as same array to display the newly selected Items only.
    let oldListSerials = this.commonService.formatSerialNumbers([...selectedListSerials]);

    this.openUpSerialNumPopup({
      oldListSerials,
      selectedSerials: selectedListSerials,
      readonly: true
    });
  }

  openUpSerialNumPopup(data?: any) {
    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: {
          numeroType: 'transfer-de-stock-numero-series',
          ...data
        }
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        if(result.data && result.data !== '') {
          if(data?.readonly && data?.readonly === true) {
            return;
          }

          this.bonTransferDeStockItemForm.get('QTE')?.setValue(result.data.length.toString());
          this.bonTransferDeStockItemForm.get('SELECTED_SERIALS_LIST')?.setValue(result.data.join(','));
          this.unitPrixInputEl?.nativeElement.select();
          this.unitPrixInputEl?.nativeElement.focus();
          this.quantiteInputEl?.nativeElement.dispatchEvent(new Event('change'));
        }
      }
    });
  }

  confirmResetNewArticle(): Promise<any> {
    const message = `<h3>${this.translateService.instant('KCONFIRM_DISCARD_NEW_ITEM')}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async modifyLineItem(item: BonTransferDeStockItem) {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    const data = {
      item: item,
      document_no: this.bonDocumentNum,
      document: this.bonTransferDeStock,
    };

    this.openUpModifyItemPopup(data);
  }

  openUpModifyItemPopup(data?: {}) {
    const dialogRef = this.dialog.open(ModifyItemComponent,
      {
        maxWidth: '35vw',
        data: {
          numeroType:'transfer-de-stock-numero-series',
          ...data
        }
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        console.log(result.data);
        if(result.data && result.data !== '') {
          this.loadingService.show();
          const bonDocument = result.data;
          this._modifyTransferDeStock(this.bonDocumentNum, bonDocument);
        }
      }
    });
  }

  onBonDocumentAssocieChange(documentAss: string) {
    if(this.documentAssocie?.value !== documentAss) {
      this.documentAssocie?.setValue(documentAss);
    }
  }

  calculatePrixTotal(){
    const formData = this.bonTransferDeStockItemForm.getRawValue();

    let totalPrix = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      this.bonTransferDeStockItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      this.bonTransferDeStockItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    this.bonTransferDeStockItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
  }

  openHistorique(){
    const formData = this.bonDocumentForm.getRawValue();
    const data = {
      document_no: formData.NUMBON,
      document_type: APP_CONSTANTS.DOCUMENT_TYPE_TRANSFER_STOCK
    };
    this.dialog.open(HistoriqueComponent,
      {
        minWidth: '80%',
        maxHeight: '90vh',
        data: data || {}
      }
    );
  }


  confirmRemoveTransferDeStock(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeTransferDeStock() {
    if(this.bonDocumentNum == undefined){
      return
    }
    if((this.__isDocumentValidated && !this.rolePermission['3_27']) || (!this.__isDocumentValidated && !this.rolePermission['3_24'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveTransferDeStock(this.bonDocumentNum);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteTransferDeStock(this.bonDocumentNum);
    }
  }

  async _deleteTransferDeStock(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/transfer-de-stock/list']);
          return;
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.transferDeStockService.deleteTransferDeStock(document_no).subscribe(observer);
  }

}
