import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import {PageEvent} from '@angular/material/paginator';
import {TransferDeStockService} from '../../../_services/transfer-de-stock.service';
import { TranslateService } from '@ngx-translate/core';
import {lastValueFrom} from "rxjs";
import {CustomHttpResponse, TransferDeStockList} from "../../../_models";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';

@Component({
  selector: 'app-transfer-de-stock-list',
  templateUrl: './transfer-de-stock-list.component.html',
  styleUrls: ['./transfer-de-stock-list.component.scss']
})
export class TransferDeStockListComponent implements OnInit {

  searchForm: FormGroup = this.formBuilder.group({
    document_no: [''],
    start_date: [{ value: '', disabled: true }],
    end_date: [{ value: '', disabled: true }]
  });

  listTransferDeStock: TransferDeStockList[] | undefined;

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  displayedColumnsTransfer: string[] = ['DOCUMENT_NO', 'DOCUMENT_DATE', 'STOCK_SOURCE', 'STOCK_DESTINATION', 'LIGNE', 'STATUS', 'ACTIONS'];
  rolePermission:any;
  constructor(private formBuilder: FormBuilder,
              private transferDeStockService: TransferDeStockService,
              private translateService: TranslateService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.checkForListCache();
  }

  isDocumentValidated(item: TransferDeStockList): boolean {
    return item.status === APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_YES;
  }

  documentStatus(item: TransferDeStockList) {
    const statusList = APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_YES:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_NO:
        result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        document_no:formCache.document_no,
        start_date: formCache.start_date,
        end_date: formCache.end_date
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST);
      if(dataCache !== undefined) {

        this.listTransferDeStock = dataCache.data as TransferDeStockList[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  get startDate() {
    return this.searchForm.get('start_date')?.value;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST_FORM, JSON.stringify(formData));
    const payload = {
      document_no: formData.document_no,
      start_date: formData.start_date,
      end_date: formData.end_date
    };

    this.currentPage = 1;

    this._getTransferDeStockList(payload, this.currentPage, this.pageSize);
  }

  _getTransferDeStockList(payload: { document_no: string, start_date: string, end_date: string }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<TransferDeStockList[]>) => {
        if(res.status === "success") {
          this.listTransferDeStock = res.data;

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.transferDeStockService.searchTransferDeStock(payload, page, pageSize).subscribe(observer);
  }

  __updateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST);
    if(listCache !== undefined) {
      let listTransferDeStockList: any[] = listCache.data;
      listTransferDeStockList = (listTransferDeStockList as TransferDeStockList[]).filter(r => r.document_no != id);

      listCache.data = listTransferDeStockList;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listTransferDeStockList.length === 0) {
        this.searchFormSubmitter();
      }
    }
  }

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      document_no: formData.document_no,
      start_date: formData.start_date,
      end_date: formData.end_date
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._getTransferDeStockList(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveTransferDeStock(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeTransferDeStock(document_no: string, item: TransferDeStockList) {
    if((this.isDocumentValidated(item) && !this.rolePermission['3_27']) || (!this.isDocumentValidated(item) && !this.rolePermission['3_24'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveTransferDeStock(document_no);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteTransferDeStock(document_no);
    }
  }

  async _deleteTransferDeStock(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(document_no);

          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.transferDeStockService.deleteTransferDeStock(document_no).subscribe(observer);
  }

}
