import { Component, OnInit } from '@angular/core';
import {FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { TreeviewConfig, TreeviewItem } from 'ngx-treeview';
import {Role, UserPermission} from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { UserManagementService } from 'src/app/_services/user-management.service';
import { confirmPasswordValidation } from '../../../_helpers/confirm-password-validation';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {checkPhoneValidation} from "../../../_helpers/telephone-validation";
@Component({
  selector: 'app-add-user',
  templateUrl: './add-user.component.html',
  styleUrls: ['./add-user.component.scss']
})
export class AddUserComponent implements OnInit {

  createUserForm:FormGroup = this.formBuilder.group({
    user_name: ['', [Validators.required,Validators.minLength(APP_CONSTANTS.USER_NAME_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.USER_NAME_MAX_LENGTH)]],
    user_password: ['', [Validators.required,Validators.minLength(APP_CONSTANTS.PASSWORD_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.PASSWORD_MAX_LENGTH)]],
    confirmation: ['', [Validators.required,Validators.minLength(APP_CONSTANTS.PASSWORD_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.PASSWORD_MAX_LENGTH)]],
    user_email: ['', [Validators.required,Validators.pattern(APP_CONSTANTS.EMAIL_PATTERN)]],
    user_telephone: ['', [Validators.minLength(APP_CONSTANTS.USER_TELEPHONE_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.USER_TELEPHONE_MAX_LENGTH)]],
    user_alt_telephone: [''],
    tele_country: ['', Validators.required],
    user_role_id: ['', [Validators.required]],
    user_status: ['', [Validators.required]]
  }, {
    validator: [
      confirmPasswordValidation('user_password', 'confirmation'),
      checkPhoneValidation('user_telephone', 'tele_country')
    ]
  });
  submitted: boolean = false;
  loading:boolean = false;
  hidePassword = true;
  hideConformation = true;
  userRoleList: Role[]=[];
  userPermissionlist:any=[];
  allComplete: boolean[] = [];
  collectPermissionData:any=[];
  statusList = APP_CONSTANTS.USER_INFO_STATUS;
  userNameMask = APP_CONSTANTS.USER_NAME_MASK;
  secondaryTelephoneMask = APP_CONSTANTS.SECONDARY_TELEPHONE; // Secondary Telephone Mask Pattern
  rolesDisabled: boolean = false;

  dropdownEnabled = true;
  items: TreeviewItem[] =[];
  values: number[] | undefined;
  config = TreeviewConfig.create({
      hasAllCheckBox: false,
      hasFilter: false,
      hasCollapseExpand: false,
      decoupleChildFromParent: false
  });
  checkedValueList:any =[];

  preferredCountries = APP_CONSTANTS.PREFERRED_TEL_COUNTRIES;

  constructor(private formBuilder: FormBuilder,
    private router: Router,
    private loadingService: NgxSpinnerService,
    private userManagementService: UserManagementService,
    private dialogService: DialogService,
    private translateService:TranslateService,
    private dataSharingService: DataSharingService
    ) {}

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.getRoleList();
    await this.loadingService.hide();
  }

  getRoleList() {
    const observer = {
      next: (res: any) => {
        if(res.status === "success") {
          this.userRoleList = res.data;
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userManagementService.getUserRoleList().subscribe(observer);
  }

  _checkAll(permissionList: UserPermission[]) {
    if(permissionList === undefined || permissionList === null) {
      return [];
    }

    return permissionList.map((item) => {
      item.status = true;

      if(item.child !== undefined && item.child !== null) {
        item.child = this._checkAll(item.child);
      }

      return item;
    })
  }

  getUserPermission(event:any){
    const roleData = this.userRoleList.find((r) => r.role_id == event.value);
    if(roleData && roleData.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE) {
      this.rolesDisabled = true;
    } else {
      this.rolesDisabled = false;
    }

    console.log(event.value);
    const observer = {
      next: (res: any) => {
        if(res.status === "success") {
          this.userPermissionlist = res.data.role_permission;
          this._getRolePermission(this.userPermissionlist)
          if(roleData && roleData.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE) {
            this._checkAll(this.userPermissionlist);
          }
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userManagementService.getUserPermission(event.value).subscribe( observer);
  }

  get userRoleStatus() {
    return this.createUserForm.get("user_role_id");
  }


  get fr() {
    return this.createUserForm.controls;
  }

  _getRolePermission(data:[]){
    const _formatTreeData = (rowData: { label: any; id: any; value: any; status: any; child: any[] | null | undefined; }) => {
      const formatted:any = {
        text: this.translateService.instant(rowData.label),
        value: `${rowData.id}_${rowData.value}`,
        checked: rowData.status,
        collapsed: true
      };

      let childrens;
      if(rowData.child !== undefined && rowData.child !== null && rowData.child.length > 0) {
        childrens = rowData.child.map(child_row => {
          return _formatTreeData(child_row);
        });
      }

      if(childrens !== undefined && childrens !== null) {
        formatted['children'] = childrens;
      }

      return formatted;
    };

    const processTreeData = (data: any[] | null | undefined) => {
      if(data === undefined || data === null || data.length === 0) return {};

      const processedData = data.map((row) => {
        return _formatTreeData(row);
      });

      return processedData;
    }
    const processedTreeData:any = processTreeData(data);
      console.log(processedTreeData);
      this.items = [new TreeviewItem({
       text: this.translateService.instant('KUSER_ROLE_PERMISSIONS'),
       value:'',
       children:processedTreeData
      })];
  }

  onCountryChange(event: any) {
    if(event != undefined && event != null) {
      const countryFlag = event.flagClass;

      this.createUserForm.get('tele_country')?.setValue(countryFlag);
    }
  }

  onSelectedChange(event: any){
    if(event === undefined || event === null || event.length === 0) {
      this.checkedValueList = [];
      return;
    }
    this.checkedValueList = [];
    let children = this.items[0].children;
    while(children !== undefined && children.length > 0) {
      const currentChilds = [...children];
      children = [];
      currentChilds.forEach((row) => {
        if(row.checked || row.indeterminate){
          this.checkedValueList.push(row.value);
        }
        if(row.children !== undefined && row.children !== null && row.children.length > 0) {
          children = [...children, ...row.children];
        }
      })
    }
  }

  createUserFormSubmiter(){
    this.submitted = true;
    if(this.createUserForm.invalid){
      return;
    }
    if(this.checkedValueList !== undefined && this.checkedValueList !== null && this.checkedValueList.length !== 0) {
      this.checkedValueList.map((key: any) =>{
        let arr = key.split('_');
        const [id, value] = arr;
        const obj:any = {
          id: +id,
          value: +value,
          status: true
        };
        this.collectPermissionData.push(obj)
      });
      const formData=this.createUserForm.getRawValue();
      const payload = {
        ...formData,
        user_role_permission: this.collectPermissionData
      };

      delete payload.user_intra_phone;

      this.createUserAndPermission(payload);
    }else{
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KSELECT_ROLE_PERMISSIONS'
      })
    }
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST_FORM);
  }


  async createUserAndPermission(payload:any){
    const observer = {
      next: async (res: any) => {
        if(res.status === "success") {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/options/user/list']);
        }else{
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          })
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userManagementService.createUserAPI(payload).subscribe( observer);
  }
}
