import { Component, OnInit } from '@angular/core';
import {FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { NgxSpinnerService } from 'ngx-spinner';
import { UserManagementService } from 'src/app/_services/user-management.service';
import { confirmPasswordValidation } from '../../../_helpers/confirm-password-validation';
import {firstValueFrom} from "rxjs";
import {CustomHttpResponse, Role, UserManagement} from "../../../_models";
import {DialogService} from "../../../_services/dailog.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import { TranslateService } from '@ngx-translate/core';
import { UserPermission } from 'src/app/_models/user-role';
import {TokenStorageService} from "../../../_services/token-storage.service";
import { TreeviewConfig, TreeviewItem } from 'ngx-treeview';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import {checkPhoneValidation} from "../../../_helpers/telephone-validation";

@Component({
  selector: 'app-edit-user',
  templateUrl: './edit-user.component.html',
  styleUrls: ['./edit-user.component.scss']
})
export class EditUserComponent implements OnInit {

  modifyUserForm: FormGroup = this.formBuilder.group({
    user_name: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.USER_NAME_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.USER_NAME_MAX_LENGTH)]],
    user_password: ['', [Validators.minLength(APP_CONSTANTS.PASSWORD_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.PASSWORD_MAX_LENGTH)]],
    confirmation: ['', [Validators.minLength(APP_CONSTANTS.PASSWORD_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.PASSWORD_MAX_LENGTH)]],
    user_email: ['', [Validators.required,Validators.pattern(APP_CONSTANTS.EMAIL_PATTERN)]],
    // telephone_type: [APP_CONSTANTS.TELEPHONE_TYPE_PHONE, [Validators.required]],
    user_telephone: ['', [Validators.minLength(APP_CONSTANTS.USER_TELEPHONE_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.USER_TELEPHONE_MAX_LENGTH)]],
    tele_country: ['', Validators.required],
    user_alt_telephone: [''],
    user_role_id: ['', [Validators.required]],
    user_status: ['', [Validators.required]]
  }, {
    validator: [
      confirmPasswordValidation('user_password', 'confirmation'),
      checkPhoneValidation('user_telephone', 'tele_country')
    ],
  });

  submitted: boolean = false;
  loading:boolean = false;
  hidePassword = true;
  hideConformation = true;
  userRoleList: Role[]=[];
  userPermissionlist: any=[];
  allComplete: boolean[] = [];
  collectPermissionData:any=[];
  user_id: number | undefined;
  pageTitle: string = 'KMODIFY_USER';
  readOnly: boolean = false;
  UserDetail: UserManagement | undefined;
  statusList = APP_CONSTANTS.USER_INFO_STATUS;
  disabled: boolean=false;

  dropdownEnabled = true;
  items: TreeviewItem[] =[];
  values: number[] | undefined;
  config = TreeviewConfig.create({
      hasAllCheckBox: false,
      hasFilter: false,
      hasCollapseExpand: false,
      decoupleChildFromParent: false
  });
  checkedValueList:any =[];

  preferredCountries = APP_CONSTANTS.PREFERRED_TEL_COUNTRIES;

  secondaryTelephoneMask = APP_CONSTANTS.SECONDARY_TELEPHONE; // Secondary Telephone Mask Pattern

  constructor(private formBuilder: FormBuilder,
    private router: Router,
    private loadingService: NgxSpinnerService,
    private userManagementService: UserManagementService,
    private route: ActivatedRoute,
    private dialogService: DialogService,
    private translateService:TranslateService,
    private tokenStorageService: TokenStorageService,
    private dataSharingService: DataSharingService
    ) {}

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.user_id = +params['user_id'];
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];
        if(this.readOnly) {
          this.disabled=true;
          this._changeToDetailMode();
        }
      }
    })
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.getRoleList();
    await this._getUserInfoDetail(this.user_id);
    this._prefillFormData();

    if(this._loggedInUser.user_id == this.user_id) {
      this.modifyUserForm.get('user_role_id')?.disable({ onlySelf: true });
      this.modifyUserForm.get('user_status')?.disable({ onlySelf: true });
    }

    if(this.UserDetail?.superadmin_status === APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE) {
      this.disabled = true;
    }

    await this.loadingService.hide();
  }

  get _loggedInUser() {
    return this.tokenStorageService.getUser();
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIGNE_LIST);
    if(listCache !== undefined) {
      let listLigneInfo = listCache.data as UserManagement[];
      listLigneInfo = listLigneInfo.map((row) => {
        if(row.user_id == this.user_id) {
          row.user_id = this.user_id;
          row.user_name = payload.user_name;
          row.user_password = payload.user_password;
          row.user_email = payload.user_email;
          row.user_telephone = payload.user_telephone;
          row.user_role_id = payload.user_role_id;
          row.user_status = payload.user_status;
        }
        return row;
      });

      listCache.data = listLigneInfo;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST, JSON.stringify(listCache));
    }
  }

  _getUserInfoDetail(user_id: number | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(user_id === undefined || user_id === null) {
        reject();
        return;
      }
      const observer = {
        next: (res: CustomHttpResponse<UserManagement>) => {
          if(res.status === "success") {
            this.UserDetail = res.data;
            this.userPermissionlist=this.UserDetail.user_role_permission;
            this._getRolePermission(this.userPermissionlist)
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/options/user/list']);
              }
            );
          }
          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          reject(err);
        }
      };
      this.userManagementService.getUserDetail(user_id).subscribe(observer);
    })
  }

  _changeToDetailMode() {
    this.pageTitle = 'KDETAIL_USER';
    this.modifyUserForm.get('user_name')?.disable({ onlySelf: true });
    this.modifyUserForm.get('user_password')?.disable({ onlySelf: true });
    this.modifyUserForm.get('Confirmation')?.disable({ onlySelf: true });
    this.modifyUserForm.get('user_email')?.disable({ onlySelf: true });
    this.modifyUserForm.get('user_telephone')?.disable({ onlySelf: true });
    this.modifyUserForm.get('user_role_id')?.disable({ onlySelf: true });
    this.modifyUserForm.get('user_status')?.disable({ onlySelf: true });
    this.modifyUserForm.get('user_alt_telephone')?.disable({ onlySelf: true });
  }

  _prefillFormData() {
    this.modifyUserForm.patchValue({
      user_name: this.UserDetail?.user_name,
      user_email: this.UserDetail?.user_email,
      telephone_type: this.UserDetail?.telephone_type,
      user_telephone: this.UserDetail?.user_telephone,
      user_alt_telephone: this.UserDetail?.user_alt_telephone,
      user_role_id: this.UserDetail?.user_role_id,
      user_status: this.UserDetail?.user_status,
    });
  }

  getRoleList() {
    const observer = {
      next: (res: any) => {
        if(res.status === "success") {
          this.userRoleList = res.data;
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userManagementService.getUserRoleList().subscribe(observer);
  }

  _checkAll(permissionList: UserPermission[]) {
    return permissionList.map((item) => {
      item.status = true;

      if(item.child) {
        item.child = this._checkAll(item.child);
      }

      return item;
    })
  }

  getUserRoleId(event:any) {
    const roleData = this.userRoleList.find((r) => r.role_id == event.value);
    if(roleData && roleData.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE) {
      this.disabled = true;
    } else {
      this.disabled = false;
    }

    this.getUserPermission(event.value);
  }

  getUserPermission(user_role_id:number) {
    const roleData = this.userRoleList.find((r) => r.role_id == user_role_id);
    const observer = {
      next: (res: any) => {
        if(res.status === "success") {
          this.userPermissionlist = res.data.role_permission;
          this._getRolePermission(this.userPermissionlist)
          if(roleData && roleData.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE) {
            this._checkAll(this.userPermissionlist);
          }
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userManagementService.getUserPermission(user_role_id).subscribe( observer);
  }

  onCountryChange(event: any) {
    if(event != undefined && event != null) {
      const countryFlag = event.flagClass;

      this.modifyUserForm.get('tele_country')?.setValue(countryFlag);
    }
  }

  onSelectedChange(event: any){
    if(event === undefined || event === null || event.length === 0) {
      this.checkedValueList = [];
      return;
    }
    this.checkedValueList = [];
    let children = this.items[0].children;
    while(children !== undefined && children.length > 0) {
      const currentChilds = [...children];
      children = [];
      currentChilds.forEach((row) => {
        if(row.checked || row.indeterminate){
          this.checkedValueList.push(row.value);
        }
        if(row.children !== undefined && row.children !== null && row.children.length > 0) {
          children = [...children, ...row.children];
        }
      })
    }
  }

  _getRolePermission(data:[]){
    const _formatTreeData = (rowData: { label: any; id: any; value: any; status: any; child: any[] | null | undefined; }) => {
      const formatted:any = {
        text: this.translateService.instant(rowData.label),
        value: `${rowData.id}_${rowData.value}`,
        checked: rowData.status,
        collapsed: true,
        disabled: this.readOnly? true : false
      };

      let childrens;
      if(rowData.child !== undefined && rowData.child !== null && rowData.child.length > 0) {
        childrens = rowData.child.map(child_row => {
          return _formatTreeData(child_row);
        });
      }

      if(childrens !== undefined && childrens !== null) {
        formatted['children'] = childrens;
      }

      return formatted;
    };

    const processTreeData = (data: any[] | null | undefined) => {
      if(data === undefined || data === null || data.length === 0) return {};

      const processedData = data.map((row) => {
        return _formatTreeData(row);
      });

      return processedData;
    }
    const processedTreeData:any = processTreeData(data);
      console.log(processedTreeData);
      this.items = [new TreeviewItem({
       text: this.translateService.instant('KUSER_ROLE_PERMISSIONS'),
       value:'',
       children:processedTreeData
      })];
  }

  get userRoleStatus() {
    return this.modifyUserForm.get("user_role_id");
  }


  get fr() {
    return this.modifyUserForm.controls;
  }

  modifyUserFormSubmiter(){
    this.submitted = true;
    if(this.modifyUserForm.invalid){
      return;
    }
    if(this.checkedValueList !== undefined && this.checkedValueList !== null && this.checkedValueList.length !== 0) {
      this.checkedValueList.map((key: any) =>{
        let arr = key.split('_');
        const [id, value] = arr;
        const obj:any = {
          id: +id,
          value: +value,
          status: true
        };
        this.collectPermissionData.push(obj)
      });
      const formData=this.modifyUserForm.getRawValue();
      const payload = {
        ...formData,
        user_role_permission: this.collectPermissionData
      };
      this.createUserAndPermission(payload);

    }else{
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KSELECT_ROLE_PERMISSIONS'
      })
    }
  }

  async createUserAndPermission(payload:UserManagement){
    const observer = {
      next: async (res: any) => {
        if(res.status === "success") {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/options/user/list']);
        }else{
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          })
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userManagementService.modifyUser(payload,this.UserDetail?.user_id).subscribe( observer);
  }
}
