import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import {PageEvent} from '@angular/material/paginator';
import {UserManagementService} from "../../../_services/user-management.service";
import {CustomHttpResponse, UserManagement} from "../../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {lastValueFrom} from "rxjs";
import {DialogService} from "../../../_services/dailog.service";
import { TranslateService } from '@ngx-translate/core';
import { Router } from '@angular/router';
import {TokenStorageService} from "../../../_services/token-storage.service";
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';

@Component({
  selector: 'app-user-list',
  templateUrl: './user-list.component.html',
  styleUrls: ['./user-list.component.scss']
})
export class UserListComponent implements OnInit {

totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  userList: UserManagement[] | undefined;
  displayedColumnsUser: string[] = ['USER_NAME', 'USER_EMAIL', 'USER_TEL', 'USER_ROLE', 'KUSER_STATUS', 'ACTIONS'];
  searchForm: FormGroup = this.formBuilder.group({
    keyword: [''],
  });
  rolePermission:any;

  constructor(private formBuilder: FormBuilder,
              private userManagementService: UserManagementService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService:TranslateService,
              private tokenStorageService: TokenStorageService,
              private router: Router,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.checkForListCache();
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();
    this.searchFormSubmitter();
    await this.loadingService.hide();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        keyword: formCache.keyword
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST);
      if(dataCache !== undefined) {

        this.userList = dataCache.data as UserManagement[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }


  get fr() {
    return this.searchForm.controls;
  }

  get _loggedInUser() {
    return this.tokenStorageService.getUser();
  }

  documentStatus(item: UserManagement) {
    const statusList = APP_CONSTANTS.USER_INFO_STATUS;

    const statusData = statusList.find((r) => r.value == item.user_status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.user_status) {
      case APP_CONSTANTS.USER_STATUS_ACTIVE:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.USER_STATUS_IN_ACTIVE:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  _modifyAccess(user: UserManagement): boolean {
    return (user.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_IN_ACTIVE || (user.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE && this._loggedInUser.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE));
  }

  _deleteAccess(user: UserManagement): boolean {
    return (this._loggedInUser.user_id != user.user_id) && (user.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_IN_ACTIVE || (user.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE && this._loggedInUser.superadmin_status == APP_CONSTANTS.USER_SUPER_ADMIN_STATUS_ACTIVE));
  }

  _getUserDeleteStatus(element: UserManagement): boolean {
    return element.user_delete_status == APP_CONSTANTS.USER_DELETED_STATUS;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST_FORM, JSON.stringify(formData));
    const payload = {
      keyword: formData.keyword,
      date_created: ""
    };

    this.currentPage = 1;

    this._searchUserInfo(payload, this.currentPage, this.pageSize);
  }

  _searchUserInfo(data: { keyword: string; date_created: string;}, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<UserManagement[]>) => {
        if(res.status === "success") {
          this.userList = res.data;

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST);
        }

        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.userManagementService.searchUserInfo(data, page, pageSize).subscribe( observer);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST);
    if(listCache !== undefined) {
      let listUserManagement: any[] = listCache.data;
      listUserManagement = (listUserManagement as UserManagement[]).filter(r => r.user_id != id);

      listCache.data = listUserManagement;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listUserManagement.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      keyword: formData.keyword,
      date_created: ""
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._searchUserInfo(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveUser(user_name: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: user_name })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeUser(user_id: number, user_name:string) {
    if(this._loggedInUser.user_id == user_id) {
      return;
    }
    if(!this.rolePermission['9_124']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveUser(user_name);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteUserInfo(user_id);
    }
  }

  async restoreUser(user_id: number) {
    if(this._loggedInUser.user_id == user_id) {
      return;
    }
    if(!this.rolePermission['9_75']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    this.loadingService.show();
    this._restoreUserInfo(user_id);
  }

  async _deleteUserInfo(user_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(user_id);
          this.searchFormSubmitter();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userManagementService.deleteUser(user_id).subscribe(observer);
  }

  async _restoreUserInfo(user_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(user_id);
          this.searchFormSubmitter();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userManagementService.restoreUser(user_id).subscribe(observer);
  }

}
