import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse } from 'src/app/_models';
import { UserPermission, UserRole } from 'src/app/_models/user-role';
import { DialogService } from 'src/app/_services/dailog.service';
import { UserRolesService } from 'src/app/_services/user-roles.service';
import { TreeviewItem, TreeviewConfig} from 'ngx-treeview';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-add-role',
  templateUrl: './add-role.component.html',
  styleUrls: ['./add-role.component.scss']
})
export class AddRoleComponent implements OnInit {

  createRoleForm:FormGroup = this.formBuilder.group({
    role_name: ['', [Validators.required]],
    role_status: ['', [Validators.required]]
  });
  submitted: boolean = false;
  loading:boolean = false;
  userPermissionlist: UserPermission[]=[];
  allComplete: boolean[] = [];
  collectPermissionData:any=[];
  checkedValueList:any =[];
  statusList = APP_CONSTANTS.USER_ROLE_INFO_STATUS;

  dropdownEnabled = true;
  items: TreeviewItem[] =[];
  values: number[] | undefined;
  config = TreeviewConfig.create({
      hasAllCheckBox: false,
      hasFilter: false,
      hasCollapseExpand: false,
      decoupleChildFromParent: false
  });

  constructor(private formBuilder: FormBuilder,
    private router: Router,
    private loadingService: NgxSpinnerService,
    private dialogService: DialogService,
    private userRolesService: UserRolesService,
    private translateService: TranslateService,
    private dataSharingService: DataSharingService
    ) {}

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.getUserDefaultPermission();
    this._getCheckboxData();
    await this.loadingService.hide();
  }

  getUserDefaultPermission(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.userRolesService.getUserPermission().subscribe({
        next: (res: CustomHttpResponse<UserPermission[]>) => {
          if(res.status === 'success') {
            this.userPermissionlist = res.data;
          }else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  get fr() {
    return this.createRoleForm.controls;
  }

  _getCheckboxData(){
    const _formatTreeData = (rowData: { label: any; id: any; value: any; status: any; child: any[] | null | undefined; }) => {
      const formatted:any = {
        text: this.translateService.instant(rowData.label),
        value: `${rowData.id}_${rowData.value}`,
        checked: rowData.status,
        collapsed: true
      };

      let childrens;
      if(rowData.child !== undefined && rowData.child !== null && rowData.child.length > 0) {
        childrens = rowData.child.map(child_row => {
          return _formatTreeData(child_row);
        });
      }

      if(childrens !== undefined && childrens !== null) {
        formatted['children'] = childrens;
      }

      return formatted;
    };

    const processTreeData = (data: any[] | null | undefined) => {
      if(data === undefined || data === null || data.length === 0) return {};

      const processedData = data.map((row) => {
        return _formatTreeData(row);
      });

      return processedData;
    }
    const processedTreeData:any = processTreeData(this.userPermissionlist);
    this.items = [new TreeviewItem({
     text: this.translateService.instant('KUSER_ROLE_PERMISSIONS'),
     value:'',
     children:processedTreeData
    })];
  }

  onSelectedChange(event: any){
    if(event === undefined || event === null || event.length === 0) {
      this.checkedValueList = [];
      return;
    }
    this.checkedValueList = [];
    let children = this.items[0].children;
    while(children !== undefined && children.length > 0) {
      const currentChilds = [...children];
      children = [];
      currentChilds.forEach((row) => {
        if(row.checked || row.indeterminate){
          this.checkedValueList.push(row.value);
        }
        if(row.children !== undefined && row.children !== null && row.children.length > 0) {
          children = [...children, ...row.children];
        }
      })
    }
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ROLE_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ROLE_LIST_FORM);
  }

  checkRoleLabel(){
    const role_name = this.createRoleForm.get('role_name')?.value.trim();
    this.createRoleForm.get('role_name')?.setValue(role_name);
  }

  createRoleFormSubmit(){
    this.submitted = true;
    if(this.createRoleForm.invalid){
      return;
    }
    if(this.checkedValueList !== undefined && this.checkedValueList !== null && this.checkedValueList.length !== 0) {
      this.checkedValueList.map((key: any) =>{
        let arr = key.split('_');
        const [id, value] = arr;
        const obj:any = {
          id: +id,
          value: +value,
          status: true
        };
        this.collectPermissionData.push(obj)
      });
      const formData=this.createRoleForm.getRawValue();
      const payload = {
        ...formData,
        role_permission: this.collectPermissionData
      };
      this.createUserRolePermission(payload);
    }else{
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KSELECT_ROLE_PERMISSIONS'
      })
    }
  }

  async createUserRolePermission(data:UserRole){
    const observer = {
      next: async (res: any) => {
        if(res.status === "success") {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/options/roles/list']);
        }else{
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          })
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userRolesService.createUserRoleAPI(data).subscribe( observer);
  }
}
function data(data: any, arg1: (any: any) => void) {
  throw new Error('Function not implemented.');
}

