import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import {PageEvent} from '@angular/material/paginator';
import {CustomHttpResponse} from "../../../_models";
import { UserRole } from "../../../_models/user-role";
import {NgxSpinnerService} from "ngx-spinner";
import {lastValueFrom} from "rxjs";
import {DialogService} from "../../../_services/dailog.service";
import { TranslateService } from '@ngx-translate/core';
import { UserRolesService } from 'src/app/_services/user-roles.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';

@Component({
  selector: 'app-roles-list',
  templateUrl: './roles-list.component.html',
  styleUrls: ['./roles-list.component.scss']
})
export class RolesListComponent implements OnInit {
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  userRoleList: UserRole[] | undefined;
  displayedColumnsUserRole: string[] = ['ROLE_NAME','ROLE_STATUS', 'ACTIONS'];
  searchForm: FormGroup = this.formBuilder.group({
    keyword: [''],
  });
  rolePermission:any;

  constructor(private formBuilder: FormBuilder,
              private userRolesService: UserRolesService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService,
              private dataSharingService: DataSharingService) {}

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.checkForListCache();
    this.loadDependencies();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_ROLE_LIST_FORM);
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        keyword: formCache.keyword
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ROLE_LIST);
      if(dataCache !== undefined) {

        this.userRoleList = dataCache.data as UserRole[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.loadingService.hide();
  }

  get fr() {
    return this.searchForm.controls;
  }

  documentStatus(item: UserRole) {
    const statusList = APP_CONSTANTS.USER_ROLE_INFO_STATUS;

    const statusData = statusList.find((r) => r.value == item.role_status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.role_status) {
      case APP_CONSTANTS.USER_ROLE_STATUS_ACTIVE:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.USER_ROLE_STATUS_IN_ACTIVE:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  searchFormSubmitter() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ROLE_LIST_FORM, JSON.stringify(formData));
    const payload = {
      keyword: formData.keyword,
      date_created: ""
    };
    this.currentPage = 1;
    this._searchUserRoleInfo(payload, this.currentPage, this.pageSize);
  }

  _searchUserRoleInfo(data: { keyword: string; date_created: string;}, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<UserRole[]>) => {
        if(res.status === "success") {
          this.userRoleList = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ROLE_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ROLE_LIST);
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.userRolesService.searchUserRoleInfo(data, page, pageSize).subscribe( observer);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ROLE_LIST);
    if(listCache !== undefined) {
      let listUserRole: any[] = listCache.data;
      listUserRole = (listUserRole as UserRole[]).filter(r => r.roleid != id);

      listCache.data = listUserRole;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ROLE_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listUserRole.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      keyword: formData.keyword,
      date_created: ""
    };
    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._searchUserRoleInfo(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveUser(role_name: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: role_name })}</h3>`;
    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeUser(role_id: number, role_name:string) {
    if(!this.rolePermission['9_74']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveUser(role_name);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteUserInfo(role_id);
    }
  }

  async _deleteUserInfo(role_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(role_id);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userRolesService.deleteUserRole(role_id).subscribe(observer);
  }

}
